@@ -21,15 +21,16 @@ def create_parser():
 
     parser.add_argument(
         '-v', '--verbose',
-        help='Program logging level in 10-DEBUG, 20-INFO, 30-WARNING, '
-             '40-ERROR or 50-CRITICAL.',
-        type=int, default=20, choices=(1, 10, 20, 30, 40, 50))
+        help='Verbosity level in 10-DEBUG, 20-INFO, 30-WARNING, '
+             'or 40-ERROR.',
+        type=int, default=20, choices=(1, 10, 20, 30, 40))
 
     sub_parsers = parser.add_subparsers(dest='command', help='[run] run simulation routine; '
-                                                             '[plot] plot simulation results; '
+                                                             '[plot] plot results; '
                                                              '[doc] quick documentation; '
-                                                             '[prepare] run the symbolic-to-numeric preparation; '
-                                                             '[misc] miscellaneous functions.'
+                                                             '[misc] misc. functions; '
+                                                             '[prepare] prepare the numerical code; '
+                                                             '[selftest] run self test.'
                                         )
 
     run = sub_parsers.add_parser('run')
@@ -44,8 +45,12 @@ def create_parser():
     run.add_argument('-o', '--output-path', help='Output path prefix', type=str, default='')
     run.add_argument('-n', '--no-output', help='Force no output of any kind', action='store_true')
     run.add_argument('--ncpu', help='Number of parallel processes', type=int, default=os.cpu_count())
-    run.add_argument('--dime', help='Specify DiME streaming server address and port', type=str)
+    run.add_argument('--dime-address', help='DiME streaming server address and port', type=str)
+    run.add_argument('--dime-protocol', help='DiME streaming protocol', type=str)
     run.add_argument('--tf', help='End time of time-domain simulation', type=float)
+    run.add_argument('--qrt', help='Enable quasi-real-time stepping', action='store_true')
+    run.add_argument('--kqrt', help='Scaling factor for quasi-real-time; e.g., kqrt=2 means the wall-clock time '
+                                    'to complete a simulation is 2x the time of that simulation', type=float)
     run.add_argument('-c', '--convert', help='Convert to format.', type=str, default='', nargs='?')
     run.add_argument('-b', '--add-book', help='Add a template workbook for the specified model.', type=str)
     run.add_argument('--convert-all', help='Convert to format with all templates.', type=str, default='',
@@ -54,6 +59,12 @@ def create_parser():
                      action='store_true')
     run.add_argument('--profile', action='store_true', help='Enable Python cProfiler')
     run.add_argument('--shell', action='store_true', help='Start in IPython shell')
+    run.add_argument('--no-preamble', action='store_true', help='Hide preamble')
+    run.add_argument('--no-pbar', action='store_true', help='Hide progress bar for time-domain')
+    run.add_argument('--flat', action='store_true', help='Run no-disturbance (flat) simulation')
+    run.add_argument('--pool', action='store_true', help='Start multiprocess with Pool '
+                                                         'and return a list of Systems')
+    run.add_argument('--from-csv', help='Use data from a CSV file instead of from simulation')
 
     plot = sub_parsers.add_parser('plot')
     plot.add_argument('filename', nargs=1, default=[], help='simulation output file name, which should end '
@@ -87,8 +98,12 @@ def create_parser():
     plot.add_argument('-n', '--no-show', action='store_false', dest='show', help='do not show the plot window')
     plot.add_argument('--ytimes', type=str, help='scale the y-axis values by YTIMES')
     plot.add_argument('-c', '--tocsv', help='convert npy output to csv', action='store_true')
+    plot.add_argument('--hline1', help='dashed horizontal line 1', type=float)
+    plot.add_argument('--hline2', help='dashed horizontal line 2', type=float)
+    plot.add_argument('--vline1', help='dashed vertical line 1', type=float)
+    plot.add_argument('--vline2', help='dashed vertical line 2', type=float)
 
-    doc = sub_parsers.add_parser('doc')  # NOQA
+    doc = sub_parsers.add_parser('doc')
     doc.add_argument('attribute', help='System attribute name to get documentation', nargs='?')
     doc.add_argument('--config', '-c', help='Config help')
     doc.add_argument('--list', '-l', help='List supported models and groups', action='store_true',
@@ -108,11 +123,17 @@ def create_parser():
                       action='store_true')
 
     prep = sub_parsers.add_parser('prepare')  # NOQA
-    prep.add_argument('-q', '--quick', action='store_true', help='quick processing by skipping pretty prints')
-    prep.add_argument('-i', '--incremental', action='store_true', help='Only incrementally generate for updated '
-                                                                       'models')
+    prep_mode = prep.add_mutually_exclusive_group()
+    prep_mode.add_argument('-q', '--quick', action='store_true',
+                           help='quick codegen by skipping pretty prints')
+    prep_mode.add_argument('-f', '--full', action='store_true', help='full codegen')
+    prep_mode.add_argument('-i', '--incremental', action='store_true',
+                           help='rapid incrementally generate for updated models')
+
 
     selftest = sub_parsers.add_parser('selftest')  # NOQA
+    selftest.add_argument('-q', '--quick', action='store_true',
+                          help='quick selftest by skipping codegen')
 
     return parser
 
@@ -151,7 +172,11 @@ def main():
 
     module = importlib.import_module('andes.main')
 
-    if args.command not in ('plot', 'doc'):
+    if args.command in ('plot', 'doc'):
+        pass
+    elif args.command == 'run' and args.no_preamble is True:
+        pass
+    else:
         preamble()
 
     if args.command is None:
