@@ -1,14 +1,26 @@
+#  [ANDES] (C)2015-2020 Hantao Cui
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 3 of the License, or
+#  (at your option) any later version.
+#
+#  File name: block.py
+#  Last modified: 8/16/20, 7:28 PM
+
 from typing import Optional, Iterable, Union, List, Tuple
 
 from andes.core.var import Algeb, State
-from andes.core.discrete import AntiWindup, LessThan, Selector, HardLimiter
+from andes.core.discrete import AntiWindup, LessThan, Selector, HardLimiter, AntiWindupRate, RateLimiter
+from andes.core.discrete import DeadBand
+from andes.core.service import EventFlag
 from andes.core.common import JacTriplet
 from andes.core.common import ModelFlags, dummify
 from collections import OrderedDict
 import numpy as np
 
 
-class Block(object):
+class Block:
     r"""
     Base class for control blocks.
 
@@ -34,6 +46,9 @@ class Block(object):
         Block LaTeX name
     info : str, optional
         Block description.
+    namespace : str, local or parent
+        Namespace of the exported elements. If 'local', the block name will be prepended by the parent.
+        If 'parent', the original element name will be used when exporting.
 
     Warnings
     --------
@@ -86,10 +101,17 @@ class Block(object):
 
     """
 
-    def __init__(self, name: Optional[str] = None, tex_name: Optional[str] = None, info: Optional[str] = None):
+    def __init__(self, name: Optional[str] = None, tex_name: Optional[str] = None, info: Optional[str] = None,
+                 namespace: str = 'local'):
         self.name = name
         self.tex_name = tex_name if tex_name else name
         self.info = info
+
+        if namespace not in ('local', 'parent'):
+            raise ValueError(f'Argument namespace must be `local` or `parent`, got {namespace}')
+
+        self.namespace = namespace
+
         self.owner = None
         self.vars = OrderedDict()
         self.triplets = JacTriplet()
@@ -99,19 +121,26 @@ class Block(object):
         # handle sub-blocks by prepending self.name
         if isinstance(value, Block):
             if self.name is None:
-                raise ValueError(f"Must specify `name` for {self.class_name} any instance.")
+                raise ValueError(f"Must specify `name` for {self.class_name} instance.")
             if not value.owner:
                 value.__dict__['owner'] = self
 
-            if not value.name:
-                value.__dict__['name'] = self.name + '_' + key
+            if value.namespace == 'local':
+                prepend = self.name + '_'
+                tex_prepend = self.tex_name + r'\ '
             else:
-                value.__dict__['name'] = self.name + '_' + value.name
+                prepend = ''
+                tex_prepend = ''
+
+            if not value.name:
+                value.__dict__['name'] = prepend + key
+            else:
+                value.__dict__['name'] = prepend + value.name
 
             if not value.tex_name:
-                value.__dict__['tex_name'] = self.name + r'\ ' + key
+                value.__dict__['tex_name'] = tex_prepend + key
             else:
-                value.__dict__['tex_name'] = self.name + r'\ ' + value.tex_name
+                value.__dict__['tex_name'] = f'{tex_prepend}{{{value.tex_name}}}'
 
         self.__dict__[key] = value
 
@@ -240,14 +269,16 @@ class Block(object):
 
 class PIController(Block):
     """
-    Proportional Integral Controller with the reference from an external variable
+    Proportional Integral Controller.
+
+    The controller takes an error signal as the input.
+    It takes an optional `ref` signal, which will be subtracted from the input.
+
 
     Parameters
     ----------
     u : BaseVar
         The input variable instance
-    ref : Union[BaseVar, BaseParam]
-        The reference instance
     kp : BaseParam
         The proportional gain parameter instance
     ki : [type]
@@ -255,18 +286,19 @@ class PIController(Block):
 
     """
 
-    def __init__(self, u, ref, kp, ki, name=None, info=None):
-        super(PIController, self).__init__(name=name, info=info)
+    def __init__(self, u, kp, ki, ref=0.0, x0=0.0, name=None, tex_name=None, info=None):
+        Block.__init__(self, name=name, tex_name=tex_name, info=info)
 
-        self.u = u
-        self.ref = ref
-        self.kp = kp
-        self.ki = ki
+        self.u = dummify(u)
+        self.kp = dummify(kp)
+        self.ki = dummify(ki)
+        self.ref = dummify(ref)
+        self.x0 = dummify(x0)
 
-        self.xi = State(info="Integration value of PI controller")
-        self.y = Algeb(info="Output value")
+        self.xi = State(info="Integrator output", tex_name='xi')
+        self.y = Algeb(info="PI output", tex_name='y')
 
-        self.vars = {'xi': self.xi, 'y': self.y}
+        self.vars = OrderedDict([('xi', self.xi), ('y', self.y)])
 
     def define(self):
         r"""
@@ -279,44 +311,229 @@ class PIController(Block):
         Equations implemented are
 
         .. math ::
-            \dot{x_i} &= k_i * (ref - var) \\
-            y &= x_i + k_i * (ref - var)
+            \dot{x_i} &= k_i * (u - ref) \\
+            y &= x_i + k_p * (u - ref)
         """
 
-        self.xi.e_str = f'ki * ({self.ref.name} - {self.u.name})'
-        self.y.e_str = f'kp * ({self.ref.name} - {self.u.name}) + {self.name}_xi'
+        self.xi.v_str = f'{self.x0.name}'
+        self.xi.e_str = f'{self.ki.name} * ({self.u.name} - {self.ref.name})'
+
+        self.y.v_str = f'{self.kp.name} * ({self.u.name} - {self.ref.name}) + {self.x0.name}'
+        self.y.e_str = f'{self.kp.name} * ({self.u.name} - {self.ref.name}) + ' \
+                       f'{self.name}_xi - {self.name}_y'
+
+
+class PIAWHardLimit(PIController):
+    """
+    PI controller with anti-windup limiter on the integrator and
+    hard limit on the output.
+
+    Limits ``lower`` and ``upper`` are on the final output,
+    and ``aw_lower`` ``aw_upper`` are on the integrator.
+
+    """
+    def __init__(self, u, kp, ki, aw_lower, aw_upper, lower, upper, no_lower=False, no_upper=False,
+                 ref=0.0, x0=0.0, name=None, tex_name=None, info=None):
+
+        PIController.__init__(self, u=u, kp=kp, ki=ki, ref=ref, x0=x0,
+                              name=name, tex_name=tex_name, info=info,
+                              )
+
+        self.lower = dummify(lower)
+        self.upper = dummify(upper)
+        self.aw_lower = dummify(aw_lower)
+        self.aw_upper = dummify(aw_upper)
+
+        self.aw = AntiWindup(u=self.xi, lower=self.aw_lower, upper=self.aw_upper,
+                             no_lower=no_lower, no_upper=no_upper, tex_name='aw'
+                             )
+
+        self.yul = Algeb("PI unlimited output", tex_name='y^{ul}',
+                         discrete=self.aw,
+                         )
+
+        self.hl = HardLimiter(u=self.yul, lower=self.lower, upper=self.upper,
+                              no_lower=no_lower, no_upper=no_upper, tex_name='hl',
+                              )
+
+        # the sequence affect the initialization order
+        self.vars = OrderedDict([('xi', self.xi), ('aw', self.aw),
+                                 ('yul', self.yul), ('hl', self.hl),
+                                 ('y', self.y)])
+
+        self.y.discrete = self.hl
+
+    def define(self):
+
+        PIController.define(self)
+
+        self.yul.v_str = f'{self.kp.name} * ({self.u.name} - {self.ref.name}) + {self.x0.name}'
+
+        self.yul.e_str = f'{self.kp.name} * ({self.u.name} - {self.ref.name}) + ' \
+                         f'{self.name}_xi - {self.name}_yul'
+
+        # overwrite existing `y` equations
+        self.y.v_str = f'{self.name}_yul * {self.name}_hl_zi + ' \
+                       f'{self.lower.name} * {self.name}_hl_zl + ' \
+                       f'{self.upper.name} * {self.name}_hl_zu'
+
+        self.y.e_str = self.y.v_str + f' - {self.name}_y'
+
+
+class PITrackAW(Block):
+    """
+    PI with tracking anti-windup limiter
+    """
+    def __init__(self, u, kp, ki, ks, lower, upper, no_lower=False, no_upper=False,
+                 ref=0.0, x0=0.0, name=None, tex_name=None, info=None):
+        Block.__init__(self, name=name, tex_name=tex_name, info=info)
+
+        self.u = dummify(u)
+        self.kp = dummify(kp)
+        self.ki = dummify(ki)
+        self.ks = dummify(ks)
+        self.lower = dummify(lower)
+        self.upper = dummify(upper)
+        self.ref = dummify(ref)
+        self.x0 = dummify(x0)
+
+        self.xi = State(info="Integrator output", tex_name='xi')
+        self.ys = Algeb(info="PI summation before limit", tex_name='ys')
+        self.lim = HardLimiter(u=self.ys, lower=self.lower, upper=self.upper,
+                               no_lower=no_lower, no_upper=no_upper, tex_name='lim')
+        self.y = Algeb(info="PI output", discrete=self.lim, tex_name='y')
+        self.vars = OrderedDict([('xi', self.xi),
+                                 ('ys', self.ys),
+                                 ('lim', self.lim),
+                                 ('y', self.y)])
+
+    def define(self):
+        self.xi.v_str = f'{self.x0.name}'
+        self.ys.v_str = f'{self.kp.name} * ({self.u.name} - {self.ref.name}) + {self.x0.name}'
+        self.y.v_str = f'{self.name}_ys * {self.name}_lim_zi + ' \
+                       f'{self.lower.name} * {self.name}_lim_zl + ' \
+                       f'{self.upper.name} * {self.name}_lim_zu'
+
+        self.xi.e_str = f'{self.ki.name} * ({self.u.name} - {self.ref.name} -' \
+                        f' {self.ks.name} * ({self.name}_ys - {self.name}_y))'
+
+        self.ys.e_str = f'{self.kp.name} * ({self.u.name} - {self.ref.name}) + ' \
+                        f'{self.name}_xi - {self.name}_ys'
+
+        self.y.e_str = self.y.v_str + f' - {self.name}_y'
+
+
+class PITrackAWFreeze(PITrackAW):
+    """
+    PI controller with tracking anti-windup limiter and state freeze.
+    """
+    def __init__(self, u, kp, ki, ks, lower, upper, freeze, no_lower=False, no_upper=False,
+                 ref=0.0, x0=0.0, name=None, tex_name=None, info=None):
+        PITrackAW.__init__(self, u, kp, ki, ks, lower, upper, no_lower=no_lower, no_upper=no_upper,
+                           ref=ref, x0=x0, name=name, tex_name=tex_name, info=info)
+        self.freeze = dummify(freeze)
+
+        self.flag = EventFlag(u=self.freeze, tex_name='z^{flag}')
+        self.vars['flag'] = self.flag
+
+        self.ys.diag_eps = True
+        self.y.diag_eps = True
+
+    def define(self):
+        PITrackAW.define(self)
+
+        self.xi.e_str = f'(1-{self.freeze.name}) * {self.ki.name} * ' \
+                        f'({self.u.name} - {self.ref.name} -' \
+                        f' {self.ks.name} * ({self.name}_ys - {self.name}_y))'
+
+        self.ys.e_str = f'(1-{self.freeze.name}) * ' \
+                        f'({self.kp.name} * ({self.u.name} - {self.ref.name}) + {self.name}_xi - {self.name}_ys)'
+
+        self.y.e_str = f'(1 - {self.freeze.name}) * ' \
+                       f'({self.name}_ys * {self.name}_lim_zi +' \
+                       f' {self.lower.name} * {self.name}_lim_zl +' \
+                       f' {self.upper.name} * {self.name}_lim_zu - {self.name}_y)'
+
+
+class PIFreeze(PIController):
+    """
+    PI controller with state freeze.
+
+    Freezes state when the corresponding `freeze == 1`.
+
+    Notes
+    -----
+    Tested in `experimental.TestPITrackAW.PIFreeze`.
+    """
+    def __init__(self, u, kp, ki, freeze, ref=0.0, x0=0.0, name=None,
+                 tex_name=None, info=None):
+        PIController.__init__(self, u=u, kp=kp, ki=ki, ref=ref, x0=x0,
+                              name=name, tex_name=tex_name, info=info)
+        self.freeze = dummify(freeze)
+
+        self.flag = EventFlag(u=self.freeze, tex_name='z^{flag}')
+        self.vars['flag'] = self.flag
+
+        self.y.diag_eps = True
+
+    def define(self):
+        r"""
+        Notes
+        -----
+        One state variable ``xi`` and one algebraic variable ``y`` are added.
+
+        Equations implemented are
+
+        .. math ::
+            \dot{x_i} &= k_i * (u - ref) \\
+            y &= (1-freeze) * (x_i + k_p * (u - ref)) + freeze * y
+        """
+        PIController.define(self)
+        # state freeze does not affect initial values
+
+        self.xi.e_str = f'(1 - {self.freeze.name}) * {self.ki.name} * ' \
+                        f'({self.u.name} - {self.ref.name})'
+
+        # Freeze and unfreeze should invoke a Jacobian update
+        # In general, any equation switching should invoke a Jacobian update
+
+        self.y.e_str = f'(1 - {self.freeze.name}) * ' \
+                       f'({self.kp.name}*({self.u.name}-{self.ref.name}) + {self.name}_xi - {self.name}_y)'
 
 
 class PIControllerNumeric(Block):
-    """A PI Controller implemented with numerical function calls"""
+    """
+    A PI Controller implemented with numerical function calls.
 
-    def __init__(self, u, ref, kp, ki, name=None, info=None):
-        super().__init__(name=name, info=info)
+    `ref` must not be a variable.
+    """
+
+    def __init__(self, u, kp, ki, ref=0.0, name=None, tex_name=None, info=None):
+        super().__init__(name=name, tex_name=tex_name, info=info)
 
         self.u = u
-        self.ref = ref
-        self.kp = kp
-        self.ki = ki
+        self.ref = dummify(ref)
+        self.kp = dummify(kp)
+        self.ki = dummify(ki)
 
-        self.xi = State(info="Integration value of PI controller")
-        self.y = Algeb(info="Output value")
+        self.xi = State(info="Integrator value")
+        self.y = Algeb(info="PI output")
 
         self.vars = {'xi': self.xi, 'y': self.y}
         self.flags.update({'f_num': True, 'g_num': True, 'j_num': True})
 
     def g_numeric(self, **kwargs):
-        self.y.e = self.kp.v * (self.ref.v - self.u.v) + self.xi.v
+        self.y.e = self.kp.v * (self.u.v - self.ref.v) + self.xi.v - self.y.v
 
     def f_numeric(self, **kwargs):
-        self.xi.e = self.ki.v * (self.ref.v - self.u.v)
+        self.xi.e = self.ki.v * (self.u.v - self.ref.v)
 
     def j_numeric(self):
         self.j_reset()
-
-        # TODO: test the following code
-        self.triplets.append_ijv('fyc', self.xi.id, self.u.id, -self.ki.v)
-        self.triplets.append_ijv('gyc', self.y.id, self.u.id, -self.kp.v)
-        self.triplets.append_ijv('gxc', self.y.id, self.xi.id, 1)
+        self.triplets.append_ijv('fyc', self.xi.id, self.u.id, self.ki.v)
+        self.triplets.append_ijv('gyc', self.y.id, self.u.id, self.kp.v)
+        self.triplets.append_ijv('gxc', self.y.id, self.xi.id, 1.0)
+        self.triplets.append_ijv('gyc', self.y.id, self.y.id, -1.0)
 
     def define(self):
         """Skip the symbolic definition"""
@@ -460,7 +677,7 @@ class Washout(Block):
         self.enforce_tex_name((self.K, self.T))
 
         self.x = State(info='State in washout filter', tex_name="x'", t_const=self.T)
-        self.y = Algeb(info='Output of washout filter', tex_name=r'y', diag_eps=1e-6)
+        self.y = Algeb(info='Output of washout filter', tex_name=r'y', diag_eps=True)
         self.vars.update({'x': self.x, 'y': self.y})
 
     def define(self):
@@ -588,6 +805,35 @@ class Lag(Block):
         self.y.e_str = f'({self.K.name} * {self.u.name} - {self.name}_y)'
 
 
+class LagFreeze(Lag):
+    """
+    Lag with a state freeze input.
+    """
+    def __init__(self, u, T, K, freeze, name=None, tex_name=None, info=None):
+        Lag.__init__(self, u, T, K, name=name, tex_name=tex_name, info=info)
+        self.freeze = dummify(freeze)
+
+        self.flag = EventFlag(u=self.freeze, tex_name='z^{flag}')
+        self.vars['flag'] = self.flag
+
+        self.y.diag_eps = True
+
+    def define(self):
+        r"""
+        Notes
+        -----
+        Equations and initial values are
+
+        .. math ::
+
+            T \dot{y} &= (1 - freeze) * (Ku - y) \\
+            y^{(0)} &= K u
+
+        """
+        Lag.define(self)
+        self.y.e_str = f'(1 - {self.freeze.name})* ({self.K.name} * {self.u.name} - {self.name}_y)'
+
+
 class LagAntiWindup(Block):
     r"""
     Lag (low pass filter) transfer function block with an anti-windup limiter. ::
@@ -622,8 +868,8 @@ class LagAntiWindup(Block):
         self.T = dummify(T)
         self.K = dummify(K)
 
-        self.lower = lower
-        self.upper = upper
+        self.lower = dummify(lower)
+        self.upper = dummify(upper)
 
         self.enforce_tex_name((self.T, self.K))
 
@@ -651,6 +897,177 @@ class LagAntiWindup(Block):
         self.y.e_str = f'{self.K.name} * {self.u.name} - {self.name}_y'
 
 
+class LagAWFreeze(LagAntiWindup):
+    """
+    Lag with anti-windup limiter and state freeze.
+    """
+    def __init__(self, u, T, K, lower, upper, freeze,
+                 name=None, tex_name=None, info=None):
+        LagAntiWindup.__init__(self, u, T, K, lower, upper,
+                               name=name, tex_name=tex_name, info=info)
+        self.freeze = dummify(freeze)
+
+        self.flag = EventFlag(u=self.freeze, tex_name='z^{flag}')
+        self.vars['flag'] = self.flag
+
+        self.y.diag_eps = True
+
+    def define(self):
+        r"""
+        Notes
+        -----
+        Equations and initial values are
+
+        .. math ::
+
+            T \dot{y} &= (1 - freeze) (Ku - y) \\
+            y^{(0)} &= K u
+
+        """
+        LagAntiWindup.define(self)
+        self.y.e_str = f'(1 - {self.freeze.name}) * ({self.K.name} * {self.u.name} - {self.name}_y)'
+
+
+class LagRate(Block):
+    r"""
+    Lag (low pass filter) transfer function block with a rate limiter and an anti-windup limiter. ::
+
+                    rate_upper
+                   /
+             ┌────────┐
+             │    K   │
+        u -> │ ────── │ -> y
+             │ 1 + sT │
+             └────────┘
+                 /
+        rate_lower
+
+    Exports one state variable `y` as the output and one AntiWindupRate instance `lim`.
+
+    Parameters
+    ----------
+    K
+        Gain
+    T
+        Time constant
+    u
+        Input variable
+
+    """
+
+    def __init__(self, u, T, K,
+                 rate_lower, rate_upper,
+                 rate_no_lower=False, rate_no_upper=False,
+                 rate_lower_cond=None, rate_upper_cond=None,
+                 name=None, tex_name=None, info=None):
+        super().__init__(name=name, tex_name=tex_name, info=info)
+        self.u = dummify(u)
+        self.T = dummify(T)
+        self.K = dummify(K)
+
+        self.enforce_tex_name((self.T, self.K))
+
+        self.y = State(info='State in lag TF', tex_name="y",
+                       t_const=self.T)
+
+        self.lim = RateLimiter(u=self.y, lower=rate_lower, upper=rate_upper,
+                               no_lower=rate_no_lower, no_upper=rate_no_upper,
+                               lower_cond=rate_lower_cond, upper_cond=rate_upper_cond,
+                               tex_name='lim',
+                               info='Rate limiter in Lag')
+
+        self.vars = {'y': self.y, 'lim': self.lim}
+
+    def define(self):
+        r"""
+
+        Notes
+        -----
+        Equations and initial values are
+
+        .. math ::
+
+            T \dot{y} &= (Ku - y) \\
+            y^{(0)} &= K u
+
+        """
+        self.y.v_str = f'{self.u.name} * {self.K.name}'
+        self.y.e_str = f'{self.K.name} * {self.u.name} - {self.name}_y'
+
+
+class LagAntiWindupRate(Block):
+    r"""
+    Lag (low pass filter) transfer function block with a rate limiter and an anti-windup limiter. ::
+
+                     upper && rate_upper
+                   /¯¯¯¯¯¯
+             ┌────────┐
+             │    K   │
+        u -> │ ────── │ -> y
+             │ 1 + sT │
+             └────────┘
+           ______/
+           lower & rate_lower
+
+    Exports one state variable `y` as the output and one AntiWindupRate instance `lim`.
+
+    Parameters
+    ----------
+    K
+        Gain
+    T
+        Time constant
+    u
+        Input variable
+
+    """
+
+    def __init__(self, u, T, K,
+                 lower, upper, rate_lower, rate_upper,
+                 no_lower=False, no_upper=False, rate_no_lower=False, rate_no_upper=False,
+                 rate_lower_cond=None, rate_upper_cond=None,
+                 name=None, tex_name=None, info=None):
+        super().__init__(name=name, tex_name=tex_name, info=info)
+        self.u = dummify(u)
+        self.T = dummify(T)
+        self.K = dummify(K)
+
+        self.lower = dummify(lower)
+        self.upper = dummify(upper)
+
+        self.enforce_tex_name((self.T, self.K))
+
+        self.y = State(info='State in lag TF', tex_name="y",
+                       t_const=self.T,
+                       )
+        self.lim = AntiWindupRate(u=self.y, lower=self.lower, upper=self.upper,
+                                  rate_lower=rate_lower, rate_upper=rate_upper,
+                                  no_lower=no_lower, no_upper=no_upper,
+                                  rate_no_lower=rate_no_lower, rate_no_upper=rate_no_upper,
+                                  rate_lower_cond=rate_lower_cond, rate_upper_cond=rate_upper_cond,
+                                  tex_name='lim',
+                                  info='Limiter in Lag',
+                                  )
+
+        self.vars = {'y': self.y, 'lim': self.lim}
+
+    def define(self):
+        r"""
+
+        Notes
+        -----
+        Equations and initial values are
+
+        .. math ::
+
+            T \dot{y} &= (Ku - y) \\
+            y^{(0)} &= K u
+
+        """
+        self.y.v_str = f'{self.u.name} * {self.K.name}'
+        self.y.e_str = f'{self.K.name} * {self.u.name} - {self.name}_y'
+
+
 class Lag2ndOrd(Block):
     r"""
     Second order lag transfer function (low-pass filter) ::
@@ -751,7 +1168,7 @@ class LeadLag(Block):
         self.enforce_tex_name((self.T1, self.T2))
 
         self.x = State(info='State in lead-lag', tex_name="x'", t_const=self.T2)
-        self.y = Algeb(info='Output of lead-lag', tex_name=r'y', diag_eps=1e-6)
+        self.y = Algeb(info='Output of lead-lag', tex_name=r'y', diag_eps=True)
         self.vars = {'x': self.x, 'y': self.y}
 
         if self.zero_out is True:
@@ -828,7 +1245,7 @@ class LeadLag2ndOrd(Block):
 
         self.x1 = State(info='State #1 in 2nd order lead-lag', tex_name="x'", t_const=self.T2)
         self.x2 = State(info='State #2 in 2nd order lead-lag', tex_name="x''")
-        self.y = Algeb(info='Output of 2nd order lead-lag', tex_name='y', diag_eps=1e-6)
+        self.y = Algeb(info='Output of 2nd order lead-lag', tex_name='y', diag_eps=True)
 
         self.vars = {'x1': self.x1, 'x2': self.x2, 'y': self.y}
 
@@ -910,7 +1327,7 @@ class LeadLagLimit(Block):
         self.x = State(info='State in lead-lag TF', tex_name="x'", t_const=self.T2)
         self.ynl = Algeb(info='Output of lead-lag TF before limiter', tex_name=r'y_{nl}')
         self.y = Algeb(info='Output of lead-lag TF after limiter', tex_name=r'y',
-                       diag_eps=1e-6)
+                       diag_eps=True)
         self.lim = AntiWindup(u=self.ynl, lower=self.lower, upper=self.upper)
 
         self.vars = {'x': self.x, 'ynl': self.ynl, 'y': self.y, 'lim': self.lim}
@@ -1050,6 +1467,8 @@ class GainLimiter(Block):
              │     │ _____/
              └─────┘ lower
 
+    TODO: Add an extra gain block "R" for y.
+
     Parameters
     ----------
     u : str, BaseVar
@@ -1057,13 +1476,13 @@ class GainLimiter(Block):
 
     """
 
-    def __init__(self, u, K, upper, lower, no_upper=False, no_lower=False,
+    def __init__(self, u, K, lower, upper, no_lower=False, no_upper=False,
                  name=None, tex_name=None, info=None):
         Block.__init__(self, name=name, tex_name=tex_name, info=info)
         self.u = dummify(u)
         self.K = dummify(K)
-        self.upper = upper
-        self.lower = lower
+        self.upper = dummify(upper)
+        self.lower = dummify(lower)
 
         if (no_upper and no_lower) is True:
             raise ValueError("no_upper or no_lower cannot both be True")
@@ -1072,12 +1491,13 @@ class GainLimiter(Block):
         self.no_upper = no_upper
 
         self.x = Algeb(info='Gain output before limiter', tex_name='x')
-        self.y = Algeb(info='Gain output after limiter', tex_name='y')
 
         self.lim = HardLimiter(u=self.x, lower=self.lower, upper=self.upper,
                                no_upper=no_upper, no_lower=no_lower,
                                tex_name='lim')
 
+        self.y = Algeb(info='Gain output after limiter', tex_name='y', discrete=self.lim)
+
         self.vars = {'lim': self.lim, 'x': self.x, 'y': self.y}
 
     def define(self):
@@ -1100,13 +1520,73 @@ class GainLimiter(Block):
         self.y.e_str += f' - {self.name}_y'
 
 
+class LimiterGain(Block):
+    """
+    Limiter followed by a gain.
+
+    Exports the limited output `y`, unlimited output `x`, and HardLimiter `lim`. ::
+
+                   upper ┌─────┐
+                  /¯¯¯¯¯ │     │
+        u  ->    /   ->  │  K  │ -> y
+           _____/        │     │
+           lower         └─────┘
+
+    The intermediate variable before the gain is not saved.
+
+    Parameters
+    ----------
+    u : str, BaseVar
+        Input variable, or an equation string for constructing an anonymous variable
+
+    """
+
+    def __init__(self, u, K, lower, upper, no_lower=False, no_upper=False,
+                 name=None, tex_name=None, info=None):
+        Block.__init__(self, name=name, tex_name=tex_name, info=info)
+        self.u = u
+        self.K = dummify(K)
+        self.upper = dummify(upper)
+        self.lower = dummify(lower)
+
+        if (no_upper and no_lower) is True:
+            raise ValueError("no_upper or no_lower cannot both be True")
+
+        self.no_lower = no_lower
+        self.no_upper = no_upper
+
+        self.lim = HardLimiter(u=self.u, lower=self.lower, upper=self.upper,
+                               no_upper=no_upper, no_lower=no_lower,
+                               tex_name='lim')
+
+        self.y = Algeb(info='Gain output after limiter', tex_name='y', discrete=self.lim)
+
+        self.vars = {'lim': self.lim, 'y': self.y}
+
+    def define(self):
+        """
+        TODO: write docstring
+        """
+        self.y.e_str = f'{self.K.name} * {self.u.name} * {self.name}_lim_zi'
+        self.y.v_str = f'{self.K.name} * {self.u.name} * {self.name}_lim_zi'
+
+        if not self.no_upper:
+            self.y.e_str += f' + {self.K.name} * {self.name}_lim_zu*{self.upper.name}'
+            self.y.v_str += f' + {self.K.name} * {self.name}_lim_zu*{self.upper.name}'
+        if not self.no_lower:
+            self.y.e_str += f' + {self.K.name} * {self.name}_lim_zl*{self.lower.name}'
+            self.y.v_str += f' + {self.K.name} * {self.name}_lim_zl*{self.lower.name}'
+
+        self.y.e_str += f' - {self.name}_y'
+
+
 class Piecewise(Block):
     """
     Piecewise block. Outputs an algebraic variable `y`.
 
-    This block takes a list of N points, [x0, x1, ...x_{n-1}] to define N+1 ranges,
+    This block takes a list of `N` points, `[x0, x1, ...x_{n-1}]` to define N+1 ranges,
     namely (-inf, x0), (x0, x1), ..., (x_{n-1}, +inf).
-    and a list of N+1 functions [fun0, ..., fun_n].
+    and a list of `N+1` function strings `[fun0, ..., fun_n]`.
 
     Inputs that fall within each range applies the corresponding function.
     The first range (-inf, x0) applies `fun_0`, and
@@ -1147,3 +1627,56 @@ class Piecewise(Block):
 
         self.y.v_str = pw_fun
         self.y.e_str = f'{pw_fun} - {self.name}_y'
+
+
+class DeadBand1(Block):
+    """
+    Deadband type 1.
+
+    Parameters
+    ----------
+    center
+        Default value when within the deadband. If the input is an error signal,
+        center should be set to zero.
+
+    Notes
+    -----
+
+    Block diagram ::
+
+              |   /
+        ______|__/___
+           /  |
+          /   |
+
+    """
+    def __init__(self, u, center, lower, upper, enable=True,
+                 name=None, tex_name=None, info=None, namespace='local'):
+        Block.__init__(self, name=name, tex_name=tex_name, info=info, namespace=namespace)
+
+        self.u = dummify(u)
+        self.center = dummify(center)
+        self.lower = dummify(lower)
+        self.upper = dummify(upper)
+        self.enable = enable
+
+        self.db = DeadBand(u=u, center=center, lower=lower, upper=upper,
+                           enable=enable, tex_name='db')
+        self.y = Algeb(info='Deadband type 1 output', tex_name='y', discrete=self.db)
+        self.vars = {'db': self.db, 'y': self.y}
+
+    def define(self):
+        """
+        Notes
+        -----
+        Implemented equation:
+
+        .. math ::
+            0 = center + z_u * (u - upper) + z_l * (u - lower) - y
+
+        """
+        self.y.v_str = f'{self.center.name} + ' \
+                       f'{self.name}_db_zu * ({self.u.name} - {self.upper.name}) +' \
+                       f'{self.name}_db_zl * ({self.u.name} - {self.lower.name})'
+
+        self.y.e_str = self.y.v_str + f' - {self.name}_y'
