@@ -8,7 +8,7 @@ from andes.utils.tab import math_wrap, make_doc_table
 logger = logging.getLogger(__name__)
 
 
-class ModelFlags(object):
+class ModelFlags:
     """
     Model flags.
 
@@ -21,6 +21,10 @@ class ModelFlags(object):
         True: called during power flow
     tds : bool
         True if called during tds; if is False, ``dae_t`` cannot be used
+    pflow_init : bool or None
+        True if initialize pflow; False otherwise; None default to `pflow`
+    tds_init : bool or None
+        True if initialize tds; False otherwise; None default to `tds`
     series : bool
         True if is series device
     nr_iter : bool
@@ -35,15 +39,20 @@ class ModelFlags(object):
         True if the model defines `s_numeric`
     sv_num : bool
         True if the model defines `s_numeric_var`
+    jited : bool
+        True if numba JIT code is generated
     """
 
-    def __init__(self, collate=False, pflow=False, tds=False, series=False,
+    def __init__(self, collate=False, pflow=False, tds=False,
+                 pflow_init=None, tds_init=None, series=False,
                  nr_iter=False, f_num=False, g_num=False, j_num=False,
                  s_num=False, sv_num=False):
 
         self.collate = collate
         self.pflow = pflow
         self.tds = tds
+        self.pflow_init = pflow_init
+        self.tds_init = tds_init
         self.series = series
         self.nr_iter = nr_iter
         self.f_num = f_num
@@ -54,12 +63,16 @@ class ModelFlags(object):
         self.sys_base = False
         self.address = False
         self.initialized = False
+        self.jited = False
 
     def update(self, dct):
         self.__dict__.update(dct)
 
+    def __repr__(self):
+        return pprint.pformat(self.__dict__)
 
-class DummyValue(object):
+
+class DummyValue:
     """
     Class for converting a scalar value to a dummy parameter with `name` and `tex_name` fields.
 
@@ -84,7 +97,7 @@ def dummify(param):
 
     Parameters
     ----------
-    param : float, int, BaseParam
+    param : float, int, str, BaseParam
         parameter object or scalar value
 
     Returns
@@ -98,7 +111,7 @@ def dummify(param):
         return param
 
 
-class JacTriplet(object):
+class JacTriplet:
     """
     Storage class for Jacobian triplet lists.
     """
@@ -160,7 +173,7 @@ class JacTriplet(object):
                 self.vjac[jname + jtype] += triplet.vjac[jname + jtype]
 
 
-class Config(object):
+class Config:
     """
     A class for storing system, model and routine configurations.
     """
@@ -192,12 +205,33 @@ class Config(object):
 
         Existing configs will NOT be overwritten.
         """
+        def warn_upper(s):
+            if any(x.isupper() for x in s):
+                logger.warning(f"Config fields must be in lower case, found {s}")
+
         if dct is not None:
+            for s in dct.keys():
+                warn_upper(s)
+
             self._add(**dct)
 
+        for s in kwargs.keys():
+            warn_upper(s)
+
         self._add(**kwargs)
 
     def add_extra(self, dest, dct=None, **kwargs):
+        """
+        Add extra contents for config.
+
+        Parameters
+        ----------
+        dest : str
+            Destination string in `_alt`, `_help` or `_tex`.
+        dct : OrderedDict, dict
+            key: value pairs
+        """
+
         if dct is not None:
             kwargs.update(dct)
         for key, value in kwargs.items():
