@@ -1,5 +1,17 @@
+#  [ANDES] (C)2015-2020 Hantao Cui
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 3 of the License, or
+#  (at your option) any later version.
+#
+#  File name: discrete.py
+#  Last modified: 8/16/20, 7:28 PM
+
 import logging
 from typing import Optional, Union, Tuple, List
+
+from andes.core.common import dummify  # NOQA
 from andes.shared import np
 from andes.utils.tab import Tab
 from andes.utils.func import interp_n2
@@ -7,14 +19,14 @@ from andes.utils.func import interp_n2
 logger = logging.getLogger(__name__)
 
 
-class Discrete(object):
+class Discrete:
     """
     Base discrete class.
 
     Discrete classes export flag arrays (usually boolean) .
     """
 
-    def __init__(self, name=None, tex_name=None, info=None):
+    def __init__(self, name=None, tex_name=None, info=None, no_warn=False):
         self.name = name
         self.tex_name = tex_name
         self.info = info
@@ -22,8 +34,12 @@ class Discrete(object):
         self.export_flags = []
         self.export_flags_tex = []
         self.x_set = list()
-        self.y_set = list()  # NOT being used
-        self.warn_flags = []  # flags in `warn_flags` not initialized to zero will be warned
+        self.y_set = list()   # NOT being used
+        self.warn_flags = []  # warn if flags in `warn_flags` not initialized to zero
+        self.no_warn = no_warn
+
+        self.has_check_var = False  # if subclass implements `check_var()`
+        self.has_check_eq = False   # if subclass implements `check_eq()`
 
     def check_var(self, *args, **kwargs):
         """
@@ -37,15 +53,7 @@ class Discrete(object):
         """
         This function is called in ``l_check_eq`` after updating equations.
 
-        It should update internal flags only.
-        """
-        pass
-
-    def set_eq(self):
-        """
-        This function is used exclusively by AntiWindup for appending equations and values to ``x_set``.
-
-        It is called after ``check_eq``.
+        It updates internal flags, set differential equations, and record pegged variables.
         """
         pass
 
@@ -94,6 +102,9 @@ class Discrete(object):
         """
         Warn if initialized at limits.
         """
+        if self.no_warn:
+            return
+
         for f, limit in self.warn_flags:
             if f not in self.export_flags:
                 logger.error(f'warn_flags contain unknown flag {f}')
@@ -103,10 +114,15 @@ class Discrete(object):
             if not len(pos):
                 continue
             err_msg = f'{self.owner.class_name}.{self.name} {self.__dict__[limit].name} at limits'
+            if isinstance(self.__dict__[limit].v, np.ndarray):
+                lim_value = self.__dict__[limit].v[pos]
+            else:
+                lim_value = self.__dict__[limit].v
+
             err_data = {'idx': [self.owner.idx.v[i] for i in pos],
                         'Flag': [f] * len(pos),
                         'Input Value': self.u.v[pos],
-                        'Limit': self.__dict__[limit].v[pos]
+                        'Limit': lim_value * np.ones_like(pos),
                         }
 
             tab = Tab(title=err_msg,
@@ -128,11 +144,11 @@ class LessThan(Discrete):
     -----
     The default z0 and z1, if not enabled, can be set through the constructor.
     """
-    def __init__(self, u, bound, equal=False, enable=True, name=None, tex_name=None, cache=False,
-                 z0=0, z1=1):
-        super().__init__(name=name, tex_name=tex_name)
+    def __init__(self, u, bound, equal=False, enable=True, name=None, tex_name=None,
+                 info=None, cache=False, z0=0, z1=1):
+        super().__init__(name=name, tex_name=tex_name, info=info)
         self.u = u
-        self.bound = bound
+        self.bound = dummify(bound)
         self.equal: bool = equal
         self.enable: bool = enable
         self.cache: bool = cache
@@ -143,12 +159,15 @@ class LessThan(Discrete):
         self.export_flags = ['z0', 'z1']
         self.export_flags_tex = ['z_0', 'z_1']
 
+        self.has_check_var = True
+
     def check_var(self, *args, **kwargs):
         """
         If enabled, set flags based on inputs. Use cached values if enabled.
         """
         if not self.enable:
             return
+
         if self.cache and self._eval:
             return
 
@@ -184,6 +203,16 @@ class Limiter(Discrete):
         True to only use the upper limit
     no_upper : bool
         True to only use the lower limit
+    equal : bool
+        True to include equal signs in comparison (>= or <=).
+    no_warn : bool
+        Disable initial limit warnings
+    zu : 0 or 1
+        Default value for `zu` if not enabled
+    zl : 0 or 1
+        Default value for `zl` if not enabled
+    zi : 0 or 1
+        Default value for `zi` if not enabled
 
     Attributes
     ----------
@@ -197,18 +226,23 @@ class Limiter(Discrete):
     """
 
     def __init__(self, u, lower, upper, enable=True, name=None, tex_name=None, info=None,
-                 no_upper=False, no_lower=False):
-        super().__init__(name=name, tex_name=tex_name, info=info)
+                 no_upper=False, no_lower=False, equal=True, no_warn=False,
+                 zu=0.0, zl=0.0, zi=1.0):
+        Discrete.__init__(self, name=name, tex_name=tex_name, info=info)
         self.u = u
-        self.lower = lower
-        self.upper = upper
+        self.lower = dummify(lower)
+        self.upper = dummify(upper)
         self.enable = enable
         self.no_upper = no_upper
         self.no_lower = no_lower
+        self.equal = equal
+        self.no_warn = no_warn
 
-        self.zu = np.array([0])
-        self.zl = np.array([0])
-        self.zi = np.array([1])
+        self.zu = np.array([zu])
+        self.zl = np.array([zl])
+        self.zi = np.array([zi])
+
+        self.has_check_var = True
 
         self.export_flags = ['zi']
         self.export_flags_tex = ['z_i']
@@ -230,9 +264,16 @@ class Limiter(Discrete):
             return
 
         if not self.no_upper:
-            self.zu[:] = np.greater_equal(self.u.v, self.upper.v)
+            if self.equal:
+                self.zu[:] = np.greater_equal(self.u.v, self.upper.v)
+            else:
+                self.zu[:] = np.greater(self.u.v, self.upper.v)
+
         if not self.no_lower:
-            self.zl[:] = np.less_equal(self.u.v, self.lower.v)
+            if self.equal:
+                self.zl[:] = np.less_equal(self.u.v, self.lower.v)
+            else:
+                self.zl[:] = np.less(self.u.v, self.lower.v)
 
         self.zi[:] = np.logical_not(np.logical_or(self.zu, self.zl))
 
@@ -299,10 +340,16 @@ class AntiWindup(Limiter):
         by the anti-windup-limiter.
     """
 
-    def __init__(self, u, lower, upper, enable=True, name=None, tex_name=None, info=None, state=None):
-        super().__init__(u, lower, upper, enable=enable, name=name, tex_name=tex_name, info=info)
+    def __init__(self, u, lower, upper, enable=True,
+                 no_lower=False, no_upper=False, name=None, tex_name=None, info=None, state=None):
+        super().__init__(u, lower, upper, enable=enable,
+                         no_lower=no_lower, no_upper=no_upper,
+                         name=name, tex_name=tex_name, info=info)
         self.state = state if state else u
 
+        self.has_check_var = False
+        self.has_check_eq = True
+
     def check_var(self, *args, **kwargs):
         """
         This function is empty. Defers `check_var` to `check_eq`.
@@ -312,15 +359,6 @@ class AntiWindup(Limiter):
     def check_eq(self):
         """
         Check the variables and equations and set the limiter flags.
-        """
-        self.zu[:] = np.logical_and(np.greater_equal(self.u.v, self.upper.v),
-                                    np.greater_equal(self.state.e, 0))
-        self.zl[:] = np.logical_and(np.less_equal(self.u.v, self.lower.v),
-                                    np.less_equal(self.state.e, 0))
-        self.zi[:] = np.logical_not(np.logical_or(self.zu, self.zl))
-
-    def set_eq(self):
-        """
         Reset differential equation values based on limiter flags.
 
         Notes
@@ -328,6 +366,15 @@ class AntiWindup(Limiter):
         The current implementation reallocates memory for `self.x_set` in each call.
         Consider improving for speed. (TODO)
         """
+        if not self.no_upper:
+            self.zu[:] = np.logical_and(np.greater_equal(self.u.v, self.upper.v),
+                                        np.greater_equal(self.state.e, 0))
+
+        if not self.no_lower:
+            self.zl[:] = np.logical_and(np.less_equal(self.u.v, self.lower.v),
+                                        np.less_equal(self.state.e, 0))
+
+        self.zi[:] = np.logical_not(np.logical_or(self.zu, self.zl))
 
         # must flush the `x_set` list at the beginning
         self.x_set = list()
@@ -336,7 +383,8 @@ class AntiWindup(Limiter):
             idx = np.where(self.zi == 0)
             self.state.e[:] = self.state.e * self.zi
             self.state.v[:] = self.state.v * self.zi + self.upper.v * self.zu + self.lower.v * self.zl
-            self.x_set.append((self.state.a[idx], self.state.v[idx]))
+            self.x_set.append((self.state.a[idx], self.state.v[idx], 0))  # (address, var. values, eqn. values)
+
             # logger.debug(f'AntiWindup for states {self.state.a[idx]}')
 
         # Very important note:
@@ -346,6 +394,105 @@ class AntiWindup(Limiter):
         # Also, equation values are processed in `TDS` for resetting the `q`.
 
 
+class RateLimiter(Discrete):
+    """
+    Rate limiter for a differential variable.
+
+    RateLimiter does not export any variable. It directly modifies the differential equation value.
+
+    Notes
+    -----
+    RateLimiter inherits from Discrete to avoid internal naming conflicts with `Limiter`.
+
+    Warnings
+    --------
+    RateLimiter cannot be applied to a state variable that already undergoes an AntiWindup limiter.
+    Use `AntiWindupRate` for a rate-limited anti-windup limiter.
+    """
+    def __init__(self, u, lower, upper, enable=True,
+                 no_lower=False, no_upper=False, lower_cond=None, upper_cond=None,
+                 name=None, tex_name=None, info=None):
+        Discrete.__init__(self, name=name, tex_name=tex_name, info=info)
+        self.u = u
+        self.rate_lower = dummify(lower)
+        self.rate_upper = dummify(upper)
+
+        # `lower_cond` and `upper_cond` are arrays of 0/1 indicating whether
+        # the corresponding rate limit should be *enabled*.
+        # 0 - disabled, 1 - enabled.
+        # If is `None`, all rate limiters will be enabled.
+
+        self.rate_lower_cond = dummify(lower_cond)
+        self.rate_upper_cond = dummify(upper_cond)
+
+        self.rate_no_lower = no_lower
+        self.rate_no_upper = no_upper
+        self.enable = enable
+
+        self.zur = np.array([0])
+        self.zlr = np.array([0])
+
+        self.has_check_eq = True
+
+        # Note: save ops by not calculating `zir`
+        # self.zir = np.array([1])
+        # self.export_flags = ['zir']
+        # self.export_flags_tex = ['z_{ir}']
+
+        if not self.rate_no_lower:
+            self.export_flags.append('zlr')
+            self.export_flags_tex.append('z_{lr}')
+            self.warn_flags.append(('zlr', 'lower'))
+        if not self.rate_no_upper:
+            self.export_flags.append('zur')
+            self.export_flags_tex.append('z_{ur}')
+            self.warn_flags.append(('zur', 'upper'))
+
+    def check_eq(self):
+        if not self.enable:
+            return
+
+        if not self.rate_no_lower:
+            self.zlr[:] = np.less(self.u.e, self.rate_lower.v)  # 1 if at the lower rate limit
+
+            if self.rate_lower_cond is not None:
+                self.zlr[:] = self.zlr * self.rate_lower_cond.v  # 1 if both at the lower rate limit and enabled
+
+            # for where `zlr == 1`, set the equation value to the lower limit
+            self.u.e[np.where(self.zlr)] = self.rate_lower.v
+
+        if not self.rate_no_upper:
+            self.zur[:] = np.greater(self.u.e, self.rate_upper.v)
+
+            if self.rate_upper_cond is not None:
+                self.zur[:] = self.zur * self.rate_upper_cond.v
+
+            self.u.e[np.where(self.zur)] = self.rate_upper.v
+
+
+class AntiWindupRate(AntiWindup, RateLimiter):
+    """
+    Anti-windup limiter with rate limits
+    """
+    def __init__(self, u, lower, upper, rate_lower, rate_upper,
+                 no_lower=False, no_upper=False, rate_no_lower=False, rate_no_upper=False,
+                 rate_lower_cond=None, rate_upper_cond=None,
+                 enable=True, name=None, tex_name=None, info=None):
+        RateLimiter.__init__(self, u, lower=rate_lower, upper=rate_upper, enable=enable,
+                             no_lower=rate_no_lower, no_upper=rate_no_upper,
+                             lower_cond=rate_lower_cond, upper_cond=rate_upper_cond,
+                             )
+
+        AntiWindup.__init__(self, u, lower=lower, upper=upper, enable=enable,
+                            no_lower=no_lower, no_upper=no_upper,
+                            name=name, tex_name=tex_name, info=info,
+                            )
+
+    def check_eq(self):
+        RateLimiter.check_eq(self)
+        AntiWindup.check_eq(self)
+
+
 class Selector(Discrete):
     """
     Selection between two variables using the provided reduce function.
@@ -406,6 +553,8 @@ class Selector(Discrete):
         self.export_flags = [f's{i}' for i in range(len(self.input_vars))]
         self.export_flags_tex = [f's_{i}' for i in range(len(self.input_vars))]
 
+        self.has_check_var = True
+
     def check_var(self, *args, **kwargs):
         """
         Set the i-th variable's flags to 1 if the return of the reduce function equals the i-th input.
@@ -476,6 +625,8 @@ class Switcher(Discrete):
         self.export_flags = [f's{i}' for i in range(len(options))]
         self.export_flags_tex = [f's_{i}' for i in range(len(options))]
 
+        self.has_check_var = True
+
     def check_var(self, *args, **kwargs):
         """
         Set the switcher flags based on inputs. Uses cached flags if cache is set to True.
@@ -502,7 +653,7 @@ class Switcher(Discrete):
 
 class DeadBand(Limiter):
     r"""
-    Dead band with the direction of return.
+    The basic deadband type.
 
     Parameters
     ----------
@@ -513,21 +664,89 @@ class DeadBand(Limiter):
     lower : NumParam
         Lower bound
     upper : NumParam
-        Upper bpund
+        Upper bound
     enable : bool
         Enabled if True; Disabled and works as a pass-through if False.
 
     Notes
     -----
 
-    Input changes within a deadband will incur no output changes. This component computes and exports five flags.
+    Input changes within a deadband will incur no output changes. This component computes and exports three flags.
 
     Three flags computed from the current input:
      - zl: True if the input is below the lower threshold
      - zi: True if the input is within the deadband
      - zu: True if is above the lower threshold
 
-    Two flags indicating the direction of return:
+    Initial condition:
+
+    All three flags are initialized to zero. All flags are updated during `check_var` when enabled. If the
+    deadband component is not enabled, all of them will remain zero.
+
+    Examples
+    --------
+
+    Exported deadband flags need to be used in the algebraic equation corresponding to the post-deadband variable.
+    Assume the pre-deadband input variable is `var_in` and the post-deadband variable is `var_out`. First, define a
+    deadband instance `db` in the model using ::
+
+        self.db = DeadBand(u=self.var_in, center=self.dbc,
+                           lower=self.dbl, upper=self.dbu)
+
+    To implement a no-memory deadband whose output returns to center when the input is within the band,
+    the equation for `var` can be written as ::
+
+        var_out.e_str = 'var_in * (1 - db_zi) + \
+                         (dbc * db_zi) - var_out'
+
+    """
+    def __init__(self, u, center, lower, upper, enable=True, equal=True, zu=0.0, zl=0.0, zi=0.0,
+                 name=None, tex_name=None, info=None):
+        Limiter.__init__(self, u, lower, upper, enable=enable, equal=equal, zi=zi, zl=zl, zu=zu,
+                         name=name, tex_name=tex_name, info=info)
+        self.center = dummify(center)  # CURRENTLY NOT IN USE
+
+    def check_var(self, *args, **kwargs):
+        """
+        Notes
+        -----
+
+        Updates three flags: zi, zu, zl based on the following rules:
+
+        zu:
+          1 if u > upper; 0 otherwise.
+
+        zl:
+          1 if u < lower; 0 otherwise.
+
+        zi:
+          not(zu or zl);
+        """
+        Limiter.check_var(self, *args, **kwargs)
+
+
+class DeadBandRT(DeadBand):
+    r"""
+    Deadband with flags for directions of return.
+
+    Parameters
+    ----------
+    u : NumParam
+        The pre-deadband input variable
+    center : NumParam
+        Neutral value of the output
+    lower : NumParam
+        Lower bound
+    upper : NumParam
+        Upper bound
+    enable : bool
+        Enabled if True; Disabled and works as a pass-through if False.
+
+    Notes
+    -----
+    Input changes within a deadband will incur no output changes. This component computes and exports five flags.
+    The additional two flags on top of `DeadBand` indicate the direction of return:
+
      - zur: True if the input is/has been within the deadband and was returned from the upper threshold
      - zlr: True if the input is/has been within the deadband and was returned from the lower threshold
 
@@ -539,21 +758,6 @@ class DeadBand(Limiter):
     Examples
     --------
 
-    Exported deadband flags need to be used in the algebraic equation corresponding to the post-deadband variable.
-    Assume the pre-deadband input variable is `var_in` and the post-deadband variable is `var_out`. First, define a
-    deadband instance `db` in the model using ::
-
-        self.db = DeadBand(u=self.var_in,
-                           center=self.dbc,
-                           lower=self.dbl,
-                           upper=self.dbu)
-
-    To implement a no-memory deadband whose output returns to center when the input is within the band,
-    the equation for `var` can be written as ::
-
-        var_out.e_str = 'var_in * (1 - db_zi) + \
-                         (dbc * db_zi) - var_out'
-
     To implement a deadband whose output is pegged at the nearest deadband bounds, the equation for `var` can be
     provided as ::
 
@@ -566,18 +770,16 @@ class DeadBand(Limiter):
         """
 
         """
-        super().__init__(u, lower, upper, enable=enable)
-        self.center = center
+        DeadBand.__init__(self, u, center=center, lower=lower, upper=upper, enable=enable)
 
         # default state if not enabled
-        self.zi = np.array([0.])
-        self.zl = np.array([0.])
-        self.zu = np.array([0.])
         self.zur = np.array([0.])
         self.zlr = np.array([0.])
 
-        self.export_flags = ['zl', 'zi', 'zu', 'zur', 'zlr']
-        self.export_flags_tex = ['z_l', 'z_i', 'z_u', 'z_ur', 'z_lr']
+        self.export_flags.extend(['zur', 'zlr'])
+        self.export_flags_tex.extend(['z_ur', 'z_lr'])
+
+        self.has_check_var = True
 
     def check_var(self, *args, **kwargs):
         """
@@ -605,22 +807,24 @@ class DeadBand(Limiter):
          - hold when (previous zi == zi)
          - clear otherwise
         """
+        DeadBand.check_var(self, *args, **kwargs)
+
         if not self.enable:
             return
-        zu = np.greater(self.u.v, self.upper.v)
-        zl = np.less(self.u.v, self.lower.v)
-        zi = np.logical_not(np.logical_or(zu, zl))
 
         # square return dead band
-        self.zur[:] = np.equal(self.zu + zi, 2) + self.zur * np.equal(zi, self.zi)
-        self.zlr[:] = np.equal(self.zl + zi, 2) + self.zlr * np.equal(zi, self.zi)
+        self.zur[:] = np.equal(self.zu + self.zi, 2) + self.zur * np.equal(self.zi, self.zi)
+        self.zlr[:] = np.equal(self.zl + self.zi, 2) + self.zlr * np.equal(self.zi, self.zi)
 
 
 class Delay(Discrete):
     """
     Delay class to memorize past variable values.
 
-    TODO: Add documentation.
+    Delay allows to impose a predefined "delay" (in either steps or seconds)
+    for an input variable. The amount of delay is a scalar and has to be fixed
+    at model definition for now.
+
     """
 
     def __init__(self, u, mode='step', delay=0, name=None, tex_name=None, info=None):
@@ -634,6 +838,7 @@ class Delay(Discrete):
         self.delay = delay
         self.export_flags = ['v']
         self.export_flags_tex = ['v']
+        self.has_check_var = True
 
         self.t = np.array([0])
         self.v = np.array([0])
@@ -739,3 +944,76 @@ class Derivative(Delay):
         else:
             self.v[:] = (self._v_mem[:, 1] - self._v_mem[:, 0]) / (self.t[1] - self.t[0])
             self.v[np.where(self.v < 1e-8)] = 0
+
+
+class Sampling(Discrete):
+    """
+    Sample an input variable repeatedly at a given time interval.
+    """
+    def __init__(self, u, interval=1.0, offset=0.0, name=None, tex_name=None, info=None):
+        Discrete.__init__(self, name=name, tex_name=tex_name, info=info)
+
+        self.u = u
+        self.interval = interval
+        self.offset = offset
+
+        self.export_flags = ['v']
+        self.export_flags_tex = ['v']
+        self.has_check_var = True
+
+        self.v = np.array([0])
+        self._last_t = np.array([0])
+        self._last_v = np.array([0])
+        self.indices = np.array([0])
+
+        self.rewind = False
+
+    def list2array(self, n):
+        super().list2array(n)
+        self._last_v = np.zeros(n)
+
+    def check_var(self, dae_t, *args, **kwargs):
+        """
+        Check and update the output.
+
+        Notes
+        -----
+        Present output stored in `v`. Output of the last step is stored in `_last_v`.
+        Time for the last output is stored in `_last_t`.
+
+        Initially, store `v` and `_last_v`.
+
+        If time progresses and `dae_t` is a multiple of `period`, update `_last_v` and then `v`.
+        Record `_last_t`.
+
+        If time does not progress, update `v`.
+
+        If time rewinds, restore `_last_v` to `v`.
+
+        """
+        self.rewind = False
+
+        if dae_t == 0:
+            # initial step
+            self._last_v[:] = self.u.v[:]
+            self.v[:] = self.u.v[:]
+
+        elif dae_t > self._last_t:
+            do_sample = (dae_t - self.offset - self._last_t) > self.interval
+
+            if do_sample:
+                self._last_v[:] = self.v
+                self.v[:] = self.u.v
+                self._last_t[0] = dae_t
+
+        elif dae_t == self._last_t:
+            if len(self.indices) > 0:
+                self.v[:] = self.u.v
+
+        else:
+            # if dae_t < self._last_t
+            self.rewind = True
+
+            if self._last_t[0] > dae_t:
+                self.v[:] = self._last_v
+                self._last_t[0] = dae_t
