@@ -1,14 +1,23 @@
+#  [ANDES] (C)2015-2020 Hantao Cui
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 3 of the License, or
+#  (at your option) any later version.
+#
+#  File name: param.py
+#  Last modified: 8/16/20, 7:27 PM
+
 from typing import Optional, Union, Callable, List, Tuple
 
 import math
 import logging
 
-from andes.models.group import GroupBase
 from andes.shared import np
 logger = logging.getLogger(__name__)
 
 
-class BaseParam(object):
+class BaseParam:
     """
     The base parameter class.
 
@@ -139,6 +148,15 @@ class BaseParam(object):
         """Return the count of elements in the value array."""
         return len(self.v)
 
+    def __repr__(self):
+        span = ''
+        if 1 <= self.n <= 20:
+            span = f', v={self.v}'
+            if hasattr(self, 'vin') and (self.vin is not None):
+                span += f', vin={self.vin}'
+
+        return f'{self.__class__.__name__}: {self.owner.__class__.__name__}.{self.name}{span}'
+
 
 class DataParam(BaseParam):
     """
@@ -280,8 +298,6 @@ class NumParam(BaseParam):
                  info: Optional[str] = None,
                  unit: Optional[str] = None,
                  vrange: Optional[Union[List, Tuple]] = None,
-                 Sn: str = 'Sn',
-                 Vn: str = 'Vn',
                  non_zero: bool = False,
                  positive: bool = False,
                  mandatory: bool = False,
@@ -314,8 +330,6 @@ class NumParam(BaseParam):
                              dc_current=dc_current,
                              dc_voltage=dc_voltage)
 
-        self.Sn = Sn
-        self.Vn = Vn
         self.pu_coeff = np.ndarray([])
         self.vin = None  # values from input
         self.vrange = vrange
@@ -542,7 +556,7 @@ class ExtParam(NumParam):
         """
         self.parent_model = ext_model
 
-        if isinstance(ext_model, GroupBase):
+        if hasattr(ext_model, "_idx2model"):
             # copy properties from models in the group
 
             # TODO: the three `get` calls below is a bit inefficient - same loops for three times
@@ -579,12 +593,17 @@ class ExtParam(NumParam):
                     uid = ext_model.idx2uid(self.indexer.v)
 
             # pull in values
-            self.v = parent_instance.v[uid]
+            if isinstance(parent_instance.v, np.ndarray):
+                self.v = parent_instance.v[uid]
+            else:
+                self.v = [parent_instance.v[i] for i in uid]
             try:
                 self.vin = parent_instance.vin[uid]
                 self.pu_coeff = parent_instance.pu_coeff[uid]
             except KeyError:
                 pass
+            except AttributeError:
+                pass
 
     def to_array(self):
         """
