@@ -1,3 +1,13 @@
+#  [ANDES] (C)2015-2020 Hantao Cui
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 3 of the License, or
+#  (at your option) any later version.
+#
+#  File name: service.py
+#  Last modified: 8/16/20, 7:28 PM
+
 from typing import Optional, Union, Callable, Type
 from andes.core.param import BaseParam
 from andes.utils.func import list_flatten
@@ -10,7 +20,7 @@ from andes.utils.tab import Tab
 logger = logging.getLogger(__name__)
 
 
-class BaseService(object):
+class BaseService:
     """
     Base class for Service.
 
@@ -32,7 +42,7 @@ class BaseService(object):
         self.name = name
         self.tex_name = tex_name if tex_name else name
         self.info = info
-        self.vtype = vtype  # type for `v`. NOT IN USE.
+        self.vtype = vtype if vtype is not None else np.float  # type for `v`
         self.owner = None
 
     def get_names(self):
@@ -70,6 +80,14 @@ class BaseService(object):
         """
         return self.__class__.__name__
 
+    def __repr__(self):
+        val_str = ''
+
+        if (1 <= self.n <= 20) and hasattr(self, 'v'):
+            val_str = f', v={self.v}'
+
+        return f'{self.class_name}: {self.owner.class_name}.{self.name}{val_str}'
+
 
 class ConstService(BaseService):
     """
@@ -97,15 +115,16 @@ class ConstService(BaseService):
     def __init__(self,
                  v_str: Optional[str] = None,
                  v_numeric: Optional[Callable] = None,
+                 vtype: Optional[type] = None,
                  name: Optional[str] = None, tex_name=None, info=None):
-        super().__init__(name=name, tex_name=tex_name, info=info)
+        super().__init__(name=name, vtype=vtype, tex_name=tex_name, info=info)
         self.v_str = v_str
         self.v_numeric = v_numeric
         self.v: Union[float, int, ndarray] = np.array([0.])
 
     def assign_memory(self, n):
         """Assign memory for ``self.v`` and set the array to zero."""
-        self.v = np.zeros(n)
+        self.v = np.zeros(n, dtype=self.vtype)
 
 
 class VarService(ConstService):
@@ -143,6 +162,172 @@ class VarService(ConstService):
     pass
 
 
+class EventFlag(VarService):
+    """
+    Service to flag events.
+
+    `EventFlag.v` stores the values of the input variable from the previous iteration/step.
+    """
+
+    def __init__(self,
+                 u,
+                 vtype: Optional[type] = None,
+                 name: Optional[str] = None, tex_name=None, info=None):
+        VarService.__init__(self, v_numeric=self.check,
+                            vtype=vtype, name=name, tex_name=tex_name, info=info)
+        self.u = dummify(u)
+
+    def check(self, **kwargs):
+        """
+        Check status and set event flags.
+        """
+        if not np.all(self.v == self.u.v):
+            self.owner.system.TDS.custom_event = True
+            logger.debug(f"Event flag set at t={self.owner.system.dae.t:.6f} sec.")
+
+        return self.u.v
+
+
+class VarHold(VarService):
+    """
+    Service for holding the input when the hold state is on.
+    """
+    def __init__(self, u, hold, vtype=None, name=None, tex_name=None, info=None):
+        VarService.__init__(self, v_numeric=self.check, vtype=vtype,
+                            name=name, tex_name=tex_name, info=info,
+                            )
+        self.u = dummify(u)
+        self.hold = dummify(hold)
+        self._init = False
+
+    def check(self, **kwargs):
+        if not np.all(self.hold.v == 0.0):
+            hold_idx = np.where(self.hold.v == 1)
+
+            ret = self.u.v.copy()
+            ret[hold_idx] = self.v[hold_idx]
+
+            return ret
+
+        else:
+            return self.u.v
+
+
+class ExtendedEvent(VarService):
+    """
+    Service to flag events that extends for period of time after event disappears.
+
+    `EventFlag.v` stores the flags whether the extended time has completed.
+    Outputs will become 1 once then event starts until the extended time ends.
+
+    Warnings
+    --------
+    The performance of this class needs to be optimized.
+
+    Parameters
+    ----------
+    trig : str, rise, fall
+        Triggering edge for the inception of an event. `rise` by default.
+
+    enable : bool or v-provider
+        If disabled, the output will be `v_disabled`
+
+    extend_only : bool
+        Only output during the extended period, not the event period.
+    """
+
+    def __init__(self,
+                 u,
+                 t_ext: Union[int, float, BaseParam, BaseService] = 0.0,
+                 trig: str = 'rise',
+                 enable=True,
+                 v_disabled=0,
+                 extend_only=False,
+                 vtype: Optional[type] = None,
+                 name: Optional[str] = None, tex_name=None, info=None):
+        VarService.__init__(self, v_numeric=self.check,
+                            vtype=vtype, name=name, tex_name=tex_name, info=info)
+
+        self.u = dummify(u)
+        self.t_ext = dummify(t_ext)
+        self.enable = dummify(enable)
+        self.v_disabled = v_disabled
+        self.extend_only = extend_only
+
+        self.t_final = None
+        self.trig = trig
+
+        self.v_event = None
+        self.u_last = None
+        self.z = None  # if is in an extended event (from event start to extension end)
+        self.n_ext = 0  # number of extended events
+
+    def assign_memory(self, n):
+        """
+        Assign memory for internal data.
+        """
+        VarService.assign_memory(self, n)
+        self.t_final = np.zeros_like(self.v)
+        self.v_event = np.zeros_like(self.v)
+        self.u_last = np.zeros_like(self.v)
+        self.z = np.zeros_like(self.v)
+
+        if isinstance(self.t_ext.v, (int, float)):
+            self.t_ext.v = np.ones_like(self.u.v) * self.t_ext.v
+
+    def check(self, **kwargs):
+        """
+        Check if an extended event is in place.
+
+        Supplied as a ``v_numeric`` to ``VarService``.
+        """
+        dae_t = self.owner.system.dae.t
+
+        if dae_t == 0.0:
+            self.u_last[:] = self.u.v
+            self.v_event[:] = self.u.v
+
+        # when any input signal changes
+        if not np.all(self.u.v == self.u_last):
+            diff = self.u.v - self.u_last
+
+            # detect the actual ending of an event
+            if self.trig == 'rise':
+                starting = np.where(diff == 1)[0]
+                ending = np.where(diff == -1)[0]
+            else:
+                starting = np.where(diff == -1)[0]
+                ending = np.where(diff == 1)[0]
+
+            if len(starting):
+                self.z[starting] = 1
+
+                if not self.extend_only:
+                    self.v_event[starting] = self.u.v[starting]
+
+            if len(ending):
+                if self.extend_only:
+                    self.v_event[ending] = self.u_last[ending]
+
+                final_times = dae_t + self.t_ext.v[ending]
+                self.t_final[ending] = final_times
+
+                self.n_ext += len(ending)
+
+                # TODO: insert extended event end times to a model-level list
+                logger.debug(f"Extended Event ending time set at t={final_times} sec.")
+
+        # final time of the extended event
+        if self.n_ext and np.any(self.t_final <= dae_t):
+            self.z[np.where(self.t_final <= dae_t)] = 0
+            self.n_ext = np.count_nonzero(self.z)
+
+        self.u_last[:] = self.u.v
+
+        return self.enable.v * (self.u.v * (1 - self.z) + self.v_event * self.z) + \
+            (1-self.enable.v) * self.v_disabled
+
+
 class PostInitService(ConstService):
     """
     Constant service that gets stored once after init.
@@ -170,81 +355,6 @@ class PostInitService(ConstService):
     pass
 
 
-class ExtService(BaseService):
-    """
-    Service constants whose value is from an external model or group.
-
-    Parameters
-    ----------
-    src : str
-        Variable or parameter name in the source model or group
-    model : str
-        A model name or a group name
-    indexer : IdxParam or BaseParam
-        An "Indexer" instance whose ``v`` field contains the ``idx`` of devices in the model or group.
-
-    Examples
-    --------
-    A synchronous generator needs to retrieve the ``p`` and ``q`` values from static generators
-    for initialization. ``ExtService`` is used for this purpose.
-
-    In a synchronous generator, one can define the following to retrieve ``StaticGen.p`` as ``p0``::
-
-        class GENCLSModel(Model):
-            def __init__(...):
-                ...
-                self.p0 = ExtService(src='p',
-                                     model='StaticGen',
-                                     indexer=self.gen,
-                                     tex_name='P_0')
-
-    """
-
-    def __init__(self,
-                 model: str,
-                 src: str,
-                 indexer: BaseParam,
-                 attr='v',
-                 allow_none=False,
-                 default=0,
-                 name: str = None,
-                 tex_name: str = None,
-                 info=None,
-                 ):
-        super().__init__(name=name, tex_name=tex_name, info=info)
-        self.model = model
-        self.src = src
-        self.indexer = indexer
-        self.attr = attr
-        self.allow_none = allow_none
-        self.default = default
-        self.v = np.array([0.])
-
-    def assign_memory(self, n):
-        """Assign memory for ``self.v`` and set the array to zero."""
-        self.v = np.zeros(n)
-
-    def link_external(self, ext_model):
-        """
-        Method to be called by ``System`` for getting values from the external model or group.
-
-        Parameters
-        ----------
-        ext_model
-            An instance of a model or group provided by System
-        """
-        # set initial v values to zero
-        self.v = np.zeros(self.n)
-        if self.n == 0:
-            return
-
-        # the same `get` api for Group and Model
-        self.v = ext_model.get(src=self.src, idx=self.indexer.v, attr=self.attr,
-                               allow_none=self.allow_none,
-                               default=self.default,
-                               )
-
-
 class BackRef(BaseService):
     """
     A special type of reference collector.
@@ -313,6 +423,82 @@ class BackRef(BaseService):
         self.v = list()
 
 
+class ExtService(BaseService):
+    """
+    Service constants whose value is from an external model or group.
+
+    Parameters
+    ----------
+    src : str
+        Variable or parameter name in the source model or group
+    model : str
+        A model name or a group name
+    indexer : IdxParam or BaseParam
+        An "Indexer" instance whose ``v`` field contains the ``idx`` of devices in the model or group.
+
+    Examples
+    --------
+    A synchronous generator needs to retrieve the ``p`` and ``q`` values from static generators
+    for initialization. ``ExtService`` is used for this purpose.
+
+    In a synchronous generator, one can define the following to retrieve ``StaticGen.p`` as ``p0``::
+
+        class GENCLSModel(Model):
+            def __init__(...):
+                ...
+                self.p0 = ExtService(src='p',
+                                     model='StaticGen',
+                                     indexer=self.gen,
+                                     tex_name='P_0')
+
+    """
+
+    def __init__(self,
+                 model: str,
+                 src: str,
+                 indexer: Union[BaseParam, BaseService],
+                 attr: str = 'v',
+                 allow_none: bool = False,
+                 default=0,
+                 name: str = None,
+                 tex_name: str = None,
+                 vtype=None,
+                 info: str = None,
+                 ):
+        super().__init__(name=name, tex_name=tex_name, info=info, vtype=vtype)
+        self.model = model
+        self.src = src
+        self.indexer = indexer
+        self.attr = attr
+        self.allow_none = allow_none
+        self.default = default
+        self.v = np.array([0.])
+
+    def assign_memory(self, n):
+        """Assign memory for ``self.v`` and set the array to zero."""
+        self.v = np.zeros(n, dtype=self.vtype)
+
+    def link_external(self, ext_model):
+        """
+        Method to be called by ``System`` for getting values from the external model or group.
+
+        Parameters
+        ----------
+        ext_model
+            An instance of a model or group provided by System
+        """
+        # set initial v values to zero
+        self.v = np.zeros(self.n)
+        if self.n == 0:
+            return
+
+        # the same `get` api for Group and Model
+        self.v = ext_model.get(src=self.src, idx=self.indexer.v, attr=self.attr,
+                               allow_none=self.allow_none,
+                               default=self.default,
+                               )
+
+
 class DataSelect(BaseService):
     """
     Class for selecting values for optional DataParam or NumParam.
@@ -520,11 +706,13 @@ class NumReduce(OperationService):
                  name=None,
                  tex_name=None,
                  info=None,
+                 cache=True,
                  ):
         super().__init__(name=name, tex_name=tex_name, info=info)
         self.u = u
         self.ref = ref
         self.fun = fun
+        self.cache = cache
 
     @property
     def v(self):
@@ -535,15 +723,17 @@ class NumReduce(OperationService):
         -------
         The array ``self._v`` storing the reduced values
         """
+        if self._v is not None and self.cache is True:
+            return self._v
+
         if self._v is None:
             self._v = np.zeros(len(self.ref.v))
-            idx = 0
-            for i, v in enumerate(self.ref.v):
-                self._v[i] = self.fun(self.u.v[idx:idx + len(v)])
-                idx += len(v)
-            return self._v
-        else:
-            return self._v
+
+        idx = 0
+        for i, v in enumerate(self.ref.v):
+            self._v[i] = self.fun(self.u.v[idx:idx + len(v)])
+            idx += len(v)
+        return self._v
 
 
 class NumRepeat(OperationService):
@@ -850,36 +1040,239 @@ class InitChecker(OperationService):
         self.v[:] = np.logical_not(self.v)
 
 
-class FlagNotNone(BaseService):
+class FlagValue(BaseService):
     """
-    Class for flagging non-None indices as 1 and None indices as 0 in a numpy array.
+    Class for flagging values that equal to the given value.
+
+    By default, values that equal to `value` will be flagged as `0`.
+    Non-matching values will be flagged as `1`.
+
+    Parameters
+    ----------
+    u
+        Input parameter
+    value
+        Value to flag. Can be None, string, or a number.
+    flag : 0 by default, only 0 or 1 is accepted.
+        The flag for the matched ones
 
     Warnings
     --------
-    FlagNotNone can only be applied to BaseParam with `cache=True`.
-    Applying to Service will fail until cache is False (at a performance cost).
+    `FlagNotNone` can only be applied to `BaseParam` with `cache=True`.
+    Applying to `Service` will fail unless `cache` is False (at a performance cost).
     """
 
-    def __init__(self: BaseParam, indexer, to_flag=None, name=None, tex_name=None, info=None, cache=True):
+    def __init__(self, u, value, flag=0, name=None, tex_name=None, info=None, cache=True):
         BaseService.__init__(self, name=name, tex_name=tex_name, info=info)
+        if flag != 0.0 and flag != 1.0:
+            raise ValueError(f"flag must be 0 or 1. The given flag = {flag}.")
+
+        self.u = u
+        self.value = value
+        self.flag = flag
+        self.flag_neg = 1 - flag
         self.cache = cache
-        self.to_flag = to_flag
-        self.indexer = indexer
+
         self._v = None
 
     @property
     def v(self):
         new = False
         if self._v is None:
-            self._v = np.zeros_like(self.indexer.v, dtype=float)
+            self._v = np.zeros_like(self.u.v, dtype=float)
             new = True
 
         if not self.cache or new:
-            self._v[:] = np.array([0 if i == self.to_flag else 1
-                                   for i in self.indexer.v])
+            # need to do it element-wise since `self.u.v` can be a list
+            self._v[:] = np.array([self.flag if i == self.value else self.flag_neg
+                                   for i in self.u.v])
         return self._v
 
 
+class ApplyFunc(BaseService):
+    """
+    Class for applying a numerical function on a parameter..
+
+
+    Warnings
+    --------
+    This class is not ready.
+
+    Parameters
+    ----------
+    u
+        Input parameter
+    func
+        A condition function that returns True or False.
+
+    """
+
+    def __init__(self, u, func, name=None, tex_name=None, info=None, cache=True):
+        BaseService.__init__(self, name=name, tex_name=tex_name, info=info)
+        self.u = u
+        self.func = func
+        self.cache = cache
+        self._v = None
+        self._eval = False  # has been evaluated previously
+
+    @property
+    def v(self):
+        if not self._eval:
+            self._v = np.zeros_like(self.u.v, dtype=float)
+
+        if not self.cache or (not self._eval):
+            self._v[:] = self.func(self.u.v)
+
+        self._eval = True
+        return self._v
+
+
+class FlagCondition(BaseService):
+    """
+    Class for flagging values based on a condition function.
+
+    By default, values whose condition function output equal
+    that equal to True/1 will be flagged as `1`.
+    `0` otherwise.
+
+    Parameters
+    ----------
+    u
+        Input parameter
+    func
+        A condition function that returns True or False.
+    flag : 1 by default, only 0 or 1 is accepted.
+        The flag for the inputs whose condition output
+        is True.
+
+    Warnings
+    --------
+    This class is not ready.
+
+    `FlagCondition` can only be applied to `BaseParam` with `cache=True`.
+    Applying to `Service` will fail unless `cache` is False (at a performance cost).
+    """
+
+    def __init__(self, u, func, flag=1, name=None, tex_name=None, info=None, cache=True):
+        BaseService.__init__(self, name=name, tex_name=tex_name, info=info)
+        if flag != 0.0 and flag != 1.0:
+            raise ValueError(f"flag must be 0 or 1. The given flag = {flag}.")
+
+        self.u = u
+        self.func = func
+        self.flag = flag
+        self.flag_neg = 1 - flag
+        self.cache = cache
+
+        self._v = None
+        self._eval = False  # has been evaluated previously
+
+    @property
+    def v(self):
+        if not self._eval:
+            self._v = np.zeros_like(self.u.v, dtype=float)
+
+        if not self.cache or (not self._eval):
+            cond_out = self.func(self.u.v)
+
+            self._v[:] = np.array([self.flag if i == 1 else self.flag_neg
+                                   for i in cond_out])
+
+        self._eval = True
+        return self._v
+
+
+class FlagLessThan(FlagCondition):
+    """
+    Service for flagging parameters < or <= the given value element-wise.
+
+    Parameters that satisfy the comparison (u < or <= value) will flagged
+    as `flag` (1 by default).
+    """
+
+    def __init__(self, u, value=0.0, flag=1, equal=False,
+                 name=None, tex_name=None, info=None, cache=True):
+
+        self.value = dummify(value)
+        self.equal = equal
+
+        if self.equal is True:
+            self.func = lambda x: np.less_equal(x, self.value.v)
+        else:
+            self.func = lambda x: np.less(x, self.value.v)
+
+        FlagCondition.__init__(self, u, func=self.func,
+                               flag=flag, name=name,
+                               tex_name=tex_name, info=info, cache=cache,
+                               )
+
+
+class FlagGreaterThan(FlagCondition):
+    """
+    Service for flagging parameters > or >= the given value element-wise.
+
+    Parameters that satisfy the comparison (u > or >= value) will flagged
+    as `flag` (1 by default).
+    """
+
+    def __init__(self, u, value=0.0, flag=1, equal=False,
+                 name=None, tex_name=None, info=None, cache=True):
+
+        self.value = dummify(value)
+        self.equal = equal
+
+        if self.equal is True:
+            self.func = lambda x: np.greater_equal(x, self.value.v)
+        else:
+            self.func = lambda x: np.greater(x, self.value.v)
+
+        FlagCondition.__init__(self, u, func=self.func,
+                               flag=flag, name=name,
+                               tex_name=tex_name, info=info, cache=cache,
+                               )
+
+
+class CurrentSign(ConstService):
+    """
+    Service for computing the sign of the current flowing through a series device.
+
+    With a given line connecting `bus1` and `bus2`, one can compute the current
+    flow using ``(v1*exp(1j*a1) - v2*exp(1j*a2)) / (r + jx)`` whose value is
+    the outflow on `bus1`.
+
+    `CurrentSign` can be used to compute the sign to be multiplied depending on
+    the observing bus.
+    For each value in `bus`, the sign will be ``+1`` if it appears in `bus1` or
+    ``-1`` otherwise.
+
+    ::
+
+        bus1          bus2
+         *------>>-----*
+        bus(+)        bus(-)
+
+    """
+    def __init__(self, bus, bus1, bus2,  name=None, tex_name=None, info=None):
+        ConstService.__init__(self, v_numeric=self.check, name=name, tex_name=tex_name, info=info)
+        self.bus = bus
+        self.bus1 = bus1
+        self.bus2 = bus2
+
+    def check(self, **kwargs):
+        out = np.zeros_like(self.v)
+
+        for idx, (bus, bus1, bus2) in enumerate(zip(self.bus.v, self.bus1.v, self.bus2.v)):
+            if bus == bus1:
+                out[idx] = 1
+            elif bus == bus2:
+                out[idx] = -1
+            else:
+                raise ValueError(f"bus {bus} is terminal of the line connecting {bus1} and {bus2}. "
+                                 f"Check the data of {self.bus.owner.class_name}.{self.bus.name}")
+
+        return out
+
+
 class Replace(BaseService):
     """
     Replace parameters with new values if the function returns True
