@@ -1,3 +1,13 @@
+#  [ANDES] (C)2015-2020 Hantao Cui
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 3 of the License, or
+#  (at your option) any later version.
+#
+#  File name: var.py
+#  Last modified: 8/16/20, 7:27 PM
+
 from typing import Optional, Union, List
 from andes.core.param import BaseParam
 from andes.core.common import DummyValue
@@ -7,7 +17,7 @@ from andes.models.group import GroupBase
 from andes.shared import np, ndarray
 
 
-class BaseVar(object):
+class BaseVar:
     """
     Base variable class.
 
@@ -62,7 +72,7 @@ class BaseVar(object):
 
         self.tex_name = tex_name if tex_name else name
         self.owner = None  # instance of the owner Model
-        self.id = None     # variable internal index inside a model (assigned in run time) FIXME: not in use
+        self.id = None     # variable internal index inside a model (assigned in run time)
 
         self.v_str = v_str  # equation string (v = v_str) for variable initialization
         self.v_iter = v_iter  # the implicit equation (0 = v_iter) for iterative initialization
@@ -73,7 +83,7 @@ class BaseVar(object):
         self.e_setter = e_setter  # True if this var sets the equation value
         self.addressable = addressable  # True if this var needs to be assigned an address FIXME: not in use
         self.export = export            # True if this var's value needs to exported
-        self.diag_eps = diag_eps        # small diagonal value (1e-6) to be added to `dae.gy`
+        self.diag_eps = diag_eps        # small diagonal value to be added to `dae.gy`
 
         # attributes assigned by `set_address`
         self.n = 0
@@ -109,21 +119,22 @@ class BaseVar(object):
         self.v_inplace = False
 
     def __repr__(self):
-        span = []
-        if 1 <= self.n <= 20:
-            span = self.a.tolist()
-            span = ', '.join([str(i) for i in span])
-        elif self.n > 20:
+        if self.n == 0:
+            span = []
+
+        elif 1 <= self.n <= 20:
+            span = f'a={self.a}, v={self.v}, e={self.e}'
+
+        else:
+            span = []
             if not isinstance(self, ExtVar):
                 span.append(self.a[0])
                 span.append(self.a[-1])
                 span.append(self.a[1] - self.a[0])
                 span = ':'.join([str(i) for i in span])
+                span = 'a=[' + span + ']'
 
-        if span:
-            span = ' [' + span + ']'
-
-        return f'{self.__class__.__name__}, {self.owner.__class__.__name__}.{self.name}{span}'
+        return f'{self.__class__.__name__}: {self.owner.__class__.__name__}.{self.name}, {span}'
 
     def set_address(self, addr: ndarray, contiguous=False):
         """
@@ -228,7 +239,7 @@ class State(BaseVar):
                  v_iter: Optional[str] = None,
                  e_str: Optional[str] = None,
                  discrete: Optional[Discrete] = None,
-                 t_const: Optional[Union[BaseParam, DummyValue]] = None,
+                 t_const: Optional[Union[BaseParam, DummyValue, BaseService]] = None,
                  v_setter: Optional[bool] = False,
                  e_setter: Optional[bool] = False,
                  addressable: Optional[bool] = True,
@@ -442,3 +453,41 @@ class ExtAlgeb(ExtVar):
     """
     e_code = 'g'
     v_code = 'y'
+
+
+class AliasAlgeb(ExtAlgeb):
+    """
+    Alias algebraic variable. Essentially ``ExtAlgeb`` that links to a a model's
+    own variable.
+
+    ``AliasAlgeb`` is useful when the final output of a model is from a block, but
+    the model must provide the final output in a pre-defined name.
+    Using ``AliasAlgeb``, A model can avoid adding an additional variable with a dummy equations.
+
+    Like ``ExtVar``, labels of ``AliasAlgeb`` will not be saved in the final output.
+    When plotting from file, one need to look up the original variable name.
+    """
+    def __init__(self, var, **kwargs):
+        ExtAlgeb.__init__(self,
+                          model=var.owner.class_name,
+                          src=var.name,
+                          indexer=var.owner.idx,
+                          info=f'Alias of {var.name}',
+                          **kwargs,
+                          )
+
+
+class AliasState(ExtState):
+    """
+    Alias state variable.
+
+    Refer to the docs of ``AliasAlgeb``.
+    """
+    def __init__(self, var, **kwargs):
+        ExtState.__init__(self,
+                          model=var.owner.class_name,
+                          src=var.name,
+                          indexer=var.owner.idx,
+                          info=f'Alias of {var.name}',
+                          **kwargs,
+                          )
