@@ -143,6 +143,44 @@ def read(system, file):
     return ret
 
 
+def _read_dyr_dict(file):
+    """
+    Parse dyr file into a dict where keys are model names and values are dataframes.
+    """
+    with open(file, 'r') as f:
+        input_list = [line.strip() for line in f]
+
+    # concatenate multi-line device data
+    input_concat_dict = defaultdict(list)
+    multi_line = list()
+    for line in input_list:
+        if line == '':
+            continue
+        if '/' not in line:
+            multi_line.append(line)
+        else:
+            multi_line.append(line.split('/')[0])
+            single_line = ' '.join(multi_line)
+
+            if single_line.strip() == '':
+                continue
+
+            single_list = single_line.split("'")
+
+            psse_model = single_list[1].strip()
+            input_concat_dict[psse_model].append(single_list[0] + single_list[2])
+            multi_line = list()
+
+    # construct pandas dataframe for all models
+    dyr_dict = dict()   # input data from dyr file
+
+    for psse_model, all_rows in input_concat_dict.items():
+        dev_params_num = [([to_number(cell) for cell in row.split()]) for row in all_rows]
+        dyr_dict[psse_model] = pd.DataFrame(dev_params_num)
+
+    return dyr_dict
+
+
 def read_add(system, file):
     """
     Read an addition PSS/E dyr file.
@@ -159,32 +197,8 @@ def read_add(system, file):
     bool
         data parsing status
     """
-    with open(file, 'r') as f:
-        input_list = [line.strip() for line in f]
-
-    # concatenate multi-line device data
-    input_concat_dict = defaultdict(list)
-    multi_line = list()
-    for i, line in enumerate(input_list):
-        if line == '':
-            continue
-        if '/' not in line:
-            multi_line.append(line)
-        else:
-            multi_line.append(line.split('/')[0])
-            single_line = ' '.join(multi_line)
-            single_list = single_line.split("'")
-
-            psse_model = single_list[1].strip()
-            input_concat_dict[psse_model].append(single_list[0] + single_list[2])
-            multi_line = list()
-
-    # construct pandas dataframe for all models
-    dyr_dict = dict()   # input data from dyr file
-
-    for psse_model, all_rows in input_concat_dict.items():
-        dev_params_num = [([to_number(cell) for cell in row.split()]) for row in all_rows]
-        dyr_dict[psse_model] = pd.DataFrame(dev_params_num)
+    dyr_dict = _read_dyr_dict(file)
+    system.dyr_dict = dyr_dict
 
     # read yaml and set header for each pss/e model
     dirname = os.path.dirname(__file__)
@@ -209,7 +223,7 @@ def read_add(system, file):
         logger.debug(f'dyr contains models {", ".join(dyr_dict.keys())}')
 
     if len(not_supported):
-        logger.warning(f'Not supported: {", ".join(not_supported)}')
+        logger.warning(f'Models not yet supported: {", ".join(not_supported)}')
     else:
         logger.debug('All dyr models are supported.')
 
@@ -246,7 +260,7 @@ def read_add(system, file):
                         find[name] = system.__dict__[model].find_idx(cond_names, cond_values,
                                                                      allow_none=allow_none)
                     except IndexError as e:
-                        logger.error("Data file may contain references to unsupported devices.")
+                        logger.error("Data file likely contains references to unsupported models.")
                         logger.error(e)
                         return False
 
@@ -437,19 +451,51 @@ def _parse_transf_v33(raw, system, max_bus):
             # R1-2,X1-2,SBASE1-2
             # WINDV1,NOMV1,ANG1,RATA1,RATB1,RATC1,COD1,CONT1,RMA1,RMI1,VMA1,VMI1,NTP1,TAB1,CR1,CX1
             # WINDV2,NOMV2
+            #
             # """
 
-            tap = data[2][0]
-            phi = data[2][2]
+            Sn = system.config.mva
+            bus_Vn1 = system.Bus.get(src='Vn', idx=data[0][0], attr='v')
+            bus_Vn2 = system.Bus.get(src='Vn', idx=data[0][1], attr='v')
 
-            # set `transf` to True for all entries from `raw['transf']`
+            Vn1 = data[2][1] if data[2][1] != 0.0 else bus_Vn1
+            Vn2 = data[3][1] if data[3][1] != 0.0 else bus_Vn2
             transf = True
+            tap = data[2][0]  # pu or in kV
+            phi = data[2][2] * deg2rad  # `ANG1` is entered in degree; convert to rad
 
-            param = {'bus1': data[0][0], 'bus2': data[0][1], 'u': data[0][11],
-                     'b': data[0][8], 'r': data[1][0], 'x': data[1][1],
-                     'trans': transf, 'tap': tap, 'phi': phi,
-                     'Vn1': system.Bus.get(src='Vn', idx=data[0][0], attr='v'),
-                     'Vn2': system.Bus.get(src='Vn', idx=data[0][1], attr='v'),
+            # CW - Winding I/O code, 1-turn ratio on pu bus base kV, 2: winding V, 3: turn ratio pu on norn wind V
+            if data[0][4] == 1:
+                tap = tap
+            elif data[0][4] == 2:
+                tap = (data[2][0] / bus_Vn1) / (data[3][0] / bus_Vn2)
+            else:
+                tap = tap * (Vn1 / bus_Vn1) / (Vn2 / bus_Vn2)
+
+            # CZ - Z code, 1-system base, 2-winding base, 3-load loss and |z|
+            if data[0][5] == 1:
+                Sn = system.config.mva
+            elif data[0][5] == 2:
+                Sn = data[1][2]
+            else:
+                raise NotImplementedError('Impedance code 3 not implemented')
+
+            # CM - Y code, 1-system base, 2-No load loss and exc. loss
+            if data[0][6] == 2:
+                raise NotImplementedError('Admittance code 2 not implemented')
+
+            param = {'bus1': data[0][0],
+                     'bus2': data[0][1],
+                     'u': data[0][11],
+                     'b': data[0][8],
+                     'r': data[1][0],
+                     'x': data[1][1],
+                     'trans': transf,
+                     'tap': tap,
+                     'phi': phi,
+                     'Sn': Sn,
+                     'Vn1': Vn1,
+                     'Vn2': Vn2,
                      }
 
             out['Line'].append(param)
@@ -462,9 +508,15 @@ def _parse_transf_v33(raw, system, max_bus):
             # WINDV2, NOMV2, ANG2, RATA2, BATB2, RATC2, COD2, CONT2, RMA2, RMI2, VMA2, VMI2, NTP2, TAB2, CR2, CX2
             # WINDV3, NOMV3, ANG3, RATA3, BATB3, RATC3, COD3, CONT3, RMA3, RMI3, VMA3, VMI3, NTP3, TAB3, CR3, CX3
 
-            param = {'idx': max_bus + xf_3_count,
-                     'name': '_'.join(str(data[0][:3])),
-                     'Vn': system.Bus.get(src='Vn', idx=data[0][0], attr='v'),
+            new_bus = data[0][2] + 1
+
+            if new_bus in system.Bus.idx.v:
+                new_bus = max_bus + xf_3_count
+                logger.warning(f'{new_bus} exists.')
+
+            param = {'idx': new_bus,
+                     'name': '_'.join([str(i) for i in data[0][:3]]),
+                     'Vn': 1.0,
                      'v0': data[1][-2],
                      'a0': data[1][-1] * deg2rad
                      }
@@ -482,13 +534,16 @@ def _parse_transf_v33(raw, system, max_bus):
 
             for i in range(0, 3):
                 param = {'trans': True,
-                         'bus1': data[0][i], 'bus2': max_bus + xf_3_count,
+                         'bus1': data[0][i],
+                         'bus2': new_bus,
                          'u': data[0][11],
-                         'b': data[0][8], 'r': r[i], 'x': x[i],
+                         'b': data[0][8],
+                         'r': r[i],
+                         'x': x[i],
                          'tap': data[2+i][0],
-                         'phi': data[2+i][2],
+                         'phi': data[2+i][2] * deg2rad,
                          'Vn1': system.Bus.get(src='Vn', idx=data[0][i], attr='v'),
-                         'Vn2': system.Bus.get(src='Vn', idx=data[0][0], attr='v'),
+                         'Vn2': 1.0,
                          }
 
                 out['Line'].append(param)
@@ -563,10 +618,10 @@ def sort_psse_models(dyr_yaml):
     """
     Sort supported models so that model names are ordered by dependency.
     """
-    from andes.models import non_jit
+    from andes.models import file_classes
     from andes.utils.func import list_flatten
 
-    andes_models = list_flatten(list(non_jit.values()))
+    andes_models = list_flatten(list(file_classes.values()))
     number = dict()
 
     for psse_model in dyr_yaml:
