@@ -1,12 +1,15 @@
 import logging
+import numpy as np
+from collections import OrderedDict
+from andes.core.service import BackRef
+from typing import Sized
+
 from andes.utils.func import list_flatten
-from andes.shared import np
 
 logger = logging.getLogger(__name__)
 
 
-# TODO: offload metadata to YAML file and create classes on the fly.
-class GroupBase(object):
+class GroupBase:
     """
     Base class for groups
     """
@@ -15,8 +18,23 @@ class GroupBase(object):
         self.common_params = ['u', 'name']
         self.common_vars = []
 
-        self.models = {}  # model name, model instance
-        self._idx2model = {}  # element idx, model name
+        self.models = OrderedDict()        # model name: model instance
+        self._idx2model = OrderedDict()    # element idx: model instance
+        self.uid = {}                      # idx - group internal 0-indexed uid
+        self.services_ref = OrderedDict()  # BackRef
+
+    def __setattr__(self, key, value):
+        if hasattr(value, 'owner'):
+            if value.owner is None:
+                value.owner = self
+        if hasattr(value, 'name'):
+            if value.name is None:
+                value.name = key
+
+        if isinstance(value, BackRef):
+            self.services_ref[key] = value
+
+        super().__setattr__(key, value)
 
     @property
     def class_name(self):
@@ -24,14 +42,30 @@ class GroupBase(object):
 
     @property
     def n(self):
-        """Total number of devices"""
+        """
+        Total number of devices.
+        """
         return len(self._idx2model)
 
-    def add_model(self, name, instance):
+    def add_model(self, name: str, instance):
+        """
+        Add a Model instance to group.
+
+        Parameters
+        ----------
+        name : str
+            Model name
+        instance : Model
+            Model instance
+
+        Returns
+        -------
+        None
+        """
         if name not in self.models:
             self.models[name] = instance
         else:
-            raise KeyError(f"Duplicate model registration if {name}")
+            raise KeyError(f"{self.class_name}: Duplicate model registration of {name}")
 
     def add(self, idx, model):
         """
@@ -52,20 +86,32 @@ class GroupBase(object):
         if idx in self._idx2model:
             raise KeyError(f'Group <{self.class_name}> already contains <{repr(idx)}> from '
                            f'<{self._idx2model[idx].class_name}>')
+        self.uid[idx] = self.n
         self._idx2model[idx] = model
 
     def idx2model(self, idx, allow_none=False):
         """
         Find model name for the given idx.
 
-        If `allow_none` is True, will return None at the corr. position.
+        Parameters
+        ----------
+        idx : float, int, str, array-like
+            idx or idx-es of devices.
+        allow_none : bool
+           If True, return `None` at the positions where idx is not found.
+
+        Returns
+        -------
+        If `idx` is a list, return a list of model instances.
+        If `idx` is a single element, return a model instance.
         """
 
         ret = []
         single = False
+
         if not isinstance(idx, (list, tuple, np.ndarray)):
             single = True
-            idx = (idx, )
+            idx = (idx,)
         elif len(idx) > 0 and isinstance(idx[0], (list, tuple, np.ndarray)):
             idx = list_flatten(idx)
 
@@ -82,6 +128,33 @@ class GroupBase(object):
             ret = ret[0]
         return ret
 
+    def idx2uid(self, idx):
+        """
+        Convert idx to the 0-indexed unique index.
+
+        Parameters
+        ----------
+        idx : array-like, numbers, or str
+            idx of devices
+
+        Returns
+        -------
+        list
+            A list containing the unique indices of the devices
+        """
+        if idx is None:
+            logger.debug("idx2uid returned None for idx None")
+            return None
+        if isinstance(idx, (float, int, str, np.int32, np.int64, np.float64)):
+            return self.uid[idx]
+        elif isinstance(idx, Sized):
+            if len(idx) > 0 and isinstance(idx[0], (list, np.ndarray)):
+                idx = list_flatten(idx)
+            return [self.uid[i] if i is not None else None
+                    for i in idx]
+        else:
+            raise NotImplementedError(f'Unknown idx type {type(idx)}')
+
     def get(self, src: str, idx, attr: str = 'v', allow_none=False, default=0.0):
         """
         Based on the indexer, get the `attr` field of the `src` parameter or variable.
@@ -101,17 +174,24 @@ class GroupBase(object):
 
         Returns
         -------
-        The requested param or variable attribute
+        The requested param or variable attribute. If `idx` is a list, return a list of values.
+        If `idx` is a single element, return a single value.
         """
         self._check_src(src)
         self._check_idx(idx)
 
+        single = False
+        if not isinstance(idx, (list, np.ndarray)):
+            idx = [idx]
+            single = True
+
         n = len(idx)
         if n == 0:
             return np.zeros(0)
 
         ret = [''] * n
         _type_set = False
+
         models = self.idx2model(idx, allow_none=allow_none)
 
         for i, idx in enumerate(idx):
@@ -132,22 +212,55 @@ class GroupBase(object):
 
             ret[i] = val
 
+        if single:
+            ret = ret[0]
+
         return ret
 
     def set(self, src: str, idx, attr, value):
+        """
+        Set the value of an attribute of a group property.
+        Performs ``self.<src>.<attr>[idx] = value``.
+
+        The user needs to ensure that the property is shared by all models
+        in this group.
+
+        Parameters
+        ----------
+        src : str
+            Name of property.
+        idx : str, int, float, array-like
+            Indices of devices.
+        attr : str, optional, default='v'
+            The internal attribute of the property to get.
+            ``v`` for values, ``a`` for address, and ``e`` for equation value.
+        value : array-like
+            New values to be set
+
+        Returns
+        -------
+        bool
+            True when successful.
+        """
         self._check_src(src)
         self._check_idx(idx)
 
+        if not isinstance(idx, (list, np.ndarray)):
+            idx = [idx]
+
         if isinstance(value, (float, str, int)):
             value = [value] * len(idx)
 
         models = self.idx2model(idx)
+
         for i, idx in enumerate(idx):
             model = models[i]
             uid = model.idx2uid(idx)
             instance = model.__dict__[src]
             instance.__dict__[attr][uid] = value[i]
 
+        return True
+
     def find_idx(self, keys, values, allow_none=False, default=None):
         """
         Find indices of devices that satisfy the given `key=value` condition.
@@ -176,7 +289,9 @@ class GroupBase(object):
 
     def _check_src(self, src: str):
         if src not in self.common_vars + self.common_params:
-            raise AttributeError(f'Group <{self.class_name}> does not share property <{src}>.')
+            # raise AttributeError(f'Group <{self.class_name}> does not share property <{src}>.')
+            logger.debug(f'Group <{self.class_name}> does not share property <{src}>.')
+            pass
 
     def _check_idx(self, idx):
         if idx is None:
@@ -184,16 +299,21 @@ class GroupBase(object):
 
     def get_next_idx(self, idx=None, model_name=None):
         """
-        Return the auto-generated next idx
+        Get a no-conflict idx for a new device.
+        Use the provided ``idx`` if no conflict.
+        Generate a new one otherwise.
 
         Parameters
         ----------
-        idx
-
-        model_name
+        idx : str or None
+            Proposed idx. If None, assign a new one.
+        model_name : str or None
+            Model name. If not, prepend the group name.
 
         Returns
         -------
+        str
+            New device name.
 
         """
         if model_name is None:
@@ -226,6 +346,9 @@ class GroupBase(object):
         return idx
 
     def doc(self, export='plain'):
+        """
+        Return the documentation of the group in a string.
+        """
         out = ''
         if export == 'rest':
             out += f'.. _{self.class_name}:\n\n'
@@ -254,6 +377,7 @@ class GroupBase(object):
             if export == 'rest':
                 def add_reference(name_list):
                     return [f'{item}_' for item in name_list]
+
                 model_name_list = add_reference(model_name_list)
 
             out += ',\n'.join(model_name_list) + '\n'
@@ -261,6 +385,19 @@ class GroupBase(object):
         return out
 
     def doc_all(self, export='plain'):
+        """
+        Return documentation of the group and its models.
+
+        Parameters
+        ----------
+        export : 'plain' or 'rest'
+            Export format, plain-text or RestructuredText
+
+        Returns
+        -------
+        str
+
+        """
         out = self.doc(export=export)
         out += '\n'
         for instance in self.models.values():
@@ -270,6 +407,9 @@ class GroupBase(object):
 
 
 class Undefined(GroupBase):
+    """
+    The undefined group. Holds models with no ``group``.
+    """
     pass
 
 
@@ -299,14 +439,20 @@ class StaticGen(GroupBase):
     """
     Static generator group for power flow calculation
     """
+
     def __init__(self):
         super().__init__()
         self.common_params.extend(('Sn', 'Vn', 'p0', 'q0', 'ra', 'xs', 'subidx'))
         self.common_vars.extend(('p', 'q', 'a', 'v'))
 
+        self.SynGen = BackRef()
+
 
 class ACLine(GroupBase):
-    pass
+    def __init__(self):
+        super(ACLine, self).__init__()
+        self.common_params.extend(('bus1', 'bus2', 'r', 'x'))
+        self.common_vars.extend(('v1', 'v2', 'a1', 'a2'))
 
 
 class StaticLoad(GroupBase):
@@ -327,17 +473,96 @@ class SynGen(GroupBase):
     """
     Synchronous generator group.
     """
+
     def __init__(self):
         super().__init__()
         self.common_params.extend(('Sn', 'Vn', 'fn', 'bus', 'M', 'D'))
         self.common_vars.extend(('omega', 'delta', 'tm', 'te', 'vf', 'XadIfd', 'vd', 'vq', 'Id', 'Iq',
                                  'a', 'v'))
 
+        self.TurbineGov = BackRef()
+        self.Exciter = BackRef()
+
+
+class RenGen(GroupBase):
+    """
+    Renewable generator (converter) group.
+    """
+
+    def __init__(self):
+        super().__init__()
+        self.common_params.extend(('bus', 'gen'))
+        self.common_vars.extend(('Ipcmd', 'Iqcmd', 'Pe', 'Qe'))
+
+
+class RenExciter(GroupBase):
+    """
+    Renewable electrical control (exciter) group.
+    """
+    def __init__(self):
+        super().__init__()
+        self.common_params.extend(('reg',))
+        self.common_vars.extend(('Pref', 'Qref', 'wg', 'Pord'))
+
+
+class RenPlant(GroupBase):
+    """
+    Renewable plant control group.
+    """
+    def __init__(self):
+        super().__init__()
+
+
+class RenGovernor(GroupBase):
+    """
+    Renewable turbine governor group.
+    """
+    def __init__(self):
+        super().__init__()
+        self.common_params.extend(('ree', 'w0', 'Sn', 'Pe0'))
+        self.common_vars.extend(('Pm', 'wr0', 'wt', 'wg', 's3_y'))
+
+
+class RenAerodynamics(GroupBase):
+    """
+    Renewable aerodynamics group.
+    """
+    def __init__(self):
+        super().__init__()
+        self.common_params.extend(('rego',))
+        self.common_vars.extend(('theta',))
+
+
+class RenPitch(GroupBase):
+    """
+    Renewable generator pitch controller group.
+    """
+    def __init__(self):
+        super().__init__()
+        self.common_params.extend(('rea',))
+
+
+class RenTorque(GroupBase):
+    """
+    Renewable torque (Pref) controller.
+    """
+    def __init__(self):
+        super().__init__()
+
+
+class DG(GroupBase):
+    """
+    Distributed generation (small-scale).
+    """
+    def __init__(self):
+        super().__init__()
+
 
 class TurbineGov(GroupBase):
     """
     Turbine governor group for synchronous generator.
     """
+
     def __init__(self):
         super().__init__()
         self.common_vars.extend(('pout',))
@@ -347,21 +572,25 @@ class Exciter(GroupBase):
     """
     Exciter group for synchronous generators.
     """
+
     def __init__(self):
         super().__init__()
-        self.common_params.extend(('syn', ))
-        self.common_vars.extend(('vout', 'vi', ))
+        self.common_params.extend(('syn',))
+        self.common_vars.extend(('vout', 'vi',))
+
+        self.PSS = BackRef()
 
 
 class PSS(GroupBase):
     """Power system stabilizer group."""
+
     def __init__(self):
         super().__init__()
-        self.common_vars.extend(('vsout', ))
+        self.common_vars.extend(('vsout',))
 
 
 class Experimental(GroupBase):
-    """Experimantal group"""
+    """Experimental group"""
     pass
 
 
@@ -382,6 +611,22 @@ class TimedEvent(GroupBase):
 
 class FreqMeasurement(GroupBase):
     """Frequency measurements."""
+
+    def __init__(self):
+        super().__init__()
+        self.common_vars.extend(('f',))
+
+
+class PhasorMeasurement(GroupBase):
+    """Phasor measurements"""
+
+    def __init__(self):
+        super().__init__()
+        self.common_vars.extend(('am', 'vm'))
+
+
+class Motor(GroupBase):
+    """Induction Motor group
+    """
     def __init__(self):
         super().__init__()
-        self.common_vars.extend(('f', ))
