@@ -178,12 +178,18 @@ class PQ(PQData, Model):
                             enable=self.config.pq2z,
                             )
 
-        self.a.e_str = "u * ((dae_t <= 0) | (p2p == 1)) * " \
+        # Note: the "or" condition "|" is not supported in sympy equation strings.
+        # They will simply be ignored.
+
+        # To modify P and Q during TDS, use `alter` to set values to `Ppf` and `Qpf`
+        # after, before simulation, setting `config.p2p=1` and `config.q2q=1`.
+
+        self.a.e_str = "u * (dae_t <= 0) * " \
                        "(p0 * vcmp_zi + Rlb * vcmp_zl * v**2 + Rub * vcmp_zu * v**2) + " \
-                       "u * ((dae_t > 0) & (p2p != 1)) * " \
+                       "u * (dae_t > 0) * " \
                        "(p2p * Ppf + p2i * Ipeq * v + p2z * Req * v**2)"
 
-        self.v.e_str = "u * ((dae_t <= 0) | (q2q == 1)) * " \
+        self.v.e_str = "u * (dae_t <= 0) * " \
                        "(q0 * vcmp_zi + Xlb * vcmp_zl * v**2 + Xub * vcmp_zu * v**2) + " \
-                       "u * ((dae_t > 0) & (q2q != 1)) * " \
+                       "u * (dae_t > 0) * " \
                        "(q2q * Qpf + q2i * Iqeq * v + q2z * Xeq * v**2)"
