@@ -3,6 +3,7 @@ from collections import OrderedDict
 from andes.core.model import Model, ModelData
 from andes.core.param import NumParam, IdxParam, DataParam
 from andes.core.var import Algeb, ExtAlgeb
+from andes.core.service import BackRef
 from andes.core.discrete import SortedLimiter
 
 logger = logging.getLogger(__name__)
@@ -67,14 +68,21 @@ class PVModel(Model):
                               npv2pq="n_{pv2pq}"
                               )
 
+        self.SynGen = BackRef()
+
         self.a = ExtAlgeb(model='Bus', src='a', indexer=self.bus, tex_name=r'\theta')
         self.v = ExtAlgeb(model='Bus', src='v', indexer=self.bus, v_setter=True, tex_name=r'V')
 
-        self.p = Algeb(info='actual active power generation', unit='p.u.', tex_name=r'p', diag_eps=1e-6)
-        self.q = Algeb(info='actual reactive power generation', unit='p.u.', tex_name='q', diag_eps=1e-6)
+        self.p = Algeb(info='actual active power generation', unit='p.u.', tex_name=r'p', diag_eps=True)
+        self.q = Algeb(info='actual reactive power generation', unit='p.u.', tex_name='q', diag_eps=True)
+
+        # NOTE:
+        # PV to PQ conversion uses a simple logic which converts a number of `npv2pq` PVs
+        # with the largest violation to PQ, starting from the first iteration.
+        # Manual tweaking of `npv2pq` may be needed for cases to converge.
 
         # TODO: implement switching starting from the second iteration
-        # NOTE: TODO: PV to PQ conversion might not be fully working
+
         self.qlim = SortedLimiter(u=self.q, lower=self.qmin, upper=self.qmax,
                                   enable=self.config.pv2pq,
                                   n_select=self.config.npv2pq)
