@@ -7,8 +7,9 @@ from andes.models.exciter import ExcQuadSat
 from andes.core.param import IdxParam, NumParam, ExtParam
 from andes.core.var import Algeb, State, ExtAlgeb
 from andes.core.discrete import LessThan
-from andes.core.service import ConstService, ExtService  # NOQA
-from andes.core.service import InitChecker, FlagNotNone
+from andes.core.service import ConstService, VarService, ExtService  # NOQA
+from andes.core.service import InitChecker, FlagValue
+import numpy as np
 
 logger = logging.getLogger(__name__)
 
@@ -21,6 +22,7 @@ class GENBaseData(ModelData):
                             mandatory=True,
                             )
         self.gen = IdxParam(info="static generator index",
+                            model='StaticGen',
                             mandatory=True,
                             )
         self.coi = IdxParam(model='COI',
@@ -129,24 +131,24 @@ class GENBase(Model):
         # algebraic variables
         # Need to be provided by specific generator models
         self.Id = Algeb(info='d-axis current',
-                        v_str='Id0',
+                        v_str='u * Id0',
                         tex_name=r'I_d',
                         e_str=''
                         )  # to be completed by subclasses
         self.Iq = Algeb(info='q-axis current',
-                        v_str='Iq0',
+                        v_str='u * Iq0',
                         tex_name=r'I_q',
                         e_str=''
                         )  # to be completed
 
         self.vd = Algeb(info='d-axis voltage',
-                        v_str='vd0',
-                        e_str='v * sin(delta - a) - vd',
+                        v_str='u * vd0',
+                        e_str='u * v * sin(delta - a) - vd',
                         tex_name=r'V_d',
                         )
         self.vq = Algeb(info='q-axis voltage',
-                        v_str='vq0',
-                        e_str='v * cos(delta - a) - vq',
+                        v_str='u * vq0',
+                        e_str='u * v * cos(delta - a) - vq',
                         tex_name=r'V_q',
                         )
 
@@ -157,13 +159,13 @@ class GENBase(Model):
                         )
         self.te = Algeb(info='electric torque',
                         tex_name=r'\tau_e',
-                        v_str='p0',
-                        e_str='psid * Iq - psiq * Id - te',
+                        v_str='u * p0',
+                        e_str='u * (psid * Iq - psiq * Id) - te',
                         )
         self.vf = Algeb(info='excitation voltage',
                         unit='pu',
-                        v_str='vf0',
-                        e_str='vf0 - vf',
+                        v_str='u * vf0',
+                        e_str='u * vf0 - vf',
                         tex_name=r'v_f'
                         )
 
@@ -176,8 +178,8 @@ class GENBase(Model):
         self.XadIfd = Algeb(tex_name='X_{ad}I_{fd}',
                             info='d-axis armature excitation current',
                             unit='p.u (kV)',
-                            v_str='vf0',
-                            e_str='vf0 - XadIfd'
+                            v_str='u * vf0',
+                            e_str='u * vf0 - XadIfd'
                             )  # e_str to be provided. Not available in GENCLS
 
         self.subidx = ExtParam(model='StaticGen',
@@ -214,7 +216,7 @@ class GENBase(Model):
         self.system.groups['StaticGen'].set(src='u', idx=self.gen.v, attr='v', value=0)
 
 
-class Flux0(object):
+class Flux0:
     """
     Flux model without electro-magnetic transients and ignore speed deviation
     """
@@ -222,12 +224,12 @@ class Flux0(object):
     def __init__(self):
         self.psid = Algeb(info='d-axis flux',
                           tex_name=r'\psi_d',
-                          v_str='psid0',
+                          v_str='u * psid0',
                           e_str='u * (ra*Iq + vq) - psid',
                           )
         self.psiq = Algeb(info='q-axis flux',
                           tex_name=r'\psi_q',
-                          v_str='psiq0',
+                          v_str='u * psiq0',
                           e_str='u * (ra*Id + vd) + psiq',
                           )
 
@@ -235,7 +237,7 @@ class Flux0(object):
         self.Iq.e_str += '+ psiq'
 
 
-class Flux1(object):
+class Flux1:
     """
     Flux model without electro-magnetic transients but considers speed deviation.
     """
@@ -243,12 +245,12 @@ class Flux1(object):
     def __init__(self):
         self.psid = Algeb(info='d-axis flux',
                           tex_name=r'\psi_d',
-                          v_str='psid0',
+                          v_str='u * psid0',
                           e_str='u * (ra * Iq + vq) - omega * psid',
                           )
         self.psiq = Algeb(info='q-axis flux',
                           tex_name=r'\psi_q',
-                          v_str='psiq0',
+                          v_str='u * psiq0',
                           e_str='u * (ra * Id + vd) + omega * psiq',
                           )
 
@@ -256,7 +258,7 @@ class Flux1(object):
         self.Iq.e_str += '+ psiq'
 
 
-class Flux2(object):
+class Flux2:
     """
     Flux model with electro-magnetic transients.
     """
@@ -264,12 +266,12 @@ class Flux2(object):
     def __init__(self):
         self.psid = State(info='d-axis flux',
                           tex_name=r'\psi_d',
-                          v_str='psid0',
+                          v_str='u * psid0',
                           e_str='u * 2 * pi * fn * (ra * Id + vd + omega * psiq)',
                           )
         self.psiq = State(info='q-axis flux',
                           tex_name=r'\psi_q',
-                          v_str='psiq0',
+                          v_str='u * psiq0',
                           e_str='u * 2 * pi * fn * (ra * Iq + vq - omega * psid)',
                           )
 
@@ -277,28 +279,31 @@ class Flux2(object):
         self.Iq.e_str += '+ psiq'
 
 
-class GENCLSModel(object):
+class GENCLSModel:
     def __init__(self):
         # internal voltage and rotor angle calculation
         self.xq = ExtService(model='GENCLS', src='xd1', indexer=self.idx,
                              )
         self._V = ConstService(v_str='v * exp(1j * a)',
                                tex_name='V_c',
+                               vtype=np.complex,
                                )
         self._S = ConstService(v_str='p0 - 1j * q0',
                                tex_name='S',
+                               vtype=np.complex,
                                )
         self._I = ConstService(v_str='_S / conj(_V)',
                                tex_name='I_c',
+                               vtype=np.complex,
                                )
-        self._E = ConstService(tex_name='E')
-        self._deltac = ConstService(tex_name=r'\delta_c')
+        self._E = ConstService(tex_name='E', vtype=np.complex)
+        self._deltac = ConstService(tex_name=r'\delta_c', vtype=np.complex)
         self.delta0 = ConstService(tex_name=r'\delta_0')
 
         self.vdq = ConstService(v_str='u * (_V * exp(1j * 0.5 * pi - _deltac))',
-                                tex_name='V_{dq}')
+                                tex_name='V_{dq}', vtype=np.complex)
         self.Idq = ConstService(v_str='u * (_I * exp(1j * 0.5 * pi - _deltac))',
-                                tex_name='I_{dq}')
+                                tex_name='I_{dq}', vtype=np.complex)
 
         self.Id0 = ConstService(v_str='re(Idq)',
                                 tex_name=r'I_{d0}')
@@ -363,7 +368,7 @@ class GENROUData(GENBaseData):
                              tex_name=r"T''_{q0}")
 
 
-class GENROUModel(object):
+class GENROUModel:
     def __init__(self):
         # parameter checking for `xl`
         self._xlc = InitChecker(u=self.xl,
@@ -383,7 +388,7 @@ class GENROUModel(object):
                                 tex_name=r"\gamma_{qd}")
 
         # correct S12 to 1.0 if is zero
-        self._fS12 = FlagNotNone(self.S12, to_flag=0)
+        self._fS12 = FlagValue(self.S12, value=0)
         self._S12 = ConstService(v_str='S12 + (1-_fS12)',
                                  info='Corrected S12',
                                  tex_name='S_{1.2}'
@@ -397,14 +402,20 @@ class GENROUModel(object):
         #   https://github.com/OpenIPSL/OpenIPSL/blob/master/OpenIPSL/Electrical/Machines/PSSE/GENROU.mo
 
         # internal voltage and rotor angle calculation
-        self._V = ConstService(v_str='v * exp(1j * a)', tex_name='V_c', info='complex bus voltage')
-        self._S = ConstService(v_str='p0 - 1j * q0', tex_name='S', info='complex terminal power')
-        self._Zs = ConstService(v_str='ra + 1j * xd2', tex_name='Z_s', info='equivalent impedance')
-        self._It = ConstService(v_str='_S / conj(_V)', tex_name='I_t', info='complex terminal current')
-        self._Is = ConstService(tex_name='I_s', v_str='_It + _V / _Zs', info='equivalent current source')
+        self._V = ConstService(v_str='v * exp(1j * a)', tex_name='V_c', info='complex bus voltage',
+                               vtype=np.complex)
+        self._S = ConstService(v_str='p0 - 1j * q0', tex_name='S', info='complex terminal power',
+                               vtype=np.complex)
+        self._Zs = ConstService(v_str='ra + 1j * xd2', tex_name='Z_s', info='equivalent impedance',
+                                vtype=np.complex)
+        self._It = ConstService(v_str='_S / conj(_V)', tex_name='I_t', info='complex terminal current',
+                                vtype=np.complex)
+        self._Is = ConstService(tex_name='I_s', v_str='_It + _V / _Zs', info='equivalent current source',
+                                vtype=np.complex)
 
         self.psi20 = ConstService(tex_name=r"\psi''_0", v_str='_Is * _Zs',
-                                  info='sub-transient flux linkage in stator reference')
+                                  info='sub-transient flux linkage in stator reference',
+                                  vtype=np.complex)
         self.psi20_arg = ConstService(tex_name=r"\theta_{\psi''0}", v_str='arg(psi20)')
         self.psi20_abs = ConstService(tex_name=r"|\psi''_0|", v_str='abs(psi20)')
         self._It_arg = ConstService(tex_name=r"\theta_{It0}", v_str='arg(_It)')
@@ -414,18 +425,21 @@ class GENROUModel(object):
         self.Se0 = ConstService(tex_name=r"S_{e0}",
                                 v_str='(psi20_abs>=SAT_A) * (psi20_abs - SAT_A) ** 2 * SAT_B / psi20_abs')
 
-        self._a = ConstService(tex_name=r"a", v_str='psi20_abs * (1 + Se0*gqd)')
-        self._b = ConstService(tex_name=r"b", v_str='abs(_It) * (xq2 - xq)')  # xd2 == xq2
+        self._a = ConstService(tex_name=r"a'", v_str='psi20_abs * (1 + Se0*gqd)')
+        self._b = ConstService(tex_name=r"b'", v_str='abs(_It) * (xq2 - xq)')  # xd2 == xq2
 
         self.delta0 = ConstService(tex_name=r'\delta_0',
                                    v_str='atan(_b * cos(_psi20_It_arg) / (_b * sin(_psi20_It_arg) - _a)) + '
                                          'psi20_arg')
         self._Tdq = ConstService(tex_name=r"T_{dq}",
-                                 v_str='cos(delta0) - 1j * sin(delta0)')
+                                 v_str='cos(delta0) - 1j * sin(delta0)',
+                                 vtype=np.complex)
         self.psi20_dq = ConstService(tex_name=r"\psi''_{0,dq}",
-                                     v_str='psi20 * _Tdq')
+                                     v_str='psi20 * _Tdq',
+                                     vtype=np.complex)
         self.It_dq = ConstService(tex_name=r"I_{t,dq}",
-                                  v_str='conj(_It * _Tdq)')
+                                  v_str='conj(_It * _Tdq)',
+                                  vtype=np.complex)
 
         self.psi2d0 = ConstService(tex_name=r"\psi_{ad0}",
                                    v_str='re(psi20_dq)')
@@ -467,22 +481,26 @@ class GENROUModel(object):
                            )
 
         self.psi2d = Algeb(tex_name=r"\psi_{ad}", info='d-axis air gap flux',
-                           v_str='psi2d0',
+                           v_str='u * psi2d0',
                            e_str='gd1*e1q + gd2*(xd1-xl)*e2d - psi2d')
 
         self.psi2 = Algeb(tex_name=r"\psi_a", info='air gap flux magnitude',
-                          v_str='abs(psi20_dq)',
-                          e_str='sqrt(psi2d **2 + psi2q ** 2) - psi2')
+                          v_str='u * abs(psi20_dq)',
+                          e_str='psi2d **2 + psi2q ** 2 - psi2 ** 2',
+                          diag_eps=True,
+                          )
 
         # `LT` is a reserved keyword for SymPy
         self.SL = LessThan(u=self.psi2, bound=self.SAT_A, equal=False, enable=True, cache=False)
 
         self.Se = Algeb(tex_name=r"S_e(|\psi_{a}|)", info='saturation output',
-                        v_str='Se0',
-                        e_str='SL_z0 * (psi2 - SAT_A) ** 2 * SAT_B / psi2 - Se')
+                        v_str='u * Se0',
+                        e_str='SL_z0 * (psi2 - SAT_A) ** 2 * SAT_B - psi2 * Se',
+                        diag_eps=True,
+                        )
 
         # separated `XadIfd` from `e1q` using \dot(e1q) = (vf - XadIfd) / Td10
-        self.XadIfd.e_str = 'e1q + (xd-xd1) * (gd1*Id - gd2*e2d + gd2*e1q) + Se*psi2d - XadIfd'
+        self.XadIfd.e_str = 'u * (e1q + (xd-xd1) * (gd1*Id - gd2*e2d + gd2*e1q) + Se*psi2d) - XadIfd'
 
         # `XadI1q` can also be given in `(xq-xq1)*gq2*(e1d-e2q+(xq1-xl)*Iq) + e1d - Iq*(xq-xq1) + Se*psi2q*gqd`
         self.XaqI1q =\
@@ -495,21 +513,31 @@ class GENROUModel(object):
 
         self.e1q = State(info='q-axis transient voltage',
                          tex_name=r"e'_q",
-                         v_str='e1q0',
-                         e_str='(-XadIfd + vf) / Td10')
+                         v_str='u * e1q0',
+                         e_str='(-XadIfd + vf)',
+                         t_const=self.Td10,
+                         )
+
         self.e1d = State(info='d-axis transient voltage',
                          tex_name=r"e'_d",
                          v_str='e1d0',
-                         e_str='-XaqI1q / Tq10')
+                         e_str='-XaqI1q',
+                         t_const=self.Tq10,
+                         )
+
         self.e2d = State(info='d-axis sub-transient voltage',
                          tex_name=r"e''_d",
-                         v_str='e2d0',
-                         e_str='(-e2d + e1q - (xd1 - xl) * Id) / Td20')
+                         v_str='u * e2d0',
+                         e_str='(-e2d + e1q - (xd1 - xl) * Id)',
+                         t_const=self.Td20,
+                         )
 
         self.e2q = State(info='q-axis sub-transient voltage',
                          tex_name=r"e''_q",
                          v_str='e2q0',
-                         e_str='(-e2q + e1d + (xq1 - xl) * Iq) / Tq20')
+                         e_str='(-e2q + e1d + (xq1 - xl) * Iq)',
+                         t_const=self.Tq20,
+                         )
 
         self.Iq.e_str += '+ xq2*Iq + psi2q'
 
