@@ -3,6 +3,7 @@ from andes.core.model import Model, ModelData
 from andes.core.var import ExtAlgeb
 from andes.core.service import ConstService
 from andes.shared import np, tqdm
+from collections import OrderedDict
 import logging
 logger = logging.getLogger(__name__)
 
@@ -26,6 +27,22 @@ class Toggler(TogglerData, Model):
         self.group = 'TimedEvent'
 
         self.t.callback = self._u_switch
+        self._init = False   # very first initialization that stores `u`
+        self._u = ConstService('1')
+
+    def v_numeric(self, **kwargs):
+        """
+        Custom initialization function that stores and restores the connectivity status.
+        """
+        if not self._init:
+            for i in range(self.n):
+                instance = self.system.__dict__[self.model.v[i]]
+                self._u.v[i] = instance.get(src='u', attr='v', idx=self.dev.v[i])
+                self._init = True
+        else:
+            for i in range(self.n):
+                instance = self.system.__dict__[self.model.v[i]]
+                instance.set(src='u', attr='v', idx=self.dev.v[i], value=self._u.v[i])
 
     def _u_switch(self, is_time: np.ndarray):
         action = False
@@ -36,7 +53,7 @@ class Toggler(TogglerData, Model):
                 instance.set(src='u', attr='v', idx=self.dev.v[i], value=1-u0)
                 action = True
                 tqdm.write(f'<Toggler {self.idx.v[i]}>: '
-                           f'{self.model.v[i]}.{self.dev.v[i]} status changed to {1-u0} at t={self.t.v[i]} sec.')
+                           f'{self.model.v[i]}.{self.dev.v[i]} status changed to {1-u0:g} at t={self.t.v[i]} sec.')
         return action
 
 
@@ -73,11 +90,25 @@ class Fault(ModelData, Model):
         Model.__init__(self, system, config)
         self.flags.update({'tds': True})
         self.group = 'TimedEvent'
+
+        self.config.add(OrderedDict((('restore', 1),
+                                     ('scale', 1.0)
+                                     )))
+        self.config.add_extra('_alt',
+                              restore=(0, 1),
+                              )
+        self.config.add_extra('_help',
+                              restore='restore algebraic variables to pre-fault values',
+                              scale='scaling factor of restored algebraic values',
+                              )
+
         self.gf = ConstService(tex_name='g_{f}',
                                v_str='re(1/(rf + 1j * xf))',
+                               vtype=np.complex,
                                )
         self.bf = ConstService(tex_name='b_{f}',
                                v_str='im(1/(rf + 1j * xf))',
+                               vtype=np.complex,
                                )
 
         # uf: an internal flag of whether the fault is in action (1) or not (0)
@@ -110,8 +141,10 @@ class Fault(ModelData, Model):
             if is_time[i] and (self.u.v[i] == 1):
                 self.uf.v[i] = 1
                 self._vstore = np.array(self.system.dae.y[self.system.Bus.n:])
+
                 tqdm.write(f'<Fault {self.idx.v[i]}>: '
-                           f'Applying fault on Bus (idx={self.bus.v[i]}) at t={self.tf.v[i]}sec.')
+                           f'Applying fault on Bus (idx={self.bus.v[i]}) at t={self.tf.v[i]} sec.')
+
                 action = True
         return action
 
@@ -123,8 +156,13 @@ class Fault(ModelData, Model):
         for i in range(self.n):
             if is_time[i] and (self.u.v[i] == 1):
                 self.uf.v[i] = 0
-                self.system.dae.y[self.system.Bus.n:] = self._vstore
+
+                if self.config.restore:
+                    self.system.dae.y[self.system.Bus.n:] = self._vstore * self.config.scale
+                    logger.debug(f"Voltage restored after fault clearance at t={self.system.dae.t:.6f}")
+
                 tqdm.write(f'<Fault {self.idx.v[i]}>: '
-                           f'Clearing fault on Bus (idx={self.bus.v[i]}) at t={self.tc.v[i]}sec.')
+                           f'Clearing fault on Bus (idx={self.bus.v[i]}) at t={self.tc.v[i]} sec.')
+
                 action = True
         return action
