@@ -3,7 +3,7 @@ from andes.core import Config
 from collections import OrderedDict
 
 
-class BaseRoutine(object):
+class BaseRoutine:
     """
     Base routine class.
 
@@ -43,8 +43,8 @@ class BaseRoutine(object):
     def run(self, **kwargs):
         raise NotImplementedError
 
-    def summary(self):
+    def summary(self, **kwargs):
         raise NotImplementedError
 
-    def report(self):
+    def report(self, **kwargs):
         raise NotImplementedError
