@@ -1,11 +1,15 @@
+"""
+Module for power flow calculation.
+"""
+
+import logging
 from collections import OrderedDict
 
 from andes.utils.misc import elapsed
 from andes.routines.base import BaseRoutine
 from andes.variables.report import Report
-from andes.shared import np, matrix, sparse, newton_krylov
+from andes.shared import np, matrix, sparse, newton_krylov, IP_ADD
 
-import logging
 logger = logging.getLogger(__name__)
 
 
@@ -49,6 +53,9 @@ class PFlow(BaseRoutine):
         self.mis = []
         self.models = OrderedDict()
 
+        self.x_sol = None
+        self.y_sol = None
+
     def init(self):
         self.models = self.system.find_models('pflow')
         self.converged = False
@@ -57,8 +64,18 @@ class PFlow(BaseRoutine):
         self.niter = None
         self.mis = []
 
-        self.system.init(self.models)
+        self.x_sol = None
+        self.y_sol = None
+
+        self.system.init(self.models, routine='pflow')
         logger.info('Power flow initialized.')
+
+        # force precompile if numba is on - improves timing accuracy
+        if self.system.config.numba:
+            self.system.f_update(self.models)
+            self.system.g_update(self.models)
+            self.system.j_update(models=self.models)
+
         return self.system.dae.xy
 
     def nr_step(self):
@@ -93,7 +110,10 @@ class PFlow(BaseRoutine):
         self.A = sparse([[system.dae.fx, system.dae.gx],
                          [system.dae.fy, system.dae.gy]])
 
-        self.inc = self.solver.solve(self.A, self.inc)
+        if not self.config.linsolve:
+            self.inc = self.solver.solve(self.A, self.inc)
+        else:
+            self.inc = self.solver.linsolve(self.A, self.inc)
 
         system.dae.x += np.ravel(np.array(self.inc[:system.dae.n]))
         system.dae.y += np.ravel(np.array(self.inc[system.dae.n:]))
@@ -109,11 +129,25 @@ class PFlow(BaseRoutine):
         """
         Output a summary for the PFlow routine.
         """
+        ipadd_status = 'CVXOPT normal (ipadd not available)'
+
+        # extract package name, `cvxopt` or `kvxopt`
+        sp_module = sparse.__module__
+        if '.' in sp_module:
+            sp_module = sp_module.split('.')[0]
+
+        if IP_ADD:
+            if self.system.config.ipadd:
+                ipadd_status = f'Fast in-place ({sp_module})'
+            else:
+                ipadd_status = 'CVXOPT normal (ipadd disabled in config)'
+
         out = list()
         out.append('')
         out.append('-> Power flow calculation')
-        out.append(f'Sparse Solver: {self.solver.sparselib.upper()}')
-        out.append(f'Method: {self.config.method} method')
+        out.append(f'{"Sparse solver":>16s}: {self.solver.sparselib.upper()}')
+        out.append(f'{"Solution method":>16s}: {self.config.method} method')
+        out.append(f'{"Sparse addition":>16s}: {ipadd_status}')
         out_str = '\n'.join(out)
         logger.info(out_str)
 
@@ -138,18 +172,18 @@ class PFlow(BaseRoutine):
         self.niter = 0
         while True:
             mis = self.nr_step()
-            logger.info(f'{self.niter}: |F(x)| = {mis:<10g}')
+            logger.info('%d: |F(x)| = %.10g', self.niter, mis)
 
             if mis < self.config.tol:
                 self.converged = True
                 break
-            elif self.niter > self.config.max_iter:
+            if self.niter > self.config.max_iter:
                 break
-            elif np.isnan(mis).any():
+            if np.isnan(mis).any():
                 logger.error('NaN found in solution. Convergence not likely')
                 self.niter = self.config.max_iter + 1
                 break
-            elif mis > 1e4 * self.mis[0]:
+            if mis > 1e4 * self.mis[0]:
                 logger.error('Mismatch increased too fast. Convergence not likely.')
                 break
             self.niter += 1
@@ -161,12 +195,17 @@ class PFlow(BaseRoutine):
                 max_idx = np.argmax(np.abs(system.dae.xy))
                 name = system.dae.xy_name[max_idx]
                 logger.error('Mismatch is not correctable possibly due to large load-generation imbalance.')
-                logger.error(f'Largest mismatch on equation associated with <{name}>')
+                logger.error('Largest mismatch on equation associated with <%s>', name)
             else:
-                logger.error(f'Power flow failed after {self.niter + 1} iterations for {system.files.case}.')
+                logger.error('Power flow failed after %d iterations for "%s".', self.niter + 1, system.files.case)
 
         else:
-            logger.info(f'Converged in {self.niter+1} iterations in {s1}.')
+            logger.info('Converged in %d iterations in %s.', self.niter + 1, s1)
+
+            # make a copy of power flow solutions
+            self.x_sol = system.dae.x.copy()
+            self.y_sol = system.dae.y.copy()
+
             if self.config.init_tds:
                 system.TDS.init()
             if self.config.report:
@@ -196,22 +235,22 @@ class PFlow(BaseRoutine):
 
         """
         system = self.system
-        system.dae.x = xy[:system.dae.n]
-        system.dae.y = xy[system.dae.n:]
+        system.dae.x[:] = xy[:system.dae.n]
+        system.dae.y[:] = xy[system.dae.n:]
         system.vars_to_models()
 
         system.dae.clear_fg()
-        system.l_update_var()
-        system.f_update()
-        system.g_update()
-        system.l_update_eq()
+        system.l_update_var(self.models)
+        system.f_update(self.models)
+        system.g_update(self.models)
+        system.l_update_eq(self.models)
         system.fg_to_dae()
 
         return system.dae.fg
 
     def newton_krylov(self, verbose=False):
         """
-        Full Newton-Krylov method
+        Full Newton-Krylov method from SciPy.
 
         Warnings
         --------
@@ -220,10 +259,12 @@ class PFlow(BaseRoutine):
         Parameters
         ----------
         verbose
+            True if verbose.
 
         Returns
         -------
-
+        np.array
+            Solutions `dae.xy`.
         """
         system = self.system
         system.init(system.exist.pflow)
