@@ -1,15 +1,20 @@
+"""
+ANDES module for time-domain simulation.
+"""
+
 import sys
 import os
+import time
 import importlib
+import logging
 from collections import OrderedDict
 
 from andes.routines.base import BaseRoutine
-from andes.utils.misc import elapsed, is_notebook
+from andes.utils.misc import elapsed, is_notebook, is_interactive
 from andes.utils.tab import Tab
-from andes.shared import tqdm, np
+from andes.shared import tqdm, np, pd
 from andes.shared import matrix, sparse, spdiag
 
-import logging
 logger = logging.getLogger(__name__)
 
 
@@ -24,8 +29,13 @@ class TDS(BaseRoutine):
                                      ('tf', 20.0),
                                      ('fixt', 1),
                                      ('shrinkt', 1),
+                                     ('honest', 0),
                                      ('tstep', 1/30),
                                      ('max_iter', 15),
+                                     ('refresh_event', 0),
+                                     ('g_scale', 1),
+                                     ('qrt', 0),
+                                     ('kqrt', 1.0),
                                      )))
         self.config.add_extra("_help",
                               tol="convergence tolerance",
@@ -33,8 +43,13 @@ class TDS(BaseRoutine):
                               tf="simulation ending time",
                               fixt="use fixed step size (1) or variable (0)",
                               shrinkt='shrink step size for fixed method if not converged',
+                              honest='honest Newton method that updates Jac at each step',
                               tstep='the initial step step size',
                               max_iter='maximum number of iterations',
+                              refresh_event='refresh events at each step',
+                              g_scale='scale algebraic residuals with time step size',
+                              qrt='quasi-real-time stepping',
+                              kqrt='quasi-real-time scaling factor; kqrt > 1 means slowing down',
                               )
         self.config.add_extra("_alt",
                               tol="float",
@@ -42,12 +57,26 @@ class TDS(BaseRoutine):
                               tf=">t0",
                               fixt=(0, 1),
                               shrinkt=(0, 1),
+                              honest=(0, 1),
                               tstep='float',
                               max_iter='>=10',
+                              refresh_event=(0, 1),
+                              g_scale=(0, 1),
+                              qrt='bool',
+                              kqrt="positive",
                               )
         # overwrite `tf` from command line
         if system.options.get('tf') is not None:
             self.config.tf = system.options.get('tf')
+        if system.options.get('qrt') is True:
+            self.config.qrt = system.options.get('qrt')
+        if system.options.get('kqrt') is not None:
+            self.config.kqrt = system.options.get('kqrt')
+
+        # if data is from a CSV file instead of simulation
+        self.from_csv = system.options.get('from_csv')
+        self.data_csv = None
+        self.k_csv = 0    # row number
 
         # to be computed
         self.deltat = 0
@@ -55,24 +84,34 @@ class TDS(BaseRoutine):
         self.deltatmax = 0
         self.h = 0
         self.next_pc = 0
-        self.eye = None
         self.Teye = None
         self.qg = np.array([])
-        self.tol_zero = self.config.tol / 100
+        self.tol_zero = self.config.tol / 1000
 
         # internal status
         self.converged = False
-        self.busted = False
+        self.last_converged = False   # True if the previous step converged
+        self.busted = False           # True if in a non-recoverable error state
         self.err_msg = ''
         self.niter = 0
-        self._switch_idx = 0  # index into `System.switch_times`
-        self._last_switch_t = -999  # the last critical time
+        self._switch_idx = 0          # index into `System.switch_times`
+        self._last_switch_t = -999    # the last critical time
+        self.custom_event = False
         self.mis = 1
         self.pbar = None
         self.callpert = None
         self.plotter = None
         self.plt = None
         self.initialized = False
+        self.qrt_start = None
+        self.headroom = 0.0
+
+        # internal storage for iterations
+        self.x0 = None
+        self.y0 = None
+        self.f0 = None
+        self.Ac = None
+        self.inc = None
 
     def init(self):
         """
@@ -90,29 +129,66 @@ class TDS(BaseRoutine):
         if self.initialized:
             return system.dae.xy
 
-        self._reset()
+        self.reset()
         self._load_pert()
-        system.set_address(models=system.exist.tds)
+
+        # restore power flow solutions
+        system.dae.x[:len(system.PFlow.x_sol)] = system.PFlow.x_sol
+        system.dae.y[:len(system.PFlow.y_sol)] = system.PFlow.y_sol
+
+        # Note:
+        #   calling `set_address` on `system.exist.pflow_tds` will point all variables
+        #   to the new array after extending `dae.y`.
+        system.set_address(models=system.exist.pflow_tds)
         system.set_dae_names(models=system.exist.tds)
 
         system.dae.clear_ts()
         system.store_sparse_pattern(models=system.exist.pflow_tds)
         system.store_adder_setter(models=system.exist.pflow_tds)
         system.vars_to_models()
-        system.init(system.exist.tds)
-        system.store_switch_times(system.exist.tds)
-        self.eye = spdiag([1] * system.dae.n)
-        self.Teye = spdiag(system.dae.Tf.tolist()) * self.eye
+
+        system.init(system.exist.tds, routine='tds')
+
+        # only store switch times when not replaying CSV data
+        if self.data_csv is None:
+            system.store_switch_times(system.exist.tds)
+
+        # Build mass matrix into `self.Teye`
+        self.Teye = spdiag(system.dae.Tf.tolist())
         self.qg = np.zeros(system.dae.n + system.dae.m)
+
+        # test if residuals are close enough to zero
+        self.initialized = self.test_init()
+
+        # discard initialized values and use that from CSV if provided
+        if self.data_csv is not None:
+            system.dae.x[:] = self.data_csv[0, 1:system.dae.n + 1]
+            system.dae.y[:] = self.data_csv[0, system.dae.n + 1:system.dae.n + system.dae.m + 1]
+            system.vars_to_models()
+
+        # connect to data streaming server
+        if system.config.dime_enabled:
+            if system.streaming.dimec is None:
+                system.streaming.connect()
+
+        # send out system data using DiME
+        self.streaming_init()
+        self.streaming_step()
+
+        # if `dae.n == 1`, `calc_h_first` depends on new `dae.gy`
         self.calc_h()
 
-        self.initialized = self.test_init()
+        # allocate for internal variables
+        self.x0 = np.zeros_like(system.dae.x)
+        self.y0 = np.zeros_like(system.dae.y)
+        self.f0 = np.zeros_like(system.dae.f)
+
         _, s1 = elapsed(t0)
 
         if self.initialized is True:
-            logger.info(f"Initialization was successful in {s1}.")
+            logger.info("Initialization for dynamics was successful in %s.", s1)
         else:
-            logger.error(f"Initialization failed in {s1}.")
+            logger.error("Initialization for dynamics failed in %s.", s1)
 
         if system.dae.n == 0:
             tqdm.write('No dynamic component loaded.')
@@ -120,31 +196,43 @@ class TDS(BaseRoutine):
 
     def summary(self):
         """
-        Print out a summary to logger.info.
+        Print out a summary of TDS options to logger.info.
 
         Returns
         -------
-
+        None
         """
         out = list()
         out.append('')
         out.append('-> Time Domain Simulation Summary:')
-        out.append(f'Sparse Solver: {self.solver.sparselib.upper()}')
-        out.append(f'Simulation time: {self.system.dae.t}-{self.config.tf}sec.')
-        if self.config.fixt == 1:
-            msg = f'Fixed step size: h={1000 * self.config.tstep:.4g}msec.'
-            if self.config.shrinkt == 1:
-                msg += ', shrink if not converged'
-            out.append(msg)
+
+        if self.data_csv is not None:
+            out.append(f'Loaded data from CSV file "{self.from_csv}".')
+            out.append('Replaying from CSV data.')
+            out.append(f'Replay time: {self.system.dae.t}-{self.config.tf} sec.')
         else:
-            out.append(f'Variable step size: h0={1000 * self.config.tstep:.4g}msec.')
+            out.append(f'Sparse Solver: {self.solver.sparselib.upper()}')
+            out.append(f'Simulation time: {self.system.dae.t}-{self.config.tf} sec.')
+            if self.config.fixt == 1:
+                msg = f'Fixed step size: h={1000 * self.config.tstep:.4g} msec.'
+                if self.config.shrinkt == 1:
+                    msg += ', shrink if not converged'
+                out.append(msg)
+            else:
+                out.append(f'Variable step size: h0={1000 * self.config.tstep:.4g} msec.')
 
         out_str = '\n'.join(out)
         logger.info(out_str)
 
+        if self.config.honest == 1:
+            logger.warning("The honest Newton method is used and will slow down the simulation.")
+            logger.warning("For significant speed up, set `honest=0` in TDS.config.")
+
     def run(self, no_pbar=False, no_summary=False, **kwargs):
         """
-        Run the implicit numerical integration for TDS.
+        Run time-domain simulation using numerical integration.
+
+        The default method is the Implicit Trapezoidal Method (ITM).
 
         Parameters
         ----------
@@ -165,6 +253,10 @@ class TDS(BaseRoutine):
             system.exit_code += 1
             return succeed
 
+        # load from csv is provided
+        if self.from_csv is not None:
+            self.data_csv = self._load_csv(self.from_csv)
+
         if no_summary is False:
             self.summary()
 
@@ -173,6 +265,7 @@ class TDS(BaseRoutine):
             self.init()
         else:  # resume simulation
             resume = True
+            self._calc_h_first()
 
         self.pbar = tqdm(total=100, ncols=70, unit='%', file=sys.stdout, disable=no_pbar)
 
@@ -181,29 +274,34 @@ class TDS(BaseRoutine):
             self.next_pc = perc + 1
             self.pbar.update(perc)
 
+        self.qrt_start = time.time()
+        self.headroom = 0.0
+
         t0, _ = elapsed()
 
-        while (system.dae.t < self.config.tf) and (not self.busted):
+        while (system.dae.t - self.h < self.config.tf) and (not self.busted):
             if self.callpert is not None:
                 self.callpert(dae.t, system)
 
-            if self._itm_step():  # simulate the current step
+            step_status = False
+            # call the stepping method of the integration method (or data replay)
+            if self.data_csv is None:
+                step_status = self.itm_step()  # compute for the current step
+            else:
+                step_status = self._csv_step()
+
+            if step_status:
                 # store values
                 dae.ts.store_txyz(dae.t.tolist(),
                                   dae.xy,
                                   self.system.get_z(models=system.exist.pflow_tds),
                                   )
+
+                self.streaming_step()
+
                 # check if the next step is critical time
-                if self.is_switch_time():
-                    self._last_switch_t = system.switch_times[self._switch_idx]
-                    system.switch_action(system.exist.pflow_tds)
-                    self._switch_idx += 1
-                    system.vars_to_models()
-
-                if self.calc_h() == 0:
-                    logger.error('Time step to zero...')
-                    break
-
+                self.do_switch()
+                self.calc_h()
                 dae.t += self.h
 
                 # show progress in percentage
@@ -212,14 +310,34 @@ class TDS(BaseRoutine):
                     self.pbar.update(1)
                     self.next_pc += 1
 
+                # quasi-real-time check and wait (except for the last step)
+                if config.qrt and self.h > 0:
+                    rt_end = self.qrt_start + self.h * config.kqrt
+
+                    # if the ending time has passed
+                    if time.time() - rt_end > 0:
+                        logger.debug('Simulation over-run at t=%4.4g s.', dae.t)
+                    else:
+                        self.headroom += (rt_end - time.time())
+
+                        while time.time() - rt_end < 0:
+                            time.sleep(1e-4)
+
+                    self.qrt_start = time.time()
+
+            else:
+                if self.calc_h() == 0:
+                    self.err_msg = "Time step reduced to zero. Convergence is not likely."
+                    self.busted = True
+                    break
+
         self.pbar.close()
-        delattr(self, 'pbar')  # removed `pbar` so that System object can be dilled
+        delattr(self, 'pbar')  # removed `pbar` so that System object can be serialized
 
         if self.busted:
             logger.error(self.err_msg)
-            logger.error(f"Simulation terminated at t={system.dae.t:.4f}.")
+            logger.error("Simulation terminated at t=%.4f s.", system.dae.t)
             system.exit_code += 1
-
         elif system.dae.t == self.config.tf:
             succeed = True   # success flag
             system.exit_code += 0
@@ -227,81 +345,24 @@ class TDS(BaseRoutine):
             system.exit_code += 1
 
         _, s1 = elapsed(t0)
-        logger.info(f'Simulation completed in {s1}.')
+        logger.info('Simulation completed in %s.', s1)
+
+        if config.qrt:
+            logger.debug('QRT headroom time: %.4g s.', self.headroom)
+
         system.TDS.save_output()
 
-        # load data into `TDS.plotter` in the notebook mode
-        if is_notebook():
+        # end data streaming
+        if system.config.dime_enabled:
+            system.streaming.finalize()
+
+        # load data into `TDS.plotter` in a notebook or in an interactive mode
+        if is_notebook() or is_interactive():
             self.load_plotter()
 
         return succeed
 
-    def rewind(self, t):
-        """
-        TODO: rewind to a past time.
-        """
-        pass
-
-    def load_plotter(self):
-        """
-        Manually load a plotter into ``TDS.plotter``.
-        """
-        from andes.plot import TDSData  # NOQA
-        self.plotter = TDSData(mode='memory', dae=self.system.dae)
-        self.plt = self.plotter
-
-    def test_init(self):
-        """
-        Update f and g to see if initialization is successful.
-        """
-        system = self.system
-        self._fg_update(system.exist.pflow_tds)
-        system.j_update(models=system.exist.pflow_tds)
-
-        # warn if variables are initialized at limits
-        if system.config.warn_limits:
-            for model in system.exist.pflow_tds.values():
-                for item in model.discrete.values():
-                    item.warn_init_limit()
-
-        if np.max(np.abs(system.dae.fg)) < self.config.tol:
-            logger.debug('Initialization tests passed.')
-            return True
-        else:
-
-            fail_idx = np.where(abs(system.dae.fg) >= self.config.tol)
-            fail_names = [system.dae.xy_name[int(i)] for i in np.ravel(fail_idx)]
-
-            title = 'Suspect initialization issue! Simulation may crash!'
-            err_data = {'Name': fail_names,
-                        'Var. Value': system.dae.xy[fail_idx],
-                        'Eqn. Mismatch': system.dae.fg[fail_idx],
-                        }
-            tab = Tab(title=title,
-                      header=err_data.keys(),
-                      data=list(map(list, zip(*err_data.values()))))
-
-            logger.error(tab.draw())
-
-            if system.options.get('verbose') == 1:
-                breakpoint()
-            system.exit_code += 1
-            return False
-
-    def _fg_update(self, models):
-        """
-        Update `f` and `g` equations.
-        """
-        system = self.system
-        system.dae.clear_fg()
-        system.l_update_var(models=models)
-        system.s_update_var(models=models)  # update VarService
-        system.f_update(models=models)
-        system.g_update(models=models)
-        system.l_update_eq(models=models)
-        system.fg_to_dae()
-
-    def _itm_step(self):
+    def itm_step(self):
         """
         Integrate with Implicit Trapezoidal Method (ITM) to the current time.
 
@@ -321,43 +382,72 @@ class TDS(BaseRoutine):
         self.niter = 0
         self.converged = False
 
-        self.x0 = np.array(dae.x)
-        self.y0 = np.array(dae.y)
-        self.f0 = np.array(dae.f)
+        self.x0[:] = dae.x
+        self.y0[:] = dae.y
+        self.f0[:] = dae.f
 
         while True:
-            self._fg_update(models=system.exist.pflow_tds)
+            self.fg_update(models=system.exist.pflow_tds)
 
             # lazy Jacobian update
-            if dae.t == 0 or self.niter > 3 or (dae.t - self._last_switch_t < 0.2):
-                system.j_update(models=system.exist.pflow_tds)
-                self.solver.factorize = True
 
-            # TODO: set the `Tf` corresponding to the pegged anti-windup limiters to zero.
-            # Although this should not affect anything since corr. mismatches in `self.qg` are reset to zero
+            reason = ''
+            if dae.t == 0:
+                reason = 't=0'
+            elif self.config.honest:
+                reason = 'using honest method'
+            elif self.custom_event:
+                reason = 'custom event set'
+            elif not self.last_converged:
+                reason = 'last step did not converge'
+            elif self.niter > 4 and (self.niter + 1) % 3 == 0:
+                reason = 'update every 6 iterations'
+            elif dae.t - self._last_switch_t < 0.1:
+                reason = 'within 0.1s of event'
+
+            if reason:
+                system.j_update(models=system.exist.pflow_tds, info=reason)
+
+                # set flag in `solver.worker.factorize`, not `solver.factorize`.
+                self.solver.worker.factorize = True
+
+            # `Tf` should remain constant throughout the simulation, even if the corresponding diff. var.
+            # is pegged by the anti-windup limiters.
 
             # solve implicit trapezoidal method (ITM) integration
-            self.Ac = sparse([[self.Teye - self.h * 0.5 * dae.fx, dae.gx],
-                              [-self.h * 0.5 * dae.fy, dae.gy]], 'd')
+            if self.config.g_scale == 1:
+                gxs = self.h * dae.gx
+                gys = self.h * dae.gy
+            else:
+                gxs = dae.gx
+                gys = dae.gy
+
+            self.Ac = sparse([[self.Teye - self.h * 0.5 * dae.fx, gxs],
+                              [-self.h * 0.5 * dae.fy, gys]], 'd')
 
             # equation `self.qg[:dae.n] = 0` is the implicit form of differential equations using ITM
             self.qg[:dae.n] = dae.Tf * (dae.x - self.x0) - self.h * 0.5 * (dae.f + self.f0)
 
             # reset the corresponding q elements for pegged anti-windup limiter
             for item in system.antiwindups:
-                for key, val in item.x_set:
-                    np.put(self.qg, key, 0)
+                for key, _, eqval in item.x_set:
+                    np.put(self.qg, key, eqval)
 
-            self.qg[dae.n:] = dae.g
-
-            if not self.config.linsolve:
-                inc = self.solver.solve(self.Ac, -matrix(self.qg))
+            # set or scale the algebraic residuals
+            if self.config.g_scale == 1:
+                self.qg[dae.n:] = self.h * dae.g
             else:
-                inc = self.solver.linsolve(self.Ac, -matrix(self.qg))
+                self.qg[dae.n:] = dae.g
+
+            # calculate variable corrections
+            if not self.config.linsolve:
+                inc = self.solver.solve(self.Ac, matrix(self.qg))
+            else:
+                inc = self.solver.linsolve(self.Ac, matrix(self.qg))
 
             # check for np.nan first
             if np.isnan(inc).any():
-                self.err_msg = 'NaN found in solution. Convergence not likely'
+                self.err_msg = 'NaN found in solution. Convergence is not likely'
                 self.niter = self.config.max_iter + 1
                 self.busted = True
                 break
@@ -366,13 +456,17 @@ class TDS(BaseRoutine):
             inc[np.where(np.abs(inc) < self.tol_zero)] = 0
 
             # set new values
-            dae.x += inc[:dae.n].ravel()
-            dae.y += inc[dae.n: dae.n + dae.m].ravel()
+            dae.x -= inc[:dae.n].ravel()
+            dae.y -= inc[dae.n: dae.n + dae.m].ravel()
 
+            # synchronize solutions to model internal storage
             system.vars_to_models()
 
-            # calculate correction
+            # store `inc` to self for debugging
+            self.inc = inc
+
             mis = np.max(np.abs(inc))
+            # store initial maximum mismatch
             if self.niter == 0:
                 self.mis = mis
 
@@ -384,108 +478,53 @@ class TDS(BaseRoutine):
                 break
             # non-convergence cases
             if self.niter > self.config.max_iter:
-                logger.debug(f'Max. iter. {self.config.max_iter} reached for t={dae.t:.6f}, '
-                             f'h={self.h:.6f}, mis={mis:.4g} ')
+                tqdm.write(f'* Max. iter. {self.config.max_iter} reached for t={dae.t:.6f}s, '
+                           f'h={self.h:.6f}s, max inc={mis:.4g} ')
 
                 # debug helpers
                 g_max = np.argmax(abs(dae.g))
                 inc_max = np.argmax(abs(inc))
                 self._debug_g(g_max)
                 self._debug_ac(inc_max)
-
                 break
-            if mis > 1000 and (mis > 1e8 * self.mis):
+
+            if mis > 1e6 and (mis > 1e6 * self.mis):
                 self.err_msg = 'Error increased too quickly. Convergence not likely.'
                 self.busted = True
                 break
 
         if not self.converged:
-            dae.x = np.array(self.x0)
-            dae.y = np.array(self.y0)
-            dae.f = np.array(self.f0)
+            dae.x[:] = np.array(self.x0)
+            dae.y[:] = np.array(self.y0)
+            dae.f[:] = np.array(self.f0)
             system.vars_to_models()
 
+        self.last_converged = self.converged
+
         return self.converged
 
-    def _debug_g(self, y_idx):
+    def _csv_step(self):
         """
-        Print out the associated variables with the given algebraic equation index.
-
-        Parameters
-        ----------
-        y_idx
-            Index of the equation into the `g` array. Diff. eqns. are not counted in.
+        Fetch data for the next step from ``data_csv``.
         """
-        y_idx = y_idx.tolist()
-        logger.debug(f'Max. algebraic mismatch associated with {self.system.dae.y_name[y_idx]} [y_idx={y_idx}]')
-        assoc_vars = self.system.dae.gy[y_idx, :]
-        vars_idx = np.where(np.ravel(matrix(assoc_vars)))[0]
+        system = self.system
+        if self.data_csv is not None:
+            system.dae.x[:] = self.data_csv[self.k_csv, 1:system.dae.n + 1]
+            system.dae.y[:] = self.data_csv[self.k_csv, system.dae.n + 1:system.dae.n + system.dae.m + 1]
+            system.vars_to_models()
 
-        logger.debug('')
-        logger.debug(f'{"y_index":<10} {"Variable":<20} {"Derivative":<20}')
-        for v in vars_idx:
-            v = v.tolist()
-            logger.debug(f'{v:<10} {self.system.dae.y_name[v]:<20} {assoc_vars[v]:<20g}')
+        self.converged = True
+        return self.converged
 
-        pass
-
-    def _debug_ac(self, xy_idx):
-        """
-        Debug Ac matrix by printing out equations and derivatives associated with the max. mismatch variable.
-
-        Parameters
-        ----------
-        xy_idx
-            Index of the maximum mismatch into the `xy` array.
-        """
-
-        xy_idx = xy_idx.tolist()
-        assoc_eqns = self.Ac[:, xy_idx]
-        assoc_vars = self.Ac[xy_idx, :]
-
-        eqns_idx = np.where(np.ravel(matrix(assoc_eqns)))[0]
-        vars_idx = np.where(np.ravel(matrix(assoc_vars)))[0]
-
-        logger.debug(f'Max. correction is for variable {self.system.dae.xy_name[xy_idx]} [{xy_idx}]')
-        logger.debug(f'Associated equation value is {self.system.dae.fg[xy_idx]:<20g}.')
-        logger.debug('')
-
-        logger.debug(f'{"xy_index":<10} {"Equation":<20} {"Derivative":<20} {"Eq. Mismatch":<20}')
-        for eq in eqns_idx:
-            eq = eq.tolist()
-            logger.debug(f'{eq:<10} {self.system.dae.xy_name[eq]:<20} {assoc_eqns[eq]:<20g} '
-                         f'{self.system.dae.fg[eq]:<20g}')
-
-        logger.debug('')
-        logger.debug(f'{"xy_index":<10} {"Variable":<20} {"Derivative":<20} {"Eq. Mismatch":<20}')
-        for v in vars_idx:
-            v = v.tolist()
-            logger.debug(f'{v:<10} {self.system.dae.xy_name[v]:<20} {assoc_vars[v]:<20g} '
-                         f'{self.system.dae.fg[v]:<20g}')
-
-    def save_output(self):
-        """
-        Save the simulation data into two files: a lst file and a npy file.
-
-        Returns
-        -------
-        bool
-            True if files are written. False otherwise.
-        """
-        if self.system.files.no_output:
-            return False
-        else:
-            t0, _ = elapsed()
-            self.system.dae.write_lst(self.system.files.lst)
-            self.system.dae.write_npy(self.system.files.npy)
-            _, s1 = elapsed(t0)
-            logger.info(f'TDS outputs saved in {s1}.')
-            return True
-
-    def calc_h(self):
+    def calc_h(self, resume=False):
         """
         Calculate the time step size during the TDS.
 
+        Parameters
+        ----------
+        resume : bool
+            If True, calculate the initial step size.
+
         Notes
         -----
         A heuristic function is used for variable time step size ::
@@ -502,7 +541,8 @@ class TDS(BaseRoutine):
         system = self.system
         config = self.config
 
-        if system.dae.t == 0:
+        # t=0, first iteration (not previously failed)
+        if (system.dae.t == 0 and self.niter == 0) or resume:
             return self._calc_h_first()
 
         if config.fixt and not config.shrinkt:
@@ -535,10 +575,19 @@ class TDS(BaseRoutine):
             self.deltat = config.tf - system.dae.t
 
         self.h = self.deltat
+
         # do not skip event switch_times
         if self._switch_idx < system.n_switches:
             if (system.dae.t + self.h) > system.switch_times[self._switch_idx]:
                 self.h = system.switch_times[self._switch_idx] - system.dae.t
+
+        if self.data_csv is not None:
+            if self.k_csv + 1 < self.data_csv.shape[0]:
+                self.k_csv += 1
+                self.h = self.data_csv[self.k_csv, 0] - system.dae.t
+            else:
+                self.h = 0
+
         return self.h
 
     def _calc_h_first(self):
@@ -579,41 +628,293 @@ class TDS(BaseRoutine):
                 logger.warning('Fixed time step is smaller than the estimated minimum.')
 
         self.h = self.deltat
+
+        # if from CSV, determine `h` from data
+        if self.data_csv is not None:
+            if self.data_csv.shape[0] > 1:
+                self.h = self.data_csv[1, 0] - self.data_csv[0, 0]
+            else:
+                logger.warning("CSV data does not contain more than one time step.")
+                self.h = 0
+
         return self.h
 
-    def is_switch_time(self):
+    def load_plotter(self):
         """
-        Return if the current time is a switching time for time domain simulation.
+        Manually load a plotter into ``TDS.plotter``.
+        """
+        from andes.plot import TDSData  # NOQA
+        self.plotter = TDSData(mode='memory', dae=self.system.dae)
+        self.plt = self.plotter
 
-        Time is approximated with a tolerance of 1e-8.
+    def test_init(self):
+        """
+        Update f and g to see if initialization is successful.
+        """
+        system = self.system
+        self.fg_update(system.exist.pflow_tds)
+        system.j_update(models=system.exist.pflow_tds)
+
+        # warn if variables are initialized at limits
+        if system.config.warn_limits:
+            for model in system.exist.pflow_tds.values():
+                for item in model.discrete.values():
+                    item.warn_init_limit()
+
+        if np.max(np.abs(system.dae.fg)) < self.config.tol:
+            logger.debug('Initialization tests passed.')
+            return True
+
+        # otherwise, show suspect initialization error
+        fail_idx = np.where(abs(system.dae.fg) >= self.config.tol)
+        fail_names = [system.dae.xy_name[int(i)] for i in np.ravel(fail_idx)]
+
+        title = 'Suspect initialization issue! Simulation may crash!'
+        err_data = {'Name': fail_names,
+                    'Var. Value': system.dae.xy[fail_idx],
+                    'Eqn. Mismatch': system.dae.fg[fail_idx],
+                    }
+        tab = Tab(title=title,
+                  header=err_data.keys(),
+                  data=list(map(list, zip(*err_data.values()))),
+                  )
+
+        logger.error(tab.draw())
+
+        if system.options.get('verbose') == 1:
+            breakpoint()
+        system.exit_code += 1
+        return False
+
+    def save_output(self, npz=True):
+        """
+        Save the simulation data into two files: a lst file and a npz file.
+
+        Parameters
+        ----------
+        npz : bool
+            True to save in npz format; False to save in npy format.
 
         Returns
         -------
         bool
-            ``True`` if is a switching time; ``False`` otherwise.
+            True if files are written. False otherwise.
+        """
+        if self.system.files.no_output:
+            return False
+
+        t0, _ = elapsed()
+
+        self.system.dae.write_lst(self.system.files.lst)
+        if npz is True:
+            self.system.dae.write_npz(self.system.files.npz)
+        else:
+            self.system.dae.write_npy(self.system.files.npy)
+
+        _, s1 = elapsed(t0)
+        logger.info('TDS outputs saved in %s.', s1)
+        return True
+
+    def do_switch(self):
+        """
+        Checks if is an event time and perform switch if true.
+
+        Time is approximated with a tolerance of 1e-8.
         """
         ret = False
-        if self._switch_idx < self.system.n_switches:
-            if abs(self.system.dae.t - self.system.switch_times[self._switch_idx]) < 1e-8:
+        system = self.system
+
+        # refresh switch times if enabled
+        if self.config.refresh_event:
+            system.store_switch_times(system.exist.pflow_tds)
+
+        # if not all events have been processed
+        if self._switch_idx < system.n_switches:
+
+            # if the current time is close enough to the next event time
+            if np.isclose(system.dae.t, system.switch_times[self._switch_idx]):
+
+                # `_last_switch_t` is used by the Jacobian updater
+                self._last_switch_t = system.switch_times[self._switch_idx]
+
+                # only call `switch_action` on the models that defined the time
+                system.switch_action(system.switch_dict[self._last_switch_t])
+
+                # progress `_switch_idx` to avoid calling the same event if time gets stuck
+                self._switch_idx += 1
+                system.vars_to_models()
+
                 ret = True
+
+        # if a `custom_event` flag is set (without a specific callback)
+        if self.custom_event is True:
+            system.switch_action(system.exist.pflow_tds)
+            self._last_switch_t = system.dae.t.tolist()
+            system.vars_to_models()
+            self.custom_event = False
+            ret = True
+
         return ret
 
-    def _reset(self):
-        # reset states
+    def fg_update(self, models):
+        """
+        Update `f` and `g` equations.
+        """
+        system = self.system
+        system.dae.clear_fg()
+        system.l_update_var(models=models)
+        system.s_update_var(models=models)  # update VarService
+        system.f_update(models=models)
+        system.g_update(models=models)
+        system.l_update_eq(models=models)
+        system.fg_to_dae()
+
+    def _fg_wrapper(self, xy):
+        """
+        Wrapper function for equations. Callable by general-purpose DAE solvers.
+
+        Parameters
+        ----------
+        xy : np.ndarray
+            Input values for evaluating equations.
+
+        Returns
+        -------
+        np.ndarray
+            RHS of diff. and algeb. equations.
+
+        """
+        system = self.system
+        system.dae.x[:] = xy[:system.dae.n]
+        system.dae.y[:] = xy[system.dae.n:]
+        system.vars_to_models()
+
+        self.fg_update(system.exist.pflow_tds)
+
+        return system.dae.fg
+
+    def _load_pert(self):
+        """
+        Load perturbation files to ``self.callpert``.
+        """
+        system = self.system
+        if not system.files.pert:
+            return False
+
+        if not os.path.isfile(system.files.pert):
+            logger.warning('Pert file not found at "%s".', system.files.pert)
+            return False
+
+        pert_path, full_name = os.path.split(system.files.pert)
+        logger.debug('Pert file "%s" located at path %s', full_name, pert_path)
+
+        sys.path.append(pert_path)
+        name, _ = os.path.splitext(full_name)
+
+        module = importlib.import_module(name)
+        self.callpert = getattr(module, 'pert')
+        logger.info('Perturbation file "%s" loaded.', system.files.pert)
+        return True
+
+    def _load_csv(self, csv_file):
+        """
+        Load simulation data from CSV file and return a numpy array.
+        """
+        if csv_file is None:
+            return None
+
+        df = pd.read_csv(csv_file)
+
+        if df.isnull().values.any():
+            raise ValueError("CSV file contains missing values. Please check data consistency.")
+
+        data = df.to_numpy()
+
+        if data.ndim != 2:
+            raise ValueError("Data from CSV is not 2-dimensional (time versus variable)")
+        if data.shape[0] < 2:
+            logger.warning("CSV data does not contain more than one time step.")
+
+        # set start and end times from data
+        self.config.t0 = data[0, 0]
+        self.config.tf = data[-1, 0]
+
+        return data
+
+    def _debug_g(self, y_idx):
+        """
+        Print out the associated variables with the given algebraic equation index.
+
+        Parameters
+        ----------
+        y_idx
+            Index of the equation into the `g` array. Diff. eqns. are not counted in.
+        """
+        y_idx = y_idx.tolist()
+        logger.debug('Max. algebraic mismatch associated with <%s> [y_idx=%d]',
+                     self.system.dae.y_name[y_idx], y_idx)
+        assoc_vars = self.system.dae.gy[y_idx, :]
+        vars_idx = np.where(np.ravel(matrix(assoc_vars)))[0]
+
+        logger.debug('')
+        logger.debug(f'{"y_index":<10} {"Variable":<20} {"Derivative":<20}')
+        for v in vars_idx:
+            v = v.tolist()
+            logger.debug('%10d %20s %20g', v, self.system.dae.y_name[v], assoc_vars[v])
+
+    def _debug_ac(self, xy_idx):
+        """
+        Debug Ac matrix by printing out equations and derivatives associated with the max. mismatch variable.
+
+        Parameters
+        ----------
+        xy_idx
+            Index of the maximum mismatch into the `xy` array.
+        """
+
+        xy_idx = xy_idx.tolist()
+        assoc_eqns = self.Ac[:, xy_idx]
+        assoc_vars = self.Ac[xy_idx, :]
+
+        eqns_idx = np.where(np.ravel(matrix(assoc_eqns)))[0]
+        vars_idx = np.where(np.ravel(matrix(assoc_vars)))[0]
+
+        logger.debug('Max. correction is for variable %s [%d]', self.system.dae.xy_name[xy_idx], xy_idx)
+        logger.debug('Associated equation value is %20g', self.system.dae.fg[xy_idx])
+        logger.debug('')
+
+        logger.debug(f'{"xy_index":<10} {"Equation":<20} {"Derivative":<20} {"Eq. Mismatch":<20}')
+        for eq in eqns_idx:
+            eq = eq.tolist()
+            logger.debug(f'{eq:<10} {self.system.dae.xy_name[eq]:<20} {assoc_eqns[eq]:<20g} '
+                         f'{self.system.dae.fg[eq]:<20g}')
+
+        logger.debug('')
+        logger.debug(f'{"xy_index":<10} {"Variable":<20} {"Derivative":<20} {"Eq. Mismatch":<20}')
+        for v in vars_idx:
+            v = v.tolist()
+            logger.debug(f'{v:<10} {self.system.dae.xy_name[v]:<20} {assoc_vars[v]:<20g} '
+                         f'{self.system.dae.fg[v]:<20g}')
+
+    def reset(self):
+        """
+        Reset internal states to pre-init condition.
+        """
         self.deltat = 0
         self.deltatmin = 0
         self.deltatmax = 0
         self.h = 0
         self.next_pc = 0.1
-        self.eye = None
         self.Teye = None
         self.qg = np.array([])
 
         self.converged = False
+        self.last_converged = False
         self.busted = False
         self.niter = 0
-        self._switch_idx = 0       # index into `System.switch_times`
-        self._last_switch_t = -999  # the last critical time
+        self._switch_idx = 0        # index into `System.switch_times`
+        self._last_switch_t = -999  # the last event time
+        self.custom_event = False
         self.mis = 1
         self.system.dae.t = np.array(0.0)
         self.pbar = None
@@ -622,21 +923,37 @@ class TDS(BaseRoutine):
 
         self.initialized = False
 
-    def _load_pert(self):
+    def rewind(self, t):
         """
-        Load perturbation files to ``self.callpert``.
+        TODO: rewind to a past time.
+        """
+        raise NotImplementedError("TDS.rewind() not implemented")
+
+    def streaming_init(self):
+        """
+        Send out initialization variables and process init from modules.
+
+        Returns
+        -------
+        None
         """
         system = self.system
-        if system.files.pert:
-            if not os.path.isfile(system.files.pert):
-                logger.warning(f'Pert file not found at <{system.files.pert}>.')
-                return False
+        if system.config.dime_enabled:
+            system.streaming.send_init(recepient='all')
+            logger.info('Broadcast system data. Waiting to receive modules init info...')
+            time.sleep(0.5)
+            system.streaming.sync_and_handle()
 
-            sys.path.append(system.files.case_path)
-            _, full_name = os.path.split(system.files.pert)
-            name, ext = os.path.splitext(full_name)
+    def streaming_step(self):
+        """
+        Sync, handle and streaming for each integration step.
 
-            module = importlib.import_module(name)
-            self.callpert = getattr(module, 'pert')
-            logger.info(f'Perturbation file <{system.files.pert}> loaded.')
-            return True
+        Returns
+        -------
+        None
+        """
+        system = self.system
+        if system.config.dime_enabled:
+            system.streaming.sync_and_handle()
+            system.streaming.vars_to_modules()
+            system.streaming.vars_to_pmu()
