@@ -2,32 +2,50 @@
 System class for power system data and methods
 """
 
+#  [ANDES] (C)2015-2020 Hantao Cui
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 3 of the License, or
+#  (at your option) any later version.
+#
+#  File name: system.py
+#  Last modified: 8/16/20, 7:26 PM
+
 import configparser
 import importlib
 import logging
 import os
+import sys
 import inspect
+import dill
 from collections import OrderedDict
 from typing import List, Dict, Tuple, Union, Optional
 
+import andes.io
 from andes import __version__
-from andes.models import non_jit
+from andes.models import file_classes
+from andes.models.group import GroupBase
 from andes.variables import FileMan, DAE
 from andes.routines import all_routines
 from andes.utils.tab import Tab
-from andes.utils.paths import get_config_path, get_pkl_path, confirm_overwrite
-from andes.core import Config, BaseParam, Model, AntiWindup
+from andes.utils.misc import elapsed
+from andes.utils.paths import get_config_path, get_pkl_path, confirm_overwrite, get_dot_andes_path
+from andes.core import Config, Model, AntiWindup
+from andes.io.streaming import Streaming
 
-from andes.shared import np, spmatrix, jac_names
+from andes.shared import np, spmatrix, jac_names, IP_ADD
 logger = logging.getLogger(__name__)
 
-if hasattr(spmatrix, 'ipadd'):
-    IP_ADD = True
-else:
-    IP_ADD = False
+sys.path.append(get_dot_andes_path())
+
+try:
+    import pycode  # NOQA
+except ImportError:
+    pycode = None  # NOQA
 
 
-class ExistingModels(object):
+class ExistingModels:
     """
     Storage class for existing models
     """
@@ -37,7 +55,7 @@ class ExistingModels(object):
         self.pflow_tds = OrderedDict()
 
 
-class System(object):
+class System:
     """
     System contains models and routines for modeling and simulation.
 
@@ -72,6 +90,7 @@ class System(object):
                  case: Optional[str] = None,
                  name: Optional[str] = None,
                  config_path: Optional[str] = None,
+                 default_config: Optional[bool] = False,
                  options: Optional[Dict] = None,
                  **kwargs
                  ):
@@ -86,13 +105,17 @@ class System(object):
         self.groups = OrderedDict()        # group names and instances
         self.routines = OrderedDict()      # routine names and instances
         self.switch_times = np.array([])   # an array of ordered event switching times
+        self.switch_dict = OrderedDict()   # time: OrderedDict of associated models
         self.n_switches = 0                # number of elements in `self.switch_times`
         self.exit_code = 0                 # command-line exit code, 0 - normal, others - error.
 
         # get and load default config file
         self._config_path = get_config_path()
-        if config_path:
+        if config_path is not None:
             self._config_path = config_path
+        if default_config is True:
+            self._config_path = None
+
         self._config_object = self.load_config(self._config_path)
         self.config = Config(self.__class__.__name__)
         self.config.load(self._config_object)
@@ -102,17 +125,32 @@ class System(object):
                                      ('mva', 100),
                                      ('store_z', 0),
                                      ('ipadd', 1),
+                                     ('diag_eps', 1e-8),
                                      ('warn_limits', 1),
                                      ('warn_abnormal', 1),
+                                     ('dime_enabled', 0),
+                                     ('dime_name', 'andes'),
+                                     ('dime_protocol', 'ipc'),
+                                     ('dime_address', '/tmp/dime2'),
+                                     ('numba', 0),
+                                     ('numba_parallel', 0),
+                                     ('save_pycode', 0),
+                                     ('yapf_pycode', 0),
+                                     ('use_pycode', 0),
                                      )))
-
         self.config.add_extra("_help",
                               freq='base frequency [Hz]',
                               mva='system base MVA',
                               store_z='store limiter status in TDS output',
-                              ipadd='Use spmatrix.ipadd if available',
+                              ipadd='use spmatrix.ipadd if available',
+                              diag_eps='small value for Jacobian diagonals',
                               warn_limits='warn variables initialized at limits',
                               warn_abnormal='warn initialization out of normal values',
+                              numba='use numba for JIT compilation',
+                              numba_parallel='enable parallel for numba.jit',
+                              save_pycode='save generated code to ~/.andes',
+                              yapf_pycode='format generated code with yapf',
+                              use_pycode='use generated, saved Python code',
                               )
         self.config.add_extra("_alt",
                               freq="float",
@@ -121,12 +159,18 @@ class System(object):
                               ipadd=(0, 1),
                               warn_limits=(0, 1),
                               warn_abnormal=(0, 1),
+                              numba=(0, 1),
+                              numba_parallel=(0, 1),
+                              save_pycode=(0, 1),
+                              yapf_pycode=(0, 1),
+                              use_pycode=(0, 1),
                               )
         self.config.check()
         self.exist = ExistingModels()
 
         self.files = FileMan(case=case, **self.options)    # file path manager
         self.dae = DAE(system=self)                        # numerical DAE storage
+        self.streaming = Streaming(self)                   # Dime2 streaming
 
         # dynamic imports of groups, models and routines
         self.import_groups()
@@ -141,6 +185,16 @@ class System(object):
         # internal flags
         self.is_setup = False              # if system has been setup
 
+    def reload(self, case, **kwargs):
+        """
+        Reload a new case in the same System object.
+        """
+        self.options.update(kwargs)
+        self.files.set(case=case, **kwargs)
+        # TODO: clear all flags and empty data
+        andes.io.parse(self)
+        self.setup()
+
     def _clear_adder_setter(self):
         """
         Clear adders and setters storage
@@ -188,18 +242,26 @@ class System(object):
         """
         import math
 
+        # info
+        if incremental is True:
+            text_mode = 'rapid incremental mode'
+        elif quick is True:
+            text_mode = 'quick mode'
+        else:
+            text_mode = 'full mode'
+
+        logger.info(f'Numerical code generation ({text_mode}) started...')
+
+        t0, _ = elapsed()
+
         # consistency check for group parameters and variables
         self._check_group_common()
 
         loaded_calls = self._load_pkl()
-        if loaded_calls is None:
+        if loaded_calls is None and incremental:
             incremental = False
             logger.debug('calls.pkl does not exist. Incremental codegen disabled.')
 
-        if quick is False and incremental is True:
-            incremental = False
-            logger.debug('Incremental codegen is only allowed in quick mode.')
-
         total = len(self.models)
         width = math.ceil(math.log(total, 10))
         for idx, (name, model) in enumerate(self.models.items()):
@@ -212,13 +274,32 @@ class System(object):
                           end='\r', flush=True)
                     continue
 
-            print(f"\r\x1b[K Code generation for {name} ({idx+1:>{width}}/{total:>{width}}).",
+            print(f"\r\x1b[K Generating code for {name} ({idx+1:>{width}}/{total:>{width}}).",
                   end='\r', flush=True)
             model.prepare(quick=quick)
 
+        if self.config.save_pycode:
+            # write `__init__.py` that imports all to the `pycode` package
+            models_dir = os.path.join(get_dot_andes_path(), 'pycode')
+            os.makedirs(models_dir, exist_ok=True)
+            init_path = os.path.join(models_dir, '__init__.py')
+
+            with open(init_path, 'w') as f:
+                f.write(f"__version__ = '{__version__}'\n\n")
+                for name in self.models.keys():
+                    f.write(f"from . import {name}  # NOQA\n")
+                f.write('\n')
+
+            # RELOAD REQUIRED as the generated Jacobian arguments may be in a different order
+            if pycode is not None:
+                importlib.reload(pycode)
+
         self._store_calls()
         self.dill()
 
+        _, s = elapsed(t0)
+        logger.info(f'Successfully generated numerical code in {s}.')
+
     def _prepare_mp(self, quick=False):
         """
         Code generation with multiprocessing. NOT WORKING NOW.
@@ -228,7 +309,6 @@ class System(object):
         Function is not working. Serialization failed for `conj`.
         """
         from andes.shared import Pool
-        import dill
         dill.settings['recurse'] = True
 
         # consistency check for group parameters and variables
@@ -263,7 +343,9 @@ class System(object):
         self._list2array()     # `list2array` must come before `link_ext_param`
         self.link_ext_param()
         self.find_devices()    # find or add required devices
-        # no device addition or removal after this point
+
+        # == no device addition or removal after this point ==
+
         self.calc_pu_coeff()
 
         # store models with routine flags
@@ -288,7 +370,7 @@ class System(object):
         self.exist.tds = self.find_models('tds')
         self.exist.pflow_tds = self.find_models(('tds', 'pflow'))
 
-    def reset(self):
+    def reset(self, force=False):
         """
         Reset to the state after reading data and setup (before power flow).
 
@@ -296,7 +378,7 @@ class System(object):
         --------
         If TDS is initialized, reset will lead to unpredictable state.
         """
-        if self.TDS.initialized is True:
+        if self.TDS.initialized is True and not force:
             logger.error('Reset failed because TDS is initialized. \nPlease reload the test case to start over.')
             return
         self.dae.reset()
@@ -350,12 +432,12 @@ class System(object):
         # set internal variable addresses
         for mdl in models.values():
             if mdl.flags.address is True:
-                logger.debug(f'{mdl.class_name} address exists')
+                logger.debug(f'{mdl.class_name} internal address exists')
                 continue
             if mdl.n == 0:
                 continue
 
-            logger.debug(f'Setting address for {mdl.class_name}')
+            logger.debug(f'Setting internal address for {mdl.class_name}')
             n = mdl.n
             m0 = self.dae.m
             n0 = self.dae.n
@@ -414,9 +496,8 @@ class System(object):
         """
         def append_model_name(model_name, idx):
             out = ''
-            if isinstance(idx, str):
-                if model_name in idx:
-                    out = idx
+            if isinstance(idx, str) and (model_name in idx):
+                out = idx
             else:
                 out = f'{model_name} {idx}'
 
@@ -464,7 +545,7 @@ class System(object):
             for var in mdl.cache.vars_int.values():
                 var.set_arrays(self.dae)
 
-    def init(self, models: OrderedDict):
+    def init(self, models: OrderedDict, routine: str):
         """
         Initialize the variables for each of the specified models.
 
@@ -474,6 +555,14 @@ class System(object):
         - Call the model `init()` method, which initializes internal variables.
         - Copy variables to DAE and then back to the model.
         """
+        if self.config.numba:
+            use_parallel = True if (self.config.numba_parallel == 1) else False
+            use_cache = True if (pycode is not None) else False
+
+            logger.info(f"Numba compilation initiated, parallel={use_parallel}, cache={use_cache}.")
+            for mdl in models.values():
+                mdl.numba_jitify(parallel=use_parallel, cache=use_cache)
+
         for mdl in models.values():
             # link externals first
             for instance in mdl.services_ext.values():
@@ -486,7 +575,7 @@ class System(object):
                 instance.link_external(ext_model)
 
             # initialize variables second
-            mdl.init()
+            mdl.init(routine=routine)
 
             self.vars_to_dae(mdl)
             self.vars_to_models()
@@ -625,13 +714,12 @@ class System(object):
 
     def l_update_eq(self, models:  OrderedDict):
         """
-        First, update equation-dependent limiter discrete components by calling ``l_check_eq`` of models.
-        Second, force set equations after evaluating equations by calling ``l_set_eq`` of models.
+        Update equation-dependent limiter discrete components by calling ``l_check_eq`` of models.
+        Force set equations after evaluating equations.
 
         This function is must be called after differential equation updates.
         """
         self.call_models('l_check_eq', models)
-        self.call_models('l_set_eq', models)
 
     def s_update_var(self, models: OrderedDict):
         """
@@ -657,16 +745,15 @@ class System(object):
         Additionally, the function resets the differential equations associated with variables pegged by
         anti-windup limiters.
         """
-        self._e_to_dae('f')
-        self._e_to_dae('g')
+        self._e_to_dae(('f', 'g'))
 
         # update variable values set by anti-windup limiters
         for item in self.antiwindups:
             if len(item.x_set) > 0:
-                for key, val in item.x_set:
+                for key, val, _ in item.x_set:
                     np.put(self.dae.x, key, val)
 
-    def f_update(self, models: Optional[Union[str, List, OrderedDict]] = None):
+    def f_update(self, models: OrderedDict):
         """
         Call the differential equation update method for models in sequence.
 
@@ -677,10 +764,10 @@ class System(object):
         try:
             self.call_models('f_update', models)
         except TypeError as e:
-            logger.error("f_update failed. Did you forget to run `andes prepare -q` after updating?")
+            logger.error("f_update failed. Have you run `andes prepare -i` after updating?")
             raise e
 
-    def g_update(self, models: Optional[Union[str, List, OrderedDict]] = None):
+    def g_update(self, models: OrderedDict):
         """
         Call the algebraic equation update method for models in sequence.
 
@@ -691,10 +778,10 @@ class System(object):
         try:
             self.call_models('g_update', models)
         except TypeError as e:
-            logger.error("g_update failed. Did you forget to run `andes prepare -q` after updating?")
+            logger.error("g_update failed. Have you run `andes prepare -i` after updating?")
             raise e
 
-    def j_update(self, models: OrderedDict):
+    def j_update(self, models: OrderedDict, info=None):
         """
         Call the Jacobian update method for models in sequence.
 
@@ -726,6 +813,11 @@ class System(object):
                                      f'j_size={j_size}')
                         raise e
 
+        if info:
+            logger.debug("Jacobian updated at t=%.6f due to %s.", self.dae.t, info)
+        else:
+            logger.debug("Jacobian updated at t=%.6f.", self.dae.t)
+
     def store_sparse_pattern(self, models: OrderedDict):
         """
         Collect and store the sparsity pattern of Jacobian matrices.
@@ -818,19 +910,23 @@ class System(object):
             if var.n > 0:
                 np.put(self.dae.__dict__[v_code], var.a, var.v)
 
-    def _e_to_dae(self, eq_name: str):
+    def _e_to_dae(self, eq_name: Union[str, Tuple] = ('f', 'g')):
         """
         Helper function for collecting equation values into `System.dae.f` and `System.dae.g`.
 
         Parameters
         ----------
-        eq_name : 'x' or 'y'
+        eq_name : 'x' or 'y' or tuple
             Equation type name
         """
-        for var in self._adders[eq_name]:
-            np.add.at(self.dae.__dict__[eq_name], var.a, var.e)
-        for var in self._setters[eq_name]:
-            np.put(self.dae.__dict__[eq_name], var.a, var.e)
+        if isinstance(eq_name, str):
+            eq_name = [eq_name]
+
+        for name in eq_name:
+            for var in self._adders[name]:
+                np.add.at(self.dae.__dict__[name], var.a, var.e)
+            for var in self._setters[name]:
+                np.put(self.dae.__dict__[name], var.a, var.e)
 
     def get_z(self, models: Optional[Union[str, List, OrderedDict]] = None):
         """
@@ -892,9 +988,9 @@ class System(object):
 
     def dill(self):
         """
-        Serialize generated numerical functions in `System.calls` with package `dill`.
+        Serialize generated numerical functions in ``System.calls`` with package ``dill``.
 
-        The serialized file will be stored to ``~/andes/calls.pkl``, where `~` is the home directory path.
+        The serialized file will be stored to ``~/.andes/calls.pkl``, where `~` is the home directory path.
 
         Notes
         -----
@@ -902,7 +998,6 @@ class System(object):
 
         """
         logger.debug("Dumping calls to calls.pkl with dill")
-        import dill
         dill.settings['recurse'] = True
 
         pkl_path = get_pkl_path()
@@ -911,7 +1006,9 @@ class System(object):
 
     @staticmethod
     def _load_pkl():
-        import dill
+        """
+        Helper function to open and load dill-pickled functions.
+        """
         dill.settings['recurse'] = True
         pkl_path = get_pkl_path()
 
@@ -930,7 +1027,7 @@ class System(object):
 
     def undill(self):
         """
-        Deserialize the function calls from ``~/andes.calls.pkl`` with dill.
+        Deserialize the function calls from ``~/.andes/calls.pkl`` with ``dill``.
 
         If no change is made to models, future calls to ``prepare()`` can be replaced with ``undill()`` for
         acceleration.
@@ -955,6 +1052,20 @@ class System(object):
             if name in self.__dict__:
                 self.__dict__[name].calls = model_call
 
+        # try to replace equations and jacobian calls with saved code
+        if pycode is not None and self.config.use_pycode:
+            for model in self.models.values():
+                pycode_model = pycode.__dict__[model.class_name]
+
+                model.calls.f = pycode_model.__dict__.get("f_update")
+                model.calls.g = pycode_model.__dict__.get("g_update")
+
+                for jname in model.calls.j:
+                    model.calls.j[jname] = pycode_model.__dict__.get(f'{jname}_update')
+            logger.info("Using generated Python code for equations and Jacobians.")
+        else:
+            logger.debug("Using undilled lambda functions.")
+
     def _get_models(self, models):
         """
         Helper function for sanitizing the ``models`` input.
@@ -1028,7 +1139,7 @@ class System(object):
             for item in group.common_params:
                 for model in group.models.values():
                     # the below includes all of BaseParam (NumParam, DataParam and ExtParam)
-                    if item not in model.__dict__ or not isinstance(model.__dict__[item], BaseParam):
+                    if item not in model.__dict__:
                         raise KeyError(f'Group <{group.class_name}> common param <{item}> does not exist '
                                        f'in model <{model.class_name}>')
             for item in group.common_vars:
@@ -1041,16 +1152,21 @@ class System(object):
         """
         Collect indices into `BackRef` for all models.
         """
-        for model in self.models.values():
+        models_and_groups = list(self.models.values()) + list(self.groups.values())
+
+        for model in models_and_groups:
             for ref in model.services_ref.values():
                 ref.v = [list() for _ in range(model.n)]
 
-        for model in self.models.values():
+        for model in models_and_groups:
             if model.n == 0:
                 continue
+            if not hasattr(model, "idx_params"):
+                # skip: group does not link to another group
+                continue
 
             for ref in model.idx_params.values():
-                if ref.model not in self.models:
+                if ref.model not in self.models and (ref.model not in self.groups):
                     continue
                 dest_model = self.__dict__[ref.model]
 
@@ -1062,13 +1178,14 @@ class System(object):
                         continue
 
                     for model_idx, dest_idx in zip(model.idx.v, ref.v):
-                        if dest_idx not in dest_model.idx.v:
+                        if dest_idx not in dest_model.uid:
                             continue
                         uid = dest_model.idx2uid(dest_idx)
                         dest_model.services_ref[n].v[uid].append(model_idx)
 
             # set model ``in_use`` flag
-            model.set_in_use()
+            if isinstance(model, Model):
+                model.set_in_use()
 
     def import_groups(self):
         """
@@ -1078,11 +1195,16 @@ class System(object):
         All groups will be stored to dictionary ``System.groups``.
         """
         module = importlib.import_module('andes.models.group')
+
         for m in inspect.getmembers(module, inspect.isclass):
-            name = m[0]
-            cls = m[1]
+
+            name, cls = m
             if name == 'GroupBase':
                 continue
+            elif not issubclass(cls, GroupBase):
+                # skip other imported classes such as `OrderedDict`
+                continue
+
             self.__dict__[name] = cls()
             self.groups[name] = self.__dict__[name]
 
@@ -1102,8 +1224,7 @@ class System(object):
 
         ``system.models['Bus']`` points the same instance as ``system.Bus``.
         """
-        # non-JIT models
-        for file, cls_list in non_jit.items():
+        for file, cls_list in file_classes.items():
             for model_name in cls_list:
                 the_module = importlib.import_module('andes.models.' + file)
                 the_class = getattr(the_module, model_name)
@@ -1115,13 +1236,6 @@ class System(object):
                 group_name = self.__dict__[model_name].group
                 self.__dict__[group_name].add_model(model_name, self.__dict__[model_name])
 
-        # *** JIT import code ***
-        # import JIT models
-        # for file, pair in jits.items():
-        #     for cls, name in pair.items():
-        #         self.__dict__[name] = JIT(self, file, cls, name)
-        # ***********************
-
     def import_routines(self):
         """
         Import routines as defined in ``routines/__init__.py``.
@@ -1143,32 +1257,68 @@ class System(object):
                 self.routines[attr_name] = self.__dict__[attr_name]
                 self.routines[attr_name].config.check()
 
-    def store_switch_times(self, models):
+    def store_switch_times(self, models, eps=1e-4):
         """
-        Store event switching time in a sorted Numpy array at ``System.switch_times``.
+        Store event switching time in a sorted Numpy array in ``System.switch_times``
+        and an OrderedDict ``System.switch_dict``.
+
+        ``System.switch_dict`` has keys as event times and values as the OrderedDict
+        of model names and instances associated with the event.
+
+        Parameters
+        ----------
+        models : OrderedDict
+            model name : model instance
+        eps : float
+            The small time step size to use immediately before
+            and after the event
 
         Returns
         -------
         array-like
             self.switch_times
         """
-        out = []
+        out = np.array([], dtype=np.float)
+
+        if self.options.get('flat') is True:
+            return out
+
+        names = []
         for instance in models.values():
-            out.extend(instance.get_times())
+            times = np.array(instance.get_times()).ravel()
+            out = np.append(out, times)
+            out = np.append(out, times - eps)
+            out = np.append(out, times + eps)
+            names.extend([instance.class_name] * (3 * len(times)))
 
-        out = np.ravel(np.array(out))
-        out = np.append(out, out + 1e-4)
-        out = np.unique(out)
-        out = out[np.where(out >= 0)]
-        out = np.sort(out)
+        # sort
+        sort_idx = np.argsort(out).astype(int)
+        out = out[sort_idx]
+        names = [names[i] for i in sort_idx]
 
-        self.switch_times = out
+        # select t > current time
+        ltzero_idx = np.where(out >= self.dae.t)[0]
+        out = out[ltzero_idx]
+        names = [names[i] for i in ltzero_idx]
+
+        # make into an OrderedDict with unique keys and model names combined
+        for i, j in zip(out, names):
+            if i not in self.switch_dict:
+                self.switch_dict[i] = {j: self.models[j]}
+            else:
+                self.switch_dict[i].update({j: self.models[j]})
+
+        self.switch_times = np.array(list(self.switch_dict.keys()))
+
+        # self.switch_times = out
         self.n_switches = len(self.switch_times)
         return self.switch_times
 
     def switch_action(self, models):
         """
         Invoke the actions associated with switch times.
+
+        Switch actions will be disabled if `flat=True` is passed to system.
         """
         for instance in models.values():
             instance.switch_action(self.dae.t)
@@ -1209,6 +1359,10 @@ class System(object):
             self.calls[name] = mdl.calls
 
     def _list2array(self):
+        """
+        Helper function to call models' ``list2array`` method, which
+        usually performs memory preallocation.
+        """
         self.call_models('list2array', self.models)
 
     def set_config(self, config=None):
@@ -1325,3 +1479,23 @@ class System(object):
                   )
 
         return tab.draw()
+
+    def as_dict(self, vin=False, skip_empty=True):
+        """
+        Return system data as a dict where the keys are model names
+        and values are dicts. Each dict has parameter names as keys
+        and corresponding data in an array as values.
+
+        Returns
+        -------
+        OrderedDict
+
+        """
+        out = OrderedDict()
+
+        for name, instance in self.models.items():
+            if skip_empty and instance.n == 0:
+                continue
+            out[name] = instance.as_dict(vin=vin)
+
+        return out
