@@ -10,7 +10,7 @@ import logging
 logger = logging.getLogger(__name__)
 
 
-class DisplayablePath(object):
+class DisplayablePath:
     display_filename_prefix_middle = '├──'
     display_filename_prefix_last = '└──'
     display_parent_prefix_middle = '    '
@@ -93,18 +93,29 @@ def tests_root():
 
 
 def get_case(rpath):
-    """Return the path to the stock cases"""
+    """
+    Return the path to a stock case for a given path relative to ``andes/cases``.
+
+    To list all cases, use ``andes.list_cases()``.
+
+    Examples
+    --------
+    To get the path to the case `kundur_full.xlsx` under folder `kundur`, do ::
+
+        andes.get_case('kundur/kundur_full.xlsx')
+
+    """
     case_path = os.path.join(cases_root(), rpath)
     case_path = os.path.normpath(case_path)
 
     if not os.path.isfile(case_path):
-        raise FileNotFoundError(f'{rpath} does not exist.')
+        raise FileNotFoundError(f'"{rpath}" is not a valid relative path to a stock case.')
     return case_path
 
 
 def list_cases(rpath='.', no_print=False):
     """
-    List stock cases under a given folder relative to ``cases``
+    List stock cases under a given folder relative to ``andes/cases``
     """
     case_path = os.path.join(cases_root(), rpath)
     case_path = os.path.normpath(case_path)
