@@ -6,7 +6,7 @@ from andes.shared import pd, np, spmatrix, jac_names
 logger = logging.getLogger(__name__)
 
 
-class DAETimeSeries(object):
+class DAETimeSeries:
     """
     DAE time series data
     """
@@ -27,12 +27,11 @@ class DAETimeSeries(object):
 
     @property
     def x(self):
-        return self.xy[:, 0:self.dae.n+1]
+        return self.xy[:, 0:self.dae.n]
 
     @property
     def y(self):
-        indices = [0] + list(range(self.dae.n + 1, self.dae.n + self.dae.m + 1))
-        return self.xy[:, indices]
+        return self.xy[:, self.dae.n:self.dae.n + self.dae.m]
 
     @property
     def txyz(self):
@@ -69,7 +68,6 @@ class DAETimeSeries(object):
             self.xy = np.zeros((n_steps, self.dae.m + self.dae.n))
             self.z = np.zeros((n_steps, self.dae.o))
 
-            # TODO: write test.
             for idx, xy in enumerate(self._xy.values()):
                 self.xy[idx, :] = xy
 
@@ -94,7 +92,7 @@ class DAETimeSeries(object):
             self._z[t] = z
 
 
-class DAE(object):
+class DAE:
     r"""
     Class for storing numerical values of the DAE system, including variables, equations and first order
     derivatives (Jacobian matrices).
@@ -167,6 +165,8 @@ class DAE(object):
 
         self.triplets = JacTriplet()
 
+        self.tpl = dict()  # sparsity templates with constants
+
     def clear_ts(self):
         self.ts = DAETimeSeries(self)
 
@@ -203,18 +203,33 @@ class DAE(object):
         """
         self.triplets.clear_ijv()
 
-    def restore_sparse(self):
+    def restore_sparse(self, names=None):
         """
-        Restore all sparse arrays to shape with non-zero constants
+        Restore all sparse matrices to the sparsity pattern
+        filled with zeros (for variable Jacobian elements)
+        and non-zero constants.
+
+        Parameters
+        ----------
+        names : None or list
+            List of Jacobian names to restore sparsity pattern
         """
-        for jname in jac_names:
-            self.build_pattern(jname)
+        if names is None:
+            names = jac_names
+        elif isinstance(names, str):
+            names = [names]
+
+        for name in names:
+            self.__dict__[name] = spmatrix(self.tpl[name].V,
+                                           self.tpl[name].I,
+                                           self.tpl[name].J,
+                                           self.tpl[name].size, 'd')
 
     def reset(self):
         """
         Reset array sizes to zero and clear all arrays.
         """
-        self.t = 0
+        self.set_t(0.0)
         self.m = 0
         self.n = 0
         self.o = 0
@@ -222,6 +237,10 @@ class DAE(object):
         self.clear_ijv()
         self.clear_ts()
 
+    def set_t(self, t):
+        """Helper function for setting time in-place"""
+        self.t.itemset(t)
+
     def get_size(self, name):
         """
         Get the size of an array or sparse matrix based on name.
@@ -280,14 +299,16 @@ class DAE(object):
             jac name
         """
         try:
-            self.__dict__[name] = spmatrix(self.triplets.vjac[name],
-                                           self.triplets.ijac[name],
-                                           self.triplets.jjac[name],
-                                           self.get_size(name), 'd')
+            self.tpl[name] = spmatrix(self.triplets.vjac[name],
+                                      self.triplets.ijac[name],
+                                      self.triplets.jjac[name],
+                                      self.get_size(name), 'd')
         except TypeError as e:
             logger.error("Your new model might have accessed an Algeb using ExtState, or vice versa.")
             raise e
 
+        self.restore_sparse(name)
+
     def _compare_pattern(self, name):
         """
         Compare the sparsity pattern for the given Jacobian name.
@@ -314,6 +335,12 @@ class DAE(object):
 
         If ``m > len(self.y)`` or ``n > len(self.x``, arrays will be extended.
         Otherwise, new empty arrays will be sliced, starting from 0 to the given size.
+
+        Warnings
+        --------
+        This function should not be called directly. Instead, it is called in
+        ``System.set_address`` which re-points variables used in power flow
+        to the new array for dynamic analyses.
         """
         self.x = self._extend_or_slice(self.x, self.n)
         self.y = self._extend_or_slice(self.y, self.m)
@@ -391,8 +418,17 @@ class DAE(object):
 
     def write_lst(self, lst_path):
         """
-        Dump the variable name lst file
-        :return: succeed flag
+        Dump the variable name lst file.
+
+        Parameters
+        ----------
+        lst_path
+            Path to the lst file.
+
+        Returns
+        -------
+        bool
+            succeed flag
         """
 
         out = ''
@@ -416,16 +452,16 @@ class DAE(object):
             f.write(out)
         return True
 
-    def write_npy(self, npy_path):
+    def write_npy(self, file_path):
         """
-        Write TDS data into NumPy output files.
-
-        TODO: Compress when the matrix is larger than 25 MB.
+        Write TDS data into NumPy uncompressed format.
         """
         txyz_data = self.ts.txyz
+        np.save(file_path, txyz_data)
 
-        # if txyz_data.size > 3125000:  # 25 megabytes of flaot64
-        #     np.savez_compressed(npy_path, txyz_data)
-        # else:
-
-        np.save(npy_path, txyz_data)
+    def write_npz(self, file_path):
+        """
+        Write TDS data into NumPy compressed format.
+        """
+        txyz_data = self.ts.txyz
+        np.savez_compressed(file_path, data=txyz_data)
