@@ -1,7 +1,6 @@
 import os
-import unittest
-
 import dill
+import unittest
 import numpy as np
 
 import andes
@@ -38,7 +37,8 @@ class TestKnownResults(unittest.TestCase):
 
 
 def compare_results(case, pkl_name, addfile=None, tf=10):
-    ss = andes.load(case, addfile=addfile)
+    ss = andes.load(case, addfile=addfile, default_config=True)
+
     ss.config.warn_limits = 0
     ss.config.warn_abnormal = 0
     ss.PFlow.run()
@@ -48,6 +48,7 @@ def compare_results(case, pkl_name, addfile=None, tf=10):
     ss.TDS.config.tol = 1e-6
     ss.TDS.config.fixt = 1
     ss.TDS.config.shrinkt = 0
+    ss.TDS.config.honest = 0
     ss.TDS.run()
 
     test_dir = os.path.dirname(__file__)
