@@ -13,8 +13,8 @@ class TestMATPOWER(unittest.TestCase):
 
     def test_pflow_mpc_pool(self):
         case_path = [get_case(os.path.join('matpower', item)) for item in self.cases]
-        andes.run(case_path, no_output=True, ncpu=2, pool=True, verbose=40)
+        andes.run(case_path, no_output=True, ncpu=2, pool=True, verbose=40, default_config=True)
 
     def test_pflow_mpc_process(self):
         case_path = [get_case(os.path.join('matpower', item)) for item in self.cases]
-        andes.run(case_path, no_output=True, ncpu=2, pool=False, verbose=40)
+        andes.run(case_path, no_output=True, ncpu=2, pool=False, verbose=40, default_config=True)
